#include <stdio.h>
#include "sip_solver.h"

extern SSO Sip_Opt;

extern void identity(double *h, int nx);



/******************************************************
Set the Hessian BFGS approximation
b      - Hessian
k      - Iteration number (for reset)
nx     - dimension
x1     - x in iteration
x0     - x in previous iteration
g1     - gradient in iteration
g0     - gradient in previous iteration
******************************************************/
void set_b(double *b, int k, int nx, double *x1, double *x0,
	   double *g1, double *g0)
{
  int i, j, l;
  double y[nx], s[nx], den;
  double auxBs[nx], teta, auxsBs, nors, nory;

  if((!(k%nx) && Sip_Opt.reset_h) || !k){

    identity(b, nx);
    printf("Identity Hessian\n");
    
  } else {

      for(i=0;i<nx;i++){
	y[i]=g1[i]-g0[i];
	s[i]=x1[i]-x0[i];
      }

    if(Sip_Opt.dampedBFGS){
      /* Damped BFGS formula */
      printf("Damped BFGS formula\n");
      den=0.0;
      for(l=0;l<nx;l++)
	den+=s[l]*y[l];

      for(i=0;i<nx;i++){
	auxBs[i]=0.0;
	for(j=0;j<nx;j++)
	  auxBs[i]+=s[j]*b[i*nx+j];
      }

      auxsBs=0.0;
      for(i=0;i<nx;i++)
	auxsBs+=auxBs[i]*s[i];

      if(den>=0.2*auxsBs){
	teta=1; /* r=s */
      } else {
	teta=(0.8*auxsBs)/(auxsBs-den);
      }

      printf("Teta=%f\n", teta);
      for(i=0;i<nx;i++)
	y[i]=teta*y[i]+(1-teta)*auxBs[i];

    }

    den=0.0;
    nors=0.0;
    nory=0.0;
    for(l=0;l<nx;l++){
      den+=s[l]*y[l];
      nors+=s[l]*s[l];
      nory+=y[l]*y[l];
    }

    if(fabs(den)<=Sip_Opt.reseth){
      if(Sip_Opt.scaleBFGS){
	teta=(sqrt(Sip_Opt.reseth))/(sqrt(sqrt(nors)*sqrt(nory)));
	for(l=0;l<nx;l++){
	  s[l]*=teta;
	  y[l]*=teta;
	}
	den=0.0;
	for(l=0;l<nx;l++)
	  den+=s[l]*y[l];

	printf("Hessian scaled by %f\n", teta);
      } else {
	printf("Not changing Hessian den=%.20f\n", den);
	return;
      }
    }


    for(i=0;i<nx;i++){
      b[i*nx+i]-=(auxBs[i]*auxBs[i])/auxsBs;
      for(j=i+1;j<nx;j++)
	b[i*nx+j]-=(auxBs[i]*auxBs[j])/auxsBs;
    }

    for(i=0;i<nx;i++){
      b[i*nx+i]+=(y[i]*y[i])/den;
      for(j=i+1;j<nx;j++)
	b[i*nx+j]+=(y[i]*y[j])/den;
    }

  }

}
