#include "sip_solver.h"


extern void printarray(double *a, int n);
extern void printmatrix(double *a, int n1, int n2);
extern double halton(int i, int k);
extern STATS stats;


extern SSO Sip_Opt;
Mesh Sip_Mesh;
Mesh R_Set;
Mesh T_Set;
Mesh S_Set;
extern SIP_S sip_s;

int start;

void discretization()
{
  int k, i; /* Outer Iteration Number */
  double *h; /* Space between points */
  double *min, *max, fx;
  int inner;


  if(Sip_Opt.disc_k<=0){
    printf("Number of mesh refines less or equal to zero\n");
    return;
  }

  if(Sip_Opt.disc_h && Sip_Opt.ntvars!=ntsip){
      printf("User specified %d values to h but there are %d t vars\n",
	     Sip_Opt.ntvars, ntsip);
      if(Sip_Opt.ntvars>ntsip){
	printf("Using the first %d values\n", ntsip);
	h=mymalloc(ntsip*sizeof(double));
	memcpy(h, Sip_Opt.disc_h, ntsip*sizeof(double));
	free(Sip_Opt.disc_h);
	Sip_Opt.ntvars=ntsip;
	Sip_Opt.disc_h=h;
	h=NULL;
      } else {
	printf("Using first %d values and repeat the last value\n",
	       Sip_Opt.ntvars);
	h=mymalloc(ntsip*sizeof(double));
	memcpy(h, Sip_Opt.disc_h, Sip_Opt.ntvars*sizeof(double));
	for(i=0;i<(ntsip-Sip_Opt.ntvars);i++)
	  h[i+Sip_Opt.ntvars]=Sip_Opt.disc_h[Sip_Opt.ntvars-1];
	free(Sip_Opt.disc_h);
	Sip_Opt.ntvars=ntsip;
	Sip_Opt.disc_h=h;
	h=NULL;
      }	
  }

  if(!Sip_Opt.disc_h){
    Sip_Opt.disc_h=mymalloc(ntsip*sizeof(double));
    for(i=0;i<ntsip;i++){
      Sip_Opt.disc_h[i]=0.01;
      /*    Sip_Opt.disc_h[i]=(TBU[i]-TBL[i])/10;*/
      /*      pot=pow(100000,(double)(1.0/ntsip));
	      Sip_Opt.disc_h[i]=maximum(((TBU[i]-TBL[i])/pot)*2*pow(3,Sip_Opt.disc_k-2), 0.2); */
    }
    /*    Sip_Opt.disc_k=rint(2+(log(pow(100000.0,1.0/ntsip)/20.0)/log(3.0)));*/
  }


  /* h=disc_h */
  h=mymalloc(ntsip*sizeof(double));
  memcpy(h,Sip_Opt.disc_h, ntsip*sizeof(double));

  printf("h=(");
  for(i=0;i<ntsip-1;i++)
    printf("%f,", h[i]);
  printf("%f)\n",h[i]);

  printf("Refinements=%d\n",Sip_Opt.disc_k);

  min=mymalloc(ntsip*sizeof(double));
  max=mymalloc(ntsip*sizeof(double));

  for(i=0;i<ntsip;i++){
    if(TBL[i]<0 && TBU[i]>0){
      min[i]=0; max[i]=maximum(abs(TBL[i]), abs(TBU[i]));
    } else {
      if(TBL[i]>=0){
	min[i]=TBL[i]; max[i]=TBU[i];
      } else {
	min[i]=-TBU[i]; max[i]=-TBL[i];
      }
    }
  }

  /* Min norm will be the point with lower coordinates*/
  Sip_Mesh.Min_Norm=norm2(ntsip, min);

  /* Max norm will be the point with upper coordinates*/
  Sip_Mesh.Max_Norm=norm2(ntsip, max);

  free(min);
  free(max);

  /* for the moment I will have HASH_SIZE hash pointers */
  Sip_Mesh.nHash=HASH_SIZE;
  Sip_Mesh.AmpHash=(Sip_Mesh.Max_Norm-Sip_Mesh.Min_Norm)/HASH_SIZE;

  /* allocate pointers to point struct */
  Sip_Mesh.Mesh=mymalloc(Sip_Mesh.nHash*sizeof(Points *));
  memset(Sip_Mesh.Mesh, 0, Sip_Mesh.nHash*sizeof(Points *));

  Sip_Mesh.nPoints=0;

  fill_mesh(h); /* T[h] */

  start=Sip_Mesh.nPoints;

  
  R_Set.Min_Norm=Sip_Mesh.Min_Norm;
  R_Set.Max_Norm=Sip_Mesh.Max_Norm;
  R_Set.nHash=HASH_SIZE;
  R_Set.AmpHash=Sip_Mesh.AmpHash;
  R_Set.Mesh=mymalloc(R_Set.nHash*sizeof(Points *));
  memset(R_Set.Mesh, 0, R_Set.nHash*sizeof(Points *));
  R_Set.nPoints=0;

  T_Set.Min_Norm=Sip_Mesh.Min_Norm;
  T_Set.Max_Norm=Sip_Mesh.Max_Norm;
  T_Set.nHash=HASH_SIZE;
  T_Set.AmpHash=Sip_Mesh.AmpHash;
  T_Set.Mesh=mymalloc(T_Set.nHash*sizeof(Points *));
  memset(T_Set.Mesh, 0, T_Set.nHash*sizeof(Points *));
  T_Set.nPoints=0;

  S_Set.Min_Norm=Sip_Mesh.Min_Norm;
  S_Set.Max_Norm=Sip_Mesh.Max_Norm;
  S_Set.nHash=HASH_SIZE;
  S_Set.AmpHash=Sip_Mesh.AmpHash;
  S_Set.Mesh=mymalloc(S_Set.nHash*sizeof(Points *));
  memset(S_Set.Mesh, 0, S_Set.nHash*sizeof(Points *));
  S_Set.nPoints=0;

  printf("Iter\tGrid\tNnlsp\tNacvi\tFullGrid\n");
  stats.nnlp=0;

  fill_T(); /* build init T~ set */
  /*  printf("Sip_Mesh\n");
      print_mesh(&Sip_Mesh);*/

  /*fill_S();
  copy_mesh(&S_Set, &T_Set);
  check_T();*/

  fx=solve_nlp(); /* find first solution */

  stats.nnlpc=0; /* first grid does not count */

  /* check_nlp(); */
  fill_R(); /* construct R set */

  printf("%d\t%d\t%d\t%d\t%s\n", 0, Sip_Mesh.nPoints,
	 (T_Set.nPoints)*ntsipc+nxsipc, R_Set.nPoints, "--");

  inner=0;
  for (k=1;k<Sip_Opt.disc_k;k++){

    /*    printf("h=");
    printarray(h, ntsip);
    printf("Sip_Mesh\n");
    print_mesh(&Sip_Mesh);
    printf("T_Set\n");
    print_mesh(&T_Set);
    printf("R_Set\n");
    print_mesh(&R_Set);
    printf("S_Set\n");
    print_mesh(&S_Set);*/

    /* actual X solves full mesh */
    if(!fill_S() && inner<Sip_Opt.disc_inner){
      /*printf("S_Set\n");
	print_mesh(&S_Set);*/
      /*      printf("X not solved full mesh\n");*/
      /* step ii */
      augment_T(); /* add point that are active or violated */
      /*      printf("augmented T_Set\n");
	      print_mesh(&T_Set);*/
      /* solve problem again, starting point will be the most recent */
      check_T();
      fx=solve_nlp();
      /*      check_nlp();*/
      augment_R(); /* augment R by at least one */
      k--; /* same iteration */
      printf("%d\t%d\t%d\t%d\t%s\n", k, Sip_Mesh.nPoints,
	     (T_Set.nPoints)*ntsipc+nxsipc,
	     R_Set.nPoints, "No");
      inner++;
    } else {
      /*printf("S_Set\n");
	print_mesh(&S_Set);*/
      /*      printf("X solved full mesh\n");*/
      next_T(h, k);
      check_T();
      fx=solve_nlp();
      /*      check_nlp();*/
      augment_R_RX();
      printf("%d\t%d\t%d\t%d\t%s\n", k, Sip_Mesh.nPoints,
	     (T_Set.nPoints)*ntsipc+nxsipc, R_Set.nPoints, "Yes");
      inner=0;
    }
  }
  write_result(fx);
}


void check_T()
{
  int i, j;
  Points *tmp;
  Points *maxi;
  double maxival;

  if(T_Set.nPoints>0)return;
  /* T set is empty, add a point */
  printf("Check_T in action\n");

  maxi=NULL;

  for(i=0;i<HASH_SIZE;i++)
    for(tmp=Sip_Mesh.Mesh[i];tmp;tmp=tmp->next){
      if(!maxi){
	maxi=tmp;
	maxival=tmp->gval[0];
      }
      for(j=0;j<ntsipc;j++){
	if(tmp->gval[j]>maxival){
	  maxi=tmp;
	  maxival=tmp->gval[j];
	}
      }
    }
  
  insertpt(maxi->coord, maxi->norm, maxi->gval, &T_Set);
}



void fill_mesh(double *h)
{
  double t[ntsip];

  free_mesh(&Sip_Mesh);
  gennext(0, t, h);
  
}

void gennext(int n, double *t, double *h)
{
  int i;
  double norm;

  if(n>=ntsip){
    norm=norm2(ntsip, t);
    insertpt(t, norm, 0, &Sip_Mesh);
    return;
      };

  /* set coordinates n and set the next */
  for(i=0,t[n]=TBL[n];TBL[n]+i*h[n]<=TBU[n];i++){
    t[n]=TBL[n]+i*h[n];
    gennext(n+1, t, h);
  }
}


void insertpt(double *t, double norm, double *gval, Mesh *mset)
{
  int cell;
  Points *thispt, *tmp;

  if(norm<mset->Min_Norm || norm>mset->Max_Norm){
    fprintf(stderr,"Norm of point out of range\n");
    fprintf(stderr,"Min=%f Max=%f val=%f\n",mset->Min_Norm,mset->Max_Norm,norm);
    return;
  }

  cell=(int)((norm-mset->Min_Norm)/mset->AmpHash);

  if(cell>=HASH_SIZE)
    cell=HASH_SIZE-1;
  if(cell<0)
    cell=0;

  thispt=mset->Mesh[cell];

  /*  for(tmp=thispt,last=thispt;tmp;last=tmp,tmp=tmp->next);*/

  tmp=mymalloc(sizeof(struct Points));
  tmp->coord=mymalloc(ntsip*sizeof(double));
  memcpy(tmp->coord, t, ntsip*sizeof(double));
  tmp->norm=norm;
  tmp->gval=mymalloc(ntsipc*sizeof(double));
  if(gval){
    memcpy(tmp->gval, gval, ntsipc*sizeof(double));
  } else {
    memset(tmp->gval, 0, ntsipc*sizeof(double));
  }
  tmp->next=mset->Mesh[cell];
  mset->Mesh[cell]=tmp;

  mset->nPoints++;
}

/*****************************************************
Check and insert point if not in list
return 0 if inserted
return 1 if exists
*****************************************************/
int check_insertpt(double *t, double norm, double *gval, Mesh *mset)
{
  int cell;
  Points *thispt, *tmp, *last;

  if(norm<mset->Min_Norm || norm>mset->Max_Norm){
    fprintf(stderr,"Norm of point out of range\n");
    fprintf(stderr,"Min=%f Max=%f val=%f\n",mset->Min_Norm,mset->Max_Norm,norm);
    return 1; /* not inserted */
  }

  cell=(int)((norm-mset->Min_Norm)/mset->AmpHash);

  if(cell>=HASH_SIZE)
    cell=HASH_SIZE-1;
  if(cell<0)
    cell=0;

  thispt=mset->Mesh[cell];

  tmp=thispt;
  while(tmp && !equal(tmp->coord, t, ntsip)){
    last=tmp;
    tmp=tmp->next;
  }

  if(tmp)
    return 1;

  tmp=mymalloc(sizeof(struct Points));
  tmp->coord=mymalloc(ntsip*sizeof(double));
  memcpy(tmp->coord, t, ntsip*sizeof(double));
  tmp->norm=norm;
  tmp->gval=mymalloc(ntsip*sizeof(double));
  memcpy(tmp->gval, gval, ntsip*sizeof(double));
  tmp->next=NULL;
  if(thispt==NULL){ /* insert first */
    mset->Mesh[cell]=tmp;
  } else {
    last->next=tmp;
  }
  mset->nPoints++;
  return 0;
}
  

double norm2(int n, double *x)
{
  int i;
  double norm;

  for(i=0, norm=0;i<n;norm+=x[i]*x[i],i++);

  return sqrt(norm);
}



void print_mesh(Mesh *mset)
{
  int i, j;
  Points *tmp;

  printf("\nThere are %d points in the mesh\n", mset->nPoints);

  for(i=0;i<HASH_SIZE;i++){
    tmp=mset->Mesh[i];
    /*    printf("Hash %d\n", i); */
    for(;tmp;tmp=tmp->next){
      printf("(");
      for(j=0;j<ntsip;j++)
	printf("%f,", tmp->coord[j]);
      printf("):norm %f\t", tmp->norm);
      printf(": g=");
      printarray(tmp->gval,ntsipc);
      printf("Real constraints value ");
      for(j=0;j<ntsipc;j++)
	printf(" %f", eval_sip(SIP_CONTVAL, j, X, tmp->coord, NULL, NULL, NULL, NULL));
      printf("\n");
    }
  }
}


/************************************************
fill first R set. If X is a solution of T[h] then
R is the t values for the actives constraints
************************************************/
void fill_R()
{
  int i, j;
  Points *tmp;
  double cval;


  for(i=0;i<HASH_SIZE;i++){
    for(tmp=Sip_Mesh.Mesh[i];tmp;tmp=tmp->next){
      /* for all t constraints */
      for(j=0;j<ntsipc;j++)
	tmp->gval[j]=eval_sip(SIP_CONTVAL, j, X, tmp->coord, NULL, NULL, NULL, NULL);
      for(j=0;j<ntsipc;j++){
	cval=tmp->gval[j];
	if(equal(&cval,TCBL+j,1) || equal(&cval,TCBU+j,1))
	  /* active constraint */
	  check_insertpt(tmp->coord, tmp->norm, tmp->gval, &R_Set);
      }
    }
  }
}

/************************************************
fill S set. If X is a solution of T[h] then
S is the t values for the actives and violated constraints
************************************************/
int fill_S()
{
  int i, j;
  Points *tmp;
  double cval;
  int rval; /* return 1 if X solves full mesh 0 otherwise */

  rval=1;

  free_mesh(&S_Set);
  
  for(i=0;i<HASH_SIZE;i++){
    for(tmp=Sip_Mesh.Mesh[i];tmp;tmp=tmp->next){
      /* for all t constraints */
      for(j=0;j<ntsipc;j++)
	tmp->gval[j]=eval_sip(SIP_CONTVAL, j, X, tmp->coord, NULL, NULL, NULL, NULL);
      for(j=0;j<ntsipc;j++){
	cval=tmp->gval[j];
	if(equal(&cval, TCBL+j, 1) || equal(&cval, TCBU+j, 1) ||
	   cval+Sip_Opt.zero<=TCBL[j] || cval-Sip_Opt.zero>=TCBU[j]){
	  /* active or violated constraint */
	  check_insertpt(tmp->coord, tmp->norm, tmp->gval, &S_Set);
	  if(!equal(&cval,TCBL+j,1) && !equal(&cval,TCBU+j,1))
	    /* violated constraint */
	    rval=0;
	}
      }
    }
  }
  return rval;
}


/************************************************
fill first T set. First T set is all T[h]
************************************************/
void fill_T()
{
  int i;
  Points *tmp;
  
  for(i=0;i<HASH_SIZE;i++){
    for(tmp=Sip_Mesh.Mesh[i];tmp;tmp=tmp->next)
      insertpt(tmp->coord, tmp->norm, tmp->gval, &T_Set);
  }
}




/***************************************************
Augment the T set with the points that are active or
violate the infinite constraints
***************************************************/			    
void augment_T()
{
  free_mesh(&T_Set);

  /* T=R+S */
  copy_mesh(&R_Set, &T_Set);
  add_mesh(&S_Set, &T_Set);

}  


/***************************************************************
If R(X) is not in R then R=R+R(X); else augment R by one T point
***************************************************************/
void augment_R()
{
  Points *tmp;
  int i, j, byone;
  double cval;


  byone=1; /* increment R by one ?*/

  for(i=0;i<HASH_SIZE;i++){
    for(tmp=Sip_Mesh.Mesh[i];tmp;tmp=tmp->next){
      /* for all t constraints */
      for(j=0;j<ntsipc;j++)
	tmp->gval[j]=eval_sip(SIP_CONTVAL, j, X, tmp->coord, NULL, NULL, NULL, NULL);
      for(j=0;j<ntsipc;j++){
	cval=tmp->gval[j];
	if(equal(&cval,TCBL+j, 1) || equal(&cval,TCBU+j,1)){
	  /* active constraint */
	  if(!check_insertpt(tmp->coord, tmp->norm, tmp->gval, &R_Set))
	    byone=0;
	}
      }
    }
  }

  if(!byone)return;
  
  /* add a arbitrary point to R from T_Set */
  for(i=0;i<HASH_SIZE;i++)
    for(tmp=T_Set.Mesh[i];tmp && check_insertpt(tmp->coord, tmp->norm, tmp->gval, &R_Set);tmp=tmp->next);

}



/***************************************************
R=R+R(X)
***************************************************/
void augment_R_RX()
{
  int i, j;
  Points *tmp;
  double cval;

  for(i=0;i<HASH_SIZE;i++){
    for(tmp=Sip_Mesh.Mesh[i];tmp;tmp=tmp->next){
      /* for all t constraints */
      for(j=0;j<ntsipc;j++)
	tmp->gval[j]=eval_sip(SIP_CONTVAL, j, X, tmp->coord, NULL, NULL, NULL, NULL);
      for(j=0;j<ntsipc;j++){
	cval=tmp->gval[j];
	if(equal(&cval,TCBL+j,1) || equal(&cval,TCBU+j,1))
	  /* active constraint */
	  check_insertpt(tmp->coord, tmp->norm, tmp->gval, &R_Set);
      }
    }
  }
}


/**************************************************
Refine mesh and update T
**************************************************/
void next_T(double *h, int k)
{
  int i, j;
  Points *tmp;

  if(k==1){ /* h1=h0/2 */
    for(i=0;i<ntsip;h[i]/=2,i++);
  } else {
    for(i=0;i<ntsip;h[i]/=3,i++);
  }

  fill_mesh(h);

  free_mesh(&T_Set);
  copy_mesh(&R_Set, &T_Set);
  add_mesh(&S_Set, &T_Set);

  /* encontrar vizinhos de S para restr. activas ou violadas */

  for(i=0;i<HASH_SIZE;i++){
    for(tmp=T_Set.Mesh[i];tmp;tmp=tmp->next){
      /* for all t constraints */
      for(j=0;j<ntsipc;j++){
	insert_neig(tmp->coord, tmp->norm, &T_Set);
      }
    }
  }

}



/************************************************
Allocate memory and abort on error
************************************************/
/*  void *mymalloc(int n) */
/*  { */
/*    void *pt; */

/*    if(n==0)return NULL; */

/*    pt=malloc(n); */
/*    if(!pt){ */
/*      printf("Unable to allocate %d bytes of memory\n", n); */
/*      printf("Aborting...\n"); */
/*      exit(1); */
/*    } */
/*    return pt; */
/*  } */


int equal(double *a, double *b, int n)
{
  int i;

  for(i=0;i<n && a[i]>b[i]-Sip_Opt.zero && a[i]<b[i]+Sip_Opt.zero;i++);

  return (i==n ? 1 : 0);
}

/************************************************
copy mesh a to b
************************************************/
void copy_mesh(Mesh *a, Mesh *b)
{
  int i;
  Points *tmp;
  
  free_mesh(b);
  
  b->Min_Norm=a->Min_Norm;
  b->Max_Norm=a->Max_Norm;
  b->nHash=HASH_SIZE;
  b->AmpHash=a->AmpHash;
  b->Mesh=malloc(b->nHash*sizeof(Points *));
  memset(b->Mesh, 0, b->nHash*sizeof(Points *));

  for(i=0;i<HASH_SIZE;i++){
    for(tmp=a->Mesh[i];tmp;tmp=tmp->next)
      insertpt(tmp->coord, tmp->norm, tmp->gval, b);
  }
}

/************************************************
add mesh a to b
************************************************/
void add_mesh(Mesh *a, Mesh *b)
{
  int i;
  Points *tmp;
  
  for(i=0;i<HASH_SIZE;i++){
    for(tmp=a->Mesh[i];tmp;tmp=tmp->next)
      check_insertpt(tmp->coord, tmp->norm, tmp->gval, b);
  }
}

/***************************************************
free up a mesh from all points
***************************************************/
void free_mesh(Mesh *a)
{
  int i;
  Points *tmp, *last;

  if(a->Mesh){
    for(i=0;i<HASH_SIZE;i++){
      tmp=a->Mesh[i];
      while(tmp){
	last=tmp;
	tmp=tmp->next;
	free(last->coord);
	free(last->gval);
	free(last);
      }
    }
    memset(a->Mesh, 0, a->nHash*sizeof(Points *));
  }
  a->nPoints=0;
}



/*********************************************************
Discretization method with halton points
*********************************************************/
void disc_halton()
{
  int k, i; /* Outer Iteration Number */
  int inner; /* inner iteration number */
  double *min, *max;
  int ht;


  if(Sip_Opt.disc_ha_mx<=0 || Sip_Opt.disc_ha_mx<Sip_Opt.disc_h_mx ||
     Sip_Opt.disc_h_mx<=0){
    Sip_Opt.disc_ha_mx=100*ntsip;
    Sip_Opt.disc_h_mx=1000*ntsip;
  }


  min=mymalloc(ntsip*sizeof(double));
  max=mymalloc(ntsip*sizeof(double));

  for(i=0;i<ntsip;i++){
    if(TBL[i]<0 && TBU[i]>0){
      min[i]=0; max[i]=maximum(abs(TBL[i]), abs(TBU[i]));
    } else {
      if(TBL[i]>=0){
	min[i]=TBL[i]; max[i]=TBU[i];
      } else {
	min[i]=-TBU[i]; max[i]=-TBL[i];
      }
    }
  }

  /* Min norm will be the point with lower coordinates*/
  Sip_Mesh.Min_Norm=norm2(ntsip, min);

  /* Max norm will be the point with upper coordinates*/
  Sip_Mesh.Max_Norm=norm2(ntsip, max);

  free(min);
  free(max);

  /* for the moment I will have HASH_SIZE hash pointers */
  Sip_Mesh.nHash=HASH_SIZE;
  Sip_Mesh.AmpHash=(Sip_Mesh.Max_Norm-Sip_Mesh.Min_Norm)/HASH_SIZE;

  /* allocate pointers to point struct */
  Sip_Mesh.Mesh=mymalloc(Sip_Mesh.nHash*sizeof(Points *));
  memset(Sip_Mesh.Mesh, 0, Sip_Mesh.nHash*sizeof(Points *));

  Sip_Mesh.nPoints=0;

  ht=0;
  fill_halton_mesh(&ht); /* T[h] */

  
  R_Set.Min_Norm=Sip_Mesh.Min_Norm;
  R_Set.Max_Norm=Sip_Mesh.Max_Norm;
  R_Set.nHash=HASH_SIZE;
  R_Set.AmpHash=Sip_Mesh.AmpHash;
  R_Set.Mesh=mymalloc(R_Set.nHash*sizeof(Points *));
  memset(R_Set.Mesh, 0, R_Set.nHash*sizeof(Points *));
  R_Set.nPoints=0;

  T_Set.Min_Norm=Sip_Mesh.Min_Norm;
  T_Set.Max_Norm=Sip_Mesh.Max_Norm;
  T_Set.nHash=HASH_SIZE;
  T_Set.AmpHash=Sip_Mesh.AmpHash;
  T_Set.Mesh=mymalloc(T_Set.nHash*sizeof(Points *));
  memset(T_Set.Mesh, 0, T_Set.nHash*sizeof(Points *));
  T_Set.nPoints=0;

  S_Set.Min_Norm=Sip_Mesh.Min_Norm;
  S_Set.Max_Norm=Sip_Mesh.Max_Norm;
  S_Set.nHash=HASH_SIZE;
  S_Set.AmpHash=Sip_Mesh.AmpHash;
  S_Set.Mesh=mymalloc(S_Set.nHash*sizeof(Points *));
  memset(S_Set.Mesh, 0, S_Set.nHash*sizeof(Points *));
  S_Set.nPoints=0;


  fill_T(); /* build init T~ set */
  /*  printf("Sip_Mesh\n");
      print_mesh(&Sip_Mesh);*/
  solve_nlp(); /* find first solution */
  /*  check_nlp();*/
  fill_R(); /* construct R set */

  inner=0;
  for (k=1;Sip_Opt.disc_h_mx>Sip_Mesh.nPoints;k++){

    /*    printf("h=");
    printarray(h, ntsip);
    printf("Sip_Mesh\n");
    print_mesh(&Sip_Mesh);
    printf("T_Set\n");
    print_mesh(&T_Set);
    printf("R_Set\n");
    print_mesh(&R_Set);
    printf("S_Set\n");
    print_mesh(&S_Set);*/

    /* actual X solves full mesh ? */
    if(!fill_S() && inner<Sip_Opt.disc_inner){
      inner++;
      printf("X not solved full mesh\n");

      /*      printf("S_Set\n");
      print_mesh(&S_Set);

      printf("T_Set\n");
      print_mesh(&T_Set);

      printf("R_Set\n");
      print_mesh(&R_Set);*/
      /* step ii */
      augment_T(); /* add point that are active or violated */
      /*      printf("augmented T_Set\n");
	      print_mesh(&T_Set);*/
      /* solve problem again, starting point will be the most recent */
      solve_nlp();
      /*      check_nlp(); */
      augment_R(); /* augment R by at least one */
      k--; /* same iteration */
    } else {
      printf("X solved full mesh\n");
      inner=0;
      next_halton_T(&ht);
      solve_nlp();
      /*      check_nlp();*/
      augment_R_RX();
    }
  }
  printf("%d halton points generated\n",ht);
}


void fill_halton_mesh(int *ht)
{
  int i, j, n;
  double *tp;
  double norm;

  if(Sip_Mesh.nPoints>=Sip_Opt.disc_h_mx)
    return; /* no more points to add */

  n=minimum(Sip_Opt.disc_h_mx-Sip_Mesh.nPoints, Sip_Opt.disc_ha_mx);
  tp=mymalloc(ntsip*sizeof(double));

  for(i=0;i<n;i++,(*ht)++){
    for(j=0;j<ntsip;j++)
      tp[j]=TBL[j]+halton(j, *ht)*(TBU[j]-TBL[j]);
    norm=norm2(ntsip, tp);
    insertpt(tp, norm, 0, &Sip_Mesh);
  }
  free(tp);
}


/**************************************************
Refine mesh and update T
**************************************************/
void next_halton_T(int *ht)
{
  int i, j;
  Points *tmp;

  fill_halton_mesh(ht);

  free_mesh(&T_Set);
  copy_mesh(&R_Set, &T_Set);
  add_mesh(&S_Set, &T_Set);

  /* encontrar vizinhos de S para restr. activas ou violadas */

  for(i=0;i<HASH_SIZE;i++){
    for(tmp=T_Set.Mesh[i];tmp;tmp=tmp->next){
      /* for all t constraints */
      for(j=0;j<ntsipc;j++){
	insert_neig(tmp->coord, tmp->norm, &T_Set);
      }
    }
  }
}


/************************************************
Check is NLP solved problem
************************************************/
void check_nlp()
{
  int i, j;
  Points *tmp;
  double cval;
  
  for(i=0;i<HASH_SIZE;i++){
    for(tmp=T_Set.Mesh[i];tmp;tmp=tmp->next){
      /* for all t constraints */
      for(j=0;j<ntsipc;j++)
	tmp->gval[j]=eval_sip(SIP_CONTVAL, j, X, tmp->coord, NULL, NULL, NULL, NULL);
      for(j=0;j<ntsipc;j++){
	cval=tmp->gval[j];
	if(cval+Sip_Opt.zero<TCBL[j] || cval-Sip_Opt.zero>TCBU[j])
	  printf("Restricao violada depois de NLP\n");
      }
    }
  }
}



/*****************************************************
insert neighbors of point (active or violated)
*****************************************************/
void insert_neig(double *t, double norm, Mesh *mset)
{
  int cell, i;
  Points *thispt, *tmp;

  if(norm<mset->Min_Norm || norm>mset->Max_Norm){
    fprintf(stderr,"Norm of point out of range\n");
    fprintf(stderr,"Min=%f Max=%f val=%f\n",mset->Min_Norm,mset->Max_Norm,norm);
    return; /* not inserted neighbors */
  }

  cell=(int)((norm-mset->Min_Norm)/mset->AmpHash);

  if(cell>=HASH_SIZE)
    cell=HASH_SIZE-1;
  if(cell<0)
    cell=0;

  cell--; /* first cell */
  
  for(i=0;i<3;i++){ /* neighbor in hash table */
    if(cell+i>=0 && cell+i<HASH_SIZE){
      thispt=mset->Mesh[cell+i];
      
      tmp=thispt;
      for(;tmp;tmp=tmp->next)
	if(neighbor(tmp->coord, t, ntsip))
	  check_insertpt(tmp->coord, tmp->norm, tmp->gval, mset);
      
    }
  }
}


int neighbor(double *a, double *b, int n)
{
  double dist;
  int i;

  dist=0;

  for(i=0;i<n;i++)
    dist+=abs((*(a+i))*(*(b+i)));

  dist=sqrt(dist);

  if(dist<Sip_Opt.disc_dist)return 1;
  return 0;
}



/*****************************************************************
Discretization method as proposed by Reemtsen
*****************************************************************/
void disc_reem()
{
  int k, i, j; /* Outer Iteration Number */
  double *h; /* Space between points */
  double *min, *max;
  double epson, fx;


  if(Sip_Opt.disc_k<=0){
    printf("Number of mesh refines less or equal to zero\n");
    return;
  }

  if(Sip_Opt.disc_h && Sip_Opt.ntvars!=ntsip){
      printf("User specified %d values to h but there are %d t vars\n",
	     Sip_Opt.ntvars, ntsip);
      if(Sip_Opt.ntvars>ntsip){
	printf("Using the first %d values\n", ntsip);
	h=mymalloc(ntsip*sizeof(double));
	memcpy(h, Sip_Opt.disc_h, ntsip*sizeof(double));
	free(Sip_Opt.disc_h);
	Sip_Opt.ntvars=ntsip;
	Sip_Opt.disc_h=h;
	h=NULL;
      } else {
	printf("Using first %d values and repeat the last value\n",
	       Sip_Opt.ntvars);
	h=mymalloc(ntsip*sizeof(double));
	memcpy(h, Sip_Opt.disc_h, Sip_Opt.ntvars*sizeof(double));
	for(i=0;i<(ntsip-Sip_Opt.ntvars);i++)
	  h[i+Sip_Opt.ntvars]=Sip_Opt.disc_h[Sip_Opt.ntvars-1];
	free(Sip_Opt.disc_h);
	Sip_Opt.ntvars=ntsip;
	Sip_Opt.disc_h=h;
	h=NULL;
      }	
  }

  if(!Sip_Opt.disc_h){
    Sip_Opt.disc_h=mymalloc(ntsip*sizeof(double));
    for(i=0;i<ntsip;i++){
      Sip_Opt.disc_h[i]=0.01;
      /*      pot=pow(100000,(double)(1.0/ntsip));
            Sip_Opt.disc_h[i]=maximum(((TBU[i]-TBL[i])/pot)*2*pow(3,Sip_Opt.disc_k-2), 0.2);*/
      /*            Sip_Opt.disc_h[i]=maximum(((TBU[i]-TBL[i])/pot)*6,((TBU[i]-TBL[i])/pow(10000,(double)(1.0/ntsip)))*6);*/
    }
  }

  /* h=disc_h */
  h=mymalloc(ntsip*sizeof(double));
  memcpy(h,Sip_Opt.disc_h, ntsip*sizeof(double));

  printf("h=(");
  for(i=0;i<ntsip-1;i++)
    printf("%f,", h[i]);
  printf("%f)\n",h[i]);


  min=mymalloc(ntsip*sizeof(double));
  max=mymalloc(ntsip*sizeof(double));

  for(i=0;i<ntsip;i++){
    if(TBL[i]<0 && TBU[i]>0){
      min[i]=0; max[i]=maximum(abs(TBL[i]), abs(TBU[i]));
    } else {
      if(TBL[i]>=0){
	min[i]=TBL[i]; max[i]=TBU[i];
      } else {
	min[i]=-TBU[i]; max[i]=-TBL[i];
      }
    }
  }

  /* Min norm will be the point with lower coordinates*/
  Sip_Mesh.Min_Norm=norm2(ntsip, min);

  /* Max norm will be the point with upper coordinates*/
  Sip_Mesh.Max_Norm=norm2(ntsip, max);

  free(min);
  free(max);

  /* for the moment I will have HASH_SIZE hash pointers */
  Sip_Mesh.nHash=HASH_SIZE;
  Sip_Mesh.AmpHash=(Sip_Mesh.Max_Norm-Sip_Mesh.Min_Norm)/HASH_SIZE;

  /* allocate pointers to point struct */
  Sip_Mesh.Mesh=mymalloc(Sip_Mesh.nHash*sizeof(Points *));
  memset(Sip_Mesh.Mesh, 0, Sip_Mesh.nHash*sizeof(Points *));

  Sip_Mesh.nPoints=0;

  fill_mesh(h); /* T[h] */

  start=Sip_Mesh.nPoints;
  
  T_Set.Min_Norm=Sip_Mesh.Min_Norm;
  T_Set.Max_Norm=Sip_Mesh.Max_Norm;
  T_Set.nHash=HASH_SIZE;
  T_Set.AmpHash=Sip_Mesh.AmpHash;
  T_Set.Mesh=mymalloc(T_Set.nHash*sizeof(Points *));
  memset(T_Set.Mesh, 0, T_Set.nHash*sizeof(Points *));
  T_Set.nPoints=0;

  epson=Sip_Opt.disc_eps;

  printf("Iter\tGrid\tNnlsp\tFullGrid\n");
  stats.nnlp=0;

  fill_T(); /* build init T~ set */

  fx=solve_nlp();
  stats.nnlpc=0; /* first grid does not count */

  printf("%d\t%d\t%d\t%s\n", 0, Sip_Mesh.nPoints,
	 (T_Set.nPoints)*ntsipc+nxsipc, "--");
  for(j=0;j<ntsip;h[j]/=2,j++);
  fill_mesh(h);

  for(k=0, i=1; i<Sip_Opt.disc_k;){
    if(check_solve()){
      printf("%d\t%d\t%d\t%s\n", i, Sip_Mesh.nPoints,
	     (T_Set.nPoints)*ntsipc+nxsipc, "Yes");
      for(j=0;j<ntsip;h[j]/=3,j++);
      epson/=pow(3, ntsip);
      i++;
      if(i<Sip_Opt.disc_k)
	fill_mesh(h);
    } else {
      fill_n_T(epson, fx);
      check_T();
      fx=solve_nlp();
      k++;
      printf("%d\t%d\t%d\t%s\n", i, Sip_Mesh.nPoints,
	     (T_Set.nPoints)*ntsipc+nxsipc, "No");
    }
  }

  /*  for(i=1;i<Sip_Opt.disc_k;i++){

    if(i==1){
      for(j=0;j<ntsip;h[j]/=2,j++);
      epson/=pow(2, ntsip);
    } else {
      for(j=0;j<ntsip;h[j]/=3,j++);
      epson/=pow(3, ntsip);
    }
    fill_mesh(h);
    
    for(k=0;k<Sip_Opt.disc_inner && !check_solve();k++){
      fill_n_T(epson,fx);
      check_T();
      printf("%d\t%d\t%d\t%s\n", i, Sip_Mesh.nPoints,
	     (T_Set.nPoints)*ntsipc+nxsipc, "No");
      fx=solve_nlp();
    }
    printf("%d\t%d\t%d\t%s\n", i, Sip_Mesh.nPoints,
	   (T_Set.nPoints)*ntsipc+nxsipc, "Yes");

}*/
  /*  print_mesh(&T_Set);*/
  write_result(fx);
}

/*    for(j=0;j<ntsip;h[j]/=2,j++); */
/*    epson/=pow(2, ntsip); */
/*    fill_mesh(h); */
/*    for (k=0, i=1;;k++){ */
/*      if(check_solve()){ */
/*        if((i)>=Sip_Opt.disc_k)break; */
/*        for(j=0;j<ntsip;h[j]/=3,j++);       */
/*        epson/=pow(3, ntsip); */
/*        fill_mesh(h); */
/*        printf("%d\t%d\t%d\t%s\n", i, Sip_Mesh.nPoints, */
/*  	     (T_Set.nPoints)*ntsipc+nxsipc, "Yes"); */
/*        k--;i++; */
/*      } else { */
/*        fill_n_T(epson,fx); */
/*        printf("%d\t%d\t%d\t%s\n", i, Sip_Mesh.nPoints, */
/*  	     (T_Set.nPoints)*ntsipc+nxsipc, "No"); */
/*        fx=solve_nlp(); */
/*      } */
  /*    } */



/*    fill_n_T(epson, fx);
if(check_solve()){
if(i==1){
for(j=0;j<ntsip;h[j]/=2,j++);
epson/=pow(2, ntsip);
} else {
for(j=0;j<ntsip;h[j]/=3,j++);
epson/=pow(3, ntsip);
}
fill_mesh(h);
k--;i++;
printf("%d\t%d\t%d\t%s\n", k, Sip_Mesh.nPoints,
(T_Set.nPoints)*ntsipc+nxsipc, "No");
} else {
fx=solve_nlp();
printf("%d\t%d\t%d\t%s\n", k, Sip_Mesh.nPoints,
(T_Set.nPoints)*ntsipc+nxsipc, "No");
} */




void fill_n_T(double epson, double fx)
{
  int i, j;
  Points *tmp;
  double cval;

  free_mesh(&T_Set);

  /*  mini=+9999999;

  for(i=0;i<HASH_SIZE;i++)
    for(tmp=Sip_Mesh.Mesh[i];tmp;tmp=tmp->next)
      for(j=0;j<ntsipc;j++){
	tmp->gval[j]=eval_sip(SIP_CONTVAL, j, X, tmp->coord, NULL, NULL, NULL, NULL);
	if(tmp->gval[j]<mini)
	  mini=tmp->gval[j];
      }
  */
  
  for(i=0;i<HASH_SIZE;i++){
    for(tmp=Sip_Mesh.Mesh[i];tmp;tmp=tmp->next){
      /* for all t constraints */
      for(j=0;j<ntsipc;j++)
	tmp->gval[j]=eval_sip(SIP_CONTVAL, j, X, tmp->coord, NULL, NULL, NULL, NULL);
      for(j=0;j<ntsipc;j++){
	cval=tmp->gval[j];
	if(cval+epson*fabs(fx)>=TCBU[j] || cval-epson*fabs(fx)<=TCBL[j]){
	  /* active constraint */
	  check_insertpt(tmp->coord, tmp->norm, tmp->gval, &T_Set);
	}
      }
    }
  }
}


int check_solve()
{
  int i, j;
  Points *tmp;
  double cval;
  int rval; /* return 1 if X solves full mesh 0 otherwise */

  rval=1;

  for(i=0;i<HASH_SIZE;i++){
    for(tmp=Sip_Mesh.Mesh[i];tmp;tmp=tmp->next){
      /* for all t constraints */
      for(j=0;j<ntsipc;j++)
	tmp->gval[j]=eval_sip(SIP_CONTVAL, j, X, tmp->coord, NULL, NULL, NULL, NULL);
      for(j=0;j<ntsipc;j++){
	/* cval=tmp->gval; calculated in fill_n_T */
	cval=tmp->gval[j];
	if(equal(&cval, TCBL+j, 1) || equal(&cval, TCBU+j, 1) ||
	   cval+Sip_Opt.zero<=TCBL[j] || cval-Sip_Opt.zero>=TCBU[j]){
	  /* active or violated constraint */
	  if(!equal(&cval,TCBL+j,1) && !equal(&cval,TCBU+j,1))
	    /* violated constraint */
	    rval=0;
	}
      }
    }
  }
  return rval;
}


void write_result(double fx)
{
  FILE *fp;


  fp=fopen("./results", "a+");
  fprintf(fp, " & %d & %d & %.1f & %d & %.6f\\\\\n", start, Sip_Mesh.nPoints, ((double)stats.nnlpc)/((double)(stats.nnlp-1)), stats.nnlp, fx);
  fclose(fp);

}
