#include "sip_interior.h"
#include "sip_int.h"


void intpkkts()
{
  double tsample[ntsip];
  double dir[nxsip+2*ntsipc];
  double hss[nxsip*nxsip];
  double x0[nxsip], x1[nxsip], laggdx1[nxsip], laggdx0[nxsip];
  double y[nxsip], yeps[nxsip];
  double s0[ntsipc], s1[ntsipc], term[ntsipc];
  double lag0[ntsipc], lag1[ntsipc], lagy[ntsipc];
  double laggdl0[ntsipc], laggdl1[ntsipc], jac[nxsip*ntsipc];
  double f0, f1;
  int ko, ki, ieps, l, tki;
  double miu;  /* penalty parameter */
  FILE *fp; /* for results file */


  miu=Sip_Opt->startmiu/10;
  tki=0;
  ieps=0;

  memcpy(x1, X, nxsip*sizeof(double)); /* x1 with initial guess */
  memcpy(yeps, X, nxsip*sizeof(double));

  /* lagrange multipliers and equality variables */
  for(l=0;l<ntsipc;l++){
    lag1[l]=-1.0;
    s1[l]=1.0;
  }    


  for(ko=0;ko<Sip_Opt->maxitero;ko++){


    /* y with initial guess for inner iter */
    memcpy(y, x1, nxsip*sizeof(double));
    memcpy(lagy, lag1, ntsipc*sizeof(double));


    printf("STARTING INNER ITERATIONS ********** miu=%f\n", miu);


    /* Get Lagrangian, miu has changed */
    lagrangian(nxsip, x1, ntsipc, lag1, s1, ntsip, tsample, miu, FIRST_DERIV,
	       laggdx1, laggdl1, jac, term, &f1);
    
    for(ki=0;ki<Sip_Opt->maxiteri;ki++){ /* Quasi-Newton inner iteration */

      /* Set Hessian */
      set_hss(hss, ki, nxsip, x1, x0, laggdx1, laggdx0);
      /*        printf("Hessian\n"); */
      /*        printmatrix(hss, nxsip, nxsip); */

    
      memcpy(x0,      x1,      nxsip*sizeof(double));
      memcpy(laggdx0, laggdx1, nxsip*sizeof(double));
      memcpy(s0,      s1,      ntsipc*sizeof(double));
      memcpy(lag0,    lag1,    ntsipc*sizeof(double));
      memcpy(laggdl0, laggdl1, ntsipc*sizeof(double));
      f0=f1;

      get_dir_interior(nxsip, x0, s0, ntsipc, lag0, hss, jac, laggdx0, laggdl0,
		       term, miu, dir);

      printf("X:\n");
      printarray(x0, nxsip);
      printf("S:\n");
      printarray(s0, ntsipc);
      printf("LAG:\n");
      printarray(lag0, ntsipc);
      printf("Gradient x:\n");
      printarray(laggdx0, nxsip);
      printf("Gradient l:\n");
      printarray(laggdl0, ntsipc);
      printf("Direction:\n");
      printarray(dir, nxsip+2*ntsipc);
      
      if(Sip_Opt->ScaleDir)
	scale_dir(nxsip+2*ntsipc, dir);

      switch (Sip_Opt->linesearch){
      default:
	printf("Unknown LineSearch\nUsing Armijo\n");
      case ARMIJO:
	armijo_interior(nxsip, x0, s0, ntsipc, lag0, dir, miu, x1, s1, lag1);
	break;
      };


      lagrangian(nxsip, x1, ntsipc, lag1, s1, ntsip, tsample, miu, FIRST_DERIV,
		 laggdx1, laggdl1, jac, term, &f1);


      /* inner stopping criteria */
      if(pf_i_stop(nxsip, x0, x1, laggdx1, 0)){
	printf("INNER STOPPING IN ITERATION ********* %d\n", ki+1);
	break;
      }
    }

    tki+=ki+1;

    /* outer stopping criteria */
    if(pf_o_stop(nxsip, y, x1, laggdx1, 0)){
      
      if(ieps>0){
	if(pf_eps_stop(nxsip, yeps, x1)){
	  break;
	}
      }
      ieps++;
      memcpy(yeps, x1,    nxsip*sizeof(double));
      printf("Decreasing epsilon\n");
      Sip_Opt->pf_epsilon/=10;
      Sip_Opt->pf_tau/=10;
      if(Sip_Opt->pf_epsilon<1e-8)
	break;
      
    } else {
      miu/=Sip_Opt->miufactor;
      if(miu<1e-8)
	break;
    }

  }
  
  memcpy(X, x1, nxsip*sizeof(double));
  printf("Solution found\n");
  printarray(X, nxsip);
  printf("Objective function value\n");
  printf("%f\n", f1);


  fp=fopen("./results", "a+");
  f0=eval_sip(SIP_OBJVAL, 0, X, NULL, NULL, NULL, NULL, NULL);

  printf("Gradient of penalty:\n");
  printarray(laggdx1, nxsip);


  fprintf(fp, " & %d & %d & %d & %d & %.6e & %.1e & %.1e \\\\\n",
	  tki, ko+1, stats.nsip_lag, stats.nsip_mer, f0, miu, Sip_Opt->pf_epsilon);
  fclose(fp);


}



/*********************************************************
Evaluate the penalty function
*********************************************************/
double lagrangian(int nx, double *x, int nl, double *lag, double *s, int nt,
		  double *t, double miu, int dlevel, double *laggdx,
		  double *laggdl, double *jac, double *term, double *f)
{
  double lagval, gd[nx];
  double num, den, dummy;
  int i, k;

  stats.nsip_lag++;

  lagval=0.0;
  if(dlevel==FIRST_DERIV){
    memset(laggdx, 0, sizeof(double)*nx);
    memset(laggdl, 0, sizeof(double)*nl);
  }


  for(i=0; i<nl; i++){
    adapt_trap(&gxtteo, TBL, TBU, nx, x, nt, t,
	       Sip_Opt->adapt_int_n, i, &num, &den,
	       Sip_Opt->adapt_int_error, Sip_Opt->adapt_int_amp,
	       dlevel, gd, 0);
    
    
    if(dlevel==FIRST_DERIV && jac!=NULL)
      memcpy(&(jac[i*nx]), gd, sizeof(double)*nx);

    lagval-=lag[i]*(num+s[i])+miu*log(s[i]+Sip_Opt->pf_tau);

    term[i]=num-miu/lag[i]-Sip_Opt->pf_tau;

    /*      printf("term[%d]=%f\n", i, term[i]); */
    /*      printf("num[%d]=%f\n", i, num); */
    /*      printf("lag[%d]=%f\n", i, lag[i]); */
    
    if(dlevel==FIRST_DERIV){
      if(laggdl)
	laggdl[i]=-num-s[i];
      for(k=0;k<nx;k++)
	laggdx[k]-=lag[i]*gd[k];
    }
  }
  
  

  lagval+=*f=eval_sip(SIP_OBJVAL, 0, x, NULL, NULL, NULL, NULL, NULL);
  
  if(dlevel==FIRST_DERIV){
    dummy=eval_sip(SIP_OBJGRD, 0, x, NULL, gd, NULL, NULL, NULL);
    for(k=0;k<nx;k++)
      laggdx[k]+=gd[k];
  }

  return lagval;
}




void get_dir_interior(int nx, double *x, double *s, int nl, double *lag,
		      double *hss, double *jac, double *laggdx, double *laggdl,
		      double *term, double miu, double *dir)
{
  int i, j, sing;
  double fullh[(nx+nl)*(nx+nl)];
  double ind[nx+nl];
  double sol[nx+nl];
  
  for(i=0;i<nx;i++)
    memcpy(&fullh[i*(nx+nl)], &hss[i*nx], nx*sizeof(double));

  for(i=0;i<nx;i++)
    for(j=0;j<nl;j++){
      fullh[j+nx+i*(nx+nl)]=-jac[j*nx+i];
      fullh[i+(j+nx)*(nx+nl)]=-jac[j*nx+i];
    }

  for(i=0;i<nl;i++){
    memset(&fullh[(nx+nl)*(nx+i)+nx], 0, nl*sizeof(double));
    fullh[(nx+nl)*(nx+i)+nx+i]=(s[i]+Sip_Opt->pf_tau)/lag[i];
  }

  for(i=0;i<nx;i++)
    ind[i]=-laggdx[i];
  for(i=0;i<nl;i++)
    ind[i+nx]=term[i];

  printf("HSS:\n");
  printmatrix(hss, nx, nx);
  printf("Jac:\n");
  printmatrix(jac, nl, nx);
  printf("Fullh:\n");
  printmatrix(fullh, (nx+nl), (nx+nl));
  printf("Ind:\n");
  printarray(ind, nx+nl);


  egpp(nx+nl, fullh, ind, sol, &sing);


  if(sing){
    printf("Full Hessian is singular\n");
    exit(1);
  }

  memcpy(dir, sol, nx*sizeof(double));
  memcpy(&dir[nx+nl], &sol[nx], nl*sizeof(double));

  for(i=0;i<nl;i++)
    dir[i+nx]=-((s[i]+Sip_Opt->pf_tau)/lag[i])*sol[nx+i]
      -miu/lag[i]-(s[i]+Sip_Opt->pf_tau);
  
}


void armijo_interior(int nx, double *x0, double *s0, int nl, double *lag0,
		     double *dir, double miu, double *x1, double *s1,
		     double *lag1)
{
  int l;
  double alpha, alpha0, dif, phi0, phiomega, gd[nx], ngd, nd;
  static double beta=0;


  phi0=interior_merit(nx, x0, s0, nl, lag0, miu, beta, gd);

  ngd=0.0;
  nd=0.0;
  for(l=0;l<nx;l++){
    ngd+=gd[l]*dir[l];
    nd+=dir[l]*dir[l];
  }

  nd=sqrt(nd);


  printf("phi0=%f\n", phi0);
  printf("Beta=%f\n", beta);

  alpha0=1.0;
  for(l=0;l<nl;l++){
    if(dir[nx+nl+l]>0 && -lag0[l]/dir[nx+nl+l]<alpha0)
      alpha0=-lag0[l]/dir[nx+nl+l];
    if(dir[nx+l]<0 && -(s0[l]+Sip_Opt->pf_tau)/dir[nx+l]<alpha0)
      alpha0=-(s0[l]+Sip_Opt->pf_tau)/dir[nx+l];
  }

  if(alpha0<0)
    alpha0=0;

  if(alpha0!=1)
    alpha0=0.95*alpha0;

  alpha=alpha0;
  printf("Alpha0=%f\n", alpha0);

  
  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    for(l=0;l<nl;l++){
      s1[l]=s0[l]+alpha*dir[l+nx];
      lag1[l]=lag0[l]+alpha*dir[nx+nl+l];
    }
    phiomega=interior_merit(nx, x1, s1, nl, lag1, miu, beta, NULL);
    printf("alpha=%f phiomega=%f\n", alpha, phiomega);
    
    dif=phi0-phiomega;
    if(dif<-(Sip_Opt->armijo)*alpha*ngd){
      alpha=alpha/2;
      if(alpha*nd<=1e-8){
	if(beta<=0)beta=0.1;
	if(beta<1e+9)
	  beta*=10;
	printf("Increasing beta %f\n", beta);
	if(beta>1e+8){
	  alpha=0.0;
	  memcpy(x1, x0, nx*sizeof(double));
	  memcpy(s1, s0, nl*sizeof(double));
	  memcpy(lag1, lag0, nl*sizeof(double));
	  return;
	}
	alpha=alpha0;
	phi0=interior_merit(nx, x0, s0, nl, lag0, miu, beta, gd);
	printf("phi0=%f\n", phi0);

	ngd=0.0;
	for(l=0;l<nx;l++)
	  ngd+=gd[l]*dir[l];
      }
    } else {
      printf("Step=%f\n",alpha);
      return;
    }
  }
}



double interior_merit(int nx, double *x, double *s, int nl, double *lag,
		      double miu, double beta, double *grad)
{
  double merval, num, den, term1, term2;
  double tsample[ntsip], gd[nx], dummy;
  int i, j;
  
  stats.nsip_mer++;
  
  term1=0.0;
  term2=0.0;

  if(grad)
    memset(grad, 0, nx*sizeof(double));

  for(i=0; i<nl; i++){
    if(grad){
      adapt_trap(&gxtteo, TBL, TBU, nx, x, ntsip, tsample,
		 Sip_Opt->adapt_int_n, i, &num, &den,
		 Sip_Opt->adapt_int_error, Sip_Opt->adapt_int_amp,
		 FIRST_DERIV, gd, 0);
    } else {
      adapt_trap(&gxtteo, TBL, TBU, nx, x, ntsip, tsample,
		 Sip_Opt->adapt_int_n, i, &num, &den,
		 Sip_Opt->adapt_int_error, Sip_Opt->adapt_int_amp,
		 NO_DERIV, NULL, 0);
    }
    term1+=log(s[i]+Sip_Opt->pf_tau);
    term2+=pow(num+s[i], 2.0);
    if(grad)
      for(j=0;j<nx;j++)
	grad[j]+=beta*gd[j]*num;
  }

  /*    printf("Merit term1 (optima  log)=%.20f\n", term1); */
  /*    printf("Merit term2 (feasibility)=%.20f\n", term2); */
  

  merval=eval_sip(SIP_OBJVAL, 0, x, NULL, NULL, NULL, NULL, NULL)
    -miu*term1+beta*term2/2;

  if(grad){
    dummy=eval_sip(SIP_OBJGRD, 0, x, NULL, gd, NULL, NULL, NULL);
    for(j=0;j<nx;j++)
      grad[j]+=gd[j];
  }
  
  return merval;
}
