#include "sip_npsol_pf.h"
#include "sip_int.h"

extern STATS stats;


#ifdef NPSOL

double ext_miu; /* must be external, for pf_objfun */

void npsol_penaltyfs()
{
  double tsample[ntsip];
  double x1[nxsip], g1[nxsip], f1, y[nxsip], gy[nxsip], fy;
  double f0, norg, dummy;
  int i;
  int ko;
  double pt;  /* penalty parameter */

  FILE *fp; /* for results file */



  ext_miu=Sip_Opt.startmiu;

  memcpy(x1, X, nxsip*sizeof(double)); /* x1 with initial guess */

  for(ko=0;ko<Sip_Opt.maxitero;ko++){


    /* y with initial guess for inner iter */
    memcpy(y, x1, nxsip*sizeof(double));

    /* update penalty parameter */
    if(ko)
      ext_miu*=Sip_Opt.miufactor;

    printf("STARTING INNER ITERATIONS ********** miu=%f\n", ext_miu);

    gpenaltyf(nxsip, y, ntsip, tsample, ext_miu, gy, &fy, &pt);

    npsol_solve_pf(x1);

    /* could be avoided if returned by solve_pf */
    gpenaltyf(nxsip, x1, ntsip, tsample, ext_miu, g1, &f1, &pt);
    
    /* outer stopping criteria */
    if(pf_o_stop(nxsip, y, x1, g1, pt)){
      printf("OUTER STOPPING IN ITERATION ********* %d\n", ko+1);
      break;
    }
  }

  printf("STOPPING IN ITERATION ********* %d\n", ko+1);  
  memcpy(X, x1, nxsip*sizeof(double));
  printf("Solution found\n");
  printarray(X, nxsip);
  printf("Penalty function value\n");
  printf("%f\n",penaltyf(nxsip, x1, ntsip, tsample, ext_miu, &pt, NO_DERIV, NULL));


  fp=fopen("./results", "a+");
  f0=eval_sip(SIP_OBJVAL, 0, X, NULL, NULL, NULL, NULL, NULL);

  printf("Gradient of penalty:\n");
  printarray(g1, nxsip);

  norg=0.0;
  for(i=0;i<nxsip;i++)
    norg+=g1[i]*g1[i];
  norg=sqrt(norg);

  dummy=eval_sip(SIP_OBJGRD, 0, x1, NULL, g1, NULL, NULL, NULL);

  printf("Gradient of objective:\n");
  printarray(g1, nxsip);
  

  fprintf(fp, " & ? & %d & %d & %d & %d & %d & %.4f & %.4f & %.2e\\\\\n",
	  ko+1, stats.nsip_objval, stats.nsip_objgrd, stats.nsip_contval,
	  stats.nsip_contgrd, f0, f1, norg);
  fclose(fp);
}


/***************************************************************
NPSOL with my penalty function
***************************************************************/


void npsol_pf_objfun(long *MODE, long *N, double *X0, double *F, double *G,
	       long *NSTATE)
{
  double t[ntsip], pt;

  if (*MODE != 1)
    *F=penaltyf(nxsip, X0, ntsip, t, ext_miu, &pt, NO_DERIV, NULL);
/*    if (*MODE > 0) { */
/*      gpenaltyf(nxsip, X0, ntsip, t, ext_miu, G, F, &pt); */
/*    } */
}



void npsol_pf_confun(long *MODE, long *NCNLN, long *N, long *NROWJ,
		     long *NEEDC, double *X0, double *C, double *CJAC,
		     long *NSTATE)
{

}



void npsol_solve_pf(double *x1)
{
  double *npsol_bl, *npsol_bu, *npsol_a, *npsol_c, *npsol_cjac, *npsol_clambda;
  double *npsol_objgrd, *npsol_r, *npsol_w;
  double npsol_objf;
  long npsol_n, npsol_nclin, npsol_ncnln, npsol_nrowa, npsol_nrowj;
  long npsol_nrowr;
  long npsol_inform, npsol_iter;
  long *npsol_iw, *npsol_istate, npsol_leniw, npsol_lenw;
  int i, coni;


  
  npsol_n=nxsip;
  npsol_nclin=0; /* no linear constraints */
  npsol_nrowa=0;
  npsol_a=NULL;

  npsol_ncnln=0; /* no nonlinear constraints */

  npsol_bl=mymalloc((nxsip+npsol_ncnln)*sizeof(double));
  npsol_bu=mymalloc((nxsip+npsol_ncnln)*sizeof(double));

  coni=0;
  /* variables bounds */
  for(i=0;i<nxsip;i++){
    npsol_bl[coni]=XBL[i];
    npsol_bu[coni++]=XBU[i];
  }


  /* allocate space for jacobian */
  npsol_nrowj=npsol_ncnln;
  npsol_cjac=mymalloc(npsol_ncnln*nxsip*sizeof(double));

  /* allocate istate array */
  npsol_istate=mymalloc((nxsip+npsol_ncnln)*sizeof(long));

  /* allocate space for c array */
  npsol_c=mymalloc(npsol_ncnln*sizeof(double));

  /* allocate clambda array */
  npsol_clambda=mymalloc((nxsip+npsol_ncnln)*sizeof(double));

  /* allocate objgrd array */
  npsol_objgrd=mymalloc((nxsip)*sizeof(double));

  /* allocate r matrix */
  npsol_nrowr=nxsip;
  npsol_r=mymalloc((nxsip*npsol_nrowr)*sizeof(double));

  /* allocate workspace for npsol */
  npsol_leniw=3*nxsip+2*npsol_ncnln;
  npsol_iw=mymalloc(npsol_leniw*sizeof(long));
  npsol_lenw=2*nxsip*nxsip+nxsip*npsol_ncnln+
    2*nxsip*npsol_ncnln+20*nxsip+11*npsol_ncnln+21*npsol_ncnln;
  npsol_w=mymalloc(npsol_lenw*sizeof(double));

  npoptn_("Derivative level   0", 20L);
/*    npoptn_("Major print level 20", 20L); */
/*    npoptn_("Minor print level 20", 20L); */

  npsol_(&npsol_n, &npsol_nclin, &npsol_ncnln, &npsol_nrowa, &npsol_nrowj,
	 &npsol_nrowr, npsol_a, npsol_bl, npsol_bu, npsol_pf_confun,
	 npsol_pf_objfun, &npsol_inform, &npsol_iter, npsol_istate, npsol_c,
	 npsol_cjac, npsol_clambda, &npsol_objf, npsol_objgrd, npsol_r, x1,
	 npsol_iw, &npsol_leniw, npsol_w, &npsol_lenw);
  printf("\tNonlinear used %ld iterations Obj=%f Inform=%ld\n",npsol_iter,
	 npsol_objf, npsol_inform);

  /*  printf("NPSOL inform=%ld\n", npsol_inform);
  printf("Exit point for NPSOL\n");
  printarray(X, nxsip);*/

  /* free space */
  free(npsol_bl);
  free(npsol_bu);
  free(npsol_cjac);
  free(npsol_istate);
  free(npsol_c);
  free(npsol_clambda);
  free(npsol_objgrd);
  free(npsol_r);
  free(npsol_iw);
  free(npsol_w);
  return;
}

#else

void npsol_penaltyfs()
{

  printf("\n\nNPSOL not available\nPlease read the user manual about it\n");
  printf("Aborting....\n");
  exit(1);

}


#endif
