#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>


#ifndef mymalloc
#define mymalloc(x) mymalloc_ASL(x)
extern void *mymalloc_ASL(size_t len);
#endif



/*  void printarray(double *a,int n) */
/*  { */
/*    int i; */

/*    for(i=0;i<n;i++) */
/*      printf("%4.10lf ", a[i]); */
/*    printf("\n"); */
/*  } */

/*  void printmatrix(double *m, int n1, int n2) */
/*  { */
/*    int i; */

/*    for(i=0;i<n1;i++) */
/*      printarray(m+n2*i, n2); */
/*  } */



/**********************************************
Posicao do pivot na egpp
**********************************************/
int maxpos(double *a, int n, int dim)
{
  int i, nmax;

  for(i=0,nmax=0;i<n;i++)
    if(fabs(a[nmax*dim])<fabs(a[i*dim]))
      nmax=i;
  return nmax;
}


/*************************************************************
 Eliminacao de Gauss com Pivotagem Parcial ax=b
*************************************************************/
void egpp(int n, double *a, double *b, double *x, int *singular)
{
  int i, j, k, npiv;
  double baux, mult;
  double *col;
#if defined __linux__ || defined linux
  double aux[n];
#else
  double *aux;
  aux=mymalloc(n*sizeof(double));
#endif
  
  
  *singular=0;
  
  for(j=0;j<n-1;j++){
    col=a+n*j+j; /* posicao na coluna do pivot */
    npiv=maxpos(col, n-j, n); /* posicao do maximo da coluna */
    if(npiv!=0){ /* fazer troca */
      memcpy(aux, col, (n-j)*sizeof(double));
      memcpy(col, col+npiv*n, (n-j)*sizeof(double));
      memcpy(col+npiv*n, aux, (n-j)*sizeof(double));
      baux=b[j];
      b[j]=b[j+npiv];
      b[j+npiv]=baux;
    }
    for(i=1;i<n-j;i++){
      mult=-(*(col+i*n))/(*col);
      /*        printf("Pivot=%f Mult=%f\n", *col, mult); */
      for(k=1;k<n-j;k++)
	*(col+i*n+k)=(*(col+k))*mult+(*(col+i*n+k));
      b[i+j]=b[j]*mult+b[i+j];
    }
  }

  /* calculate solution */

  for(i=n-1;i>=0;i--){ /* from bottom to top */
    x[i]=b[i];
    for(j=n-1;j>i;j--)
      x[i]-=x[j]*(*(a+i*n+j));
    if(!(*(a+i*n+i))){
      *singular=1;
#if !defined __linux__ && !defined linux
      free(aux);
#endif
      return;
    }
    x[i]/=(*(a+i*n+i));
  }

#if !defined __linux__ && !defined linux
  free(aux);
#endif

}


/*************************************************************
 Cholesky
*************************************************************/
void cholesky(int n, double *a, double *b, double *x, int *singular)
{
  int i, j, k;
#if defined __linux__ || defined linux
  double aux[n*n], auxv[n];
#else
  double *aux, *auxv;
  aux=mymalloc(n*n*sizeof(double));
  auxv=mymalloc(n*sizeof(double));
#endif
  
  
  *singular=0;

  memset(aux, 0, n*n*sizeof(double));

  for(i=0;i<n;i++){
    if(a[i*n+i]<0)
      printf("Negative value in the diagonal\n");
    aux[i*n+i]=sqrt(fabs(a[i*n+i]));
    for(j=i;j<n;j++){
      aux[j+i*n]=a[j+i*n]/aux[i*n+i];
      for(k=i+1;k<j+1;k++)
	a[j+k*n]-=aux[j+i*n]*aux[k+i*n];
    }
  }
  
  /*    printf("L\n"); */
  /*    printmatrix(aux, n, n); */
  
  /*    printf("A\n"); */
  /*    printmatrix(a, n, n); */

  /* calculate solution */

  for(i=0;i<n;i++){ /* from top to bottom*/
    auxv[i]=b[i];
    for(j=0;j<i;j++)
      auxv[i]-=auxv[j]*(aux[j*n+i]);
    if(!aux[i+i*n]){
      *singular=1;
#if !defined __linux__ && !defined linux
      free(aux);
      free(auxv);
#endif
      return;
    }
    auxv[i]/=aux[i+i*n];
  }

  for(i=n-1;i>=0;i--){ /* from bottom to top */
    x[i]=auxv[i];
    for(j=n-1;j>i;j--)
      x[i]-=x[j]*(aux[j+i*n]);
    if(!aux[i+i*n]){
      *singular=1;
#if !defined __linux__ && !defined linux
      free(aux);
      free(auxv);
#endif
      return;
    }
    x[i]/=aux[i+i*n];
  }

#if !defined __linux__ && !defined linux
  free(aux);
  free(auxv);
#endif

}

/*************************************************************
 Modified Cholesky
*************************************************************/
void mod_cholesky(int n, double *a, double *b, double *x, double eps,
		  int *singular)
{
  int i, j, k;
  double beta;
#if defined __linux__ || defined linux
  double aux[n*n], auxv[n];
#else
  double *aux, *auxv;
  aux=mymalloc(n*n*sizeof(double));
  auxv=mymalloc(n*sizeof(double));
#endif

/*    int J[n]; */
  
  
  *singular=0;

  memset(aux, 0, n*n*sizeof(double));
/*    memset(J, 0, n*sizeof(double)); */

  beta=*a; /* A_00 */
  for(i=1;i<n;i++)
    if(fabs(a[i*n+i])>beta)
      beta=fabs(a[i*n+i]);
    

  for(i=0;i<n;i++){
    if(a[i*n+i]<beta*eps){
      /*        printf("Modifying cholesky\n"); */
      /*      J[i]=1;  keep a record of small pivot */
      a[i*n+i]=1.0e+128;
/*        for(j=i;j<n;j++) */
/*  	aux[j+i*n]=0.0; */
    }
    aux[i*n+i]=sqrt(a[i*n+i]);
    for(j=i;j<n;j++){
      aux[j+i*n]=a[j+i*n]/aux[i*n+i];
      for(k=i+1;k<j+1;k++)
	a[j+k*n]-=aux[j+i*n]*aux[k+i*n];
    }
  }
  
  /*    printf("L\n"); */
  /*    printmatrix(aux, n, n); */
  
  /*    printf("A\n"); */
  /*    printmatrix(a, n, n); */

  /* calculate solution */

  for(i=0;i<n;i++){ /* from top to bottom*/
    /*      if(J[i]){ */
    /*        auxv[i]=0.0; */
    /*      } else { */
    auxv[i]=b[i];
    for(j=0;j<i;j++)
      auxv[i]-=auxv[j]*(aux[j*n+i]);
    if(!aux[i+i*n]){
      *singular=1;
#if !defined __linux__ && !defined linux
      free(aux);
      free(auxv);
#endif
      return;
    }
    auxv[i]/=aux[i+i*n];
  }


  for(i=n-1;i>=0;i--){ /* from bottom to top */
/*      if(J[i]){ */
/*        x[i]=0.0; */
/*      } else { */
    x[i]=auxv[i];
    for(j=n-1;j>i;j--)
      x[i]-=x[j]*(aux[j+i*n]);
    if(!aux[i+i*n]){ /* just in case */
      *singular=1;
#if !defined __linux__ && !defined linux
      free(aux);
      free(auxv);
#endif
      return;
    }
    x[i]/=aux[i+i*n];
  }
#if !defined __linux__ && !defined linux
  free(aux);
  free(auxv);
#endif

}


/*************************************************************
 Modified Cholesky
*************************************************************/
void zero_cholesky(int n, double *a, double *b, double *x, double eps,
		   int *singular)
{
  int i, j, k;
  double beta;
#if defined __linux__ || defined linux
  double aux[n*n], auxv[n];
  int J[n];
#else
  double *aux, *auxv;
  int *J;
  aux=mymalloc(n*n*sizeof(double));
  auxv=mymalloc(n*sizeof(double));
  J=mymalloc(n*sizeof(int));
#endif
  
  
  *singular=0;

  memset(aux, 0, n*n*sizeof(double));
  memset(J, 0, n*sizeof(double));

  beta=*a; /* A_00 */
  for(i=1;i<n;i++)
    if(fabs(a[i*n+i])>beta)
      beta=fabs(a[i*n+i]);
    

  for(i=0;i<n;i++){
    if(a[i*n+i]<beta*eps){
      printf("Modifying cholesky\n");
      J[i]=1; /* keep a record of small pivot */
      for(j=i;j<n;j++)
  	aux[j+i*n]=0.0;
    } else {
      aux[i*n+i]=sqrt(a[i*n+i]);
      for(j=i;j<n;j++){
	aux[j+i*n]=a[j+i*n]/aux[i*n+i];
	for(k=i+1;k<j+1;k++)
	  a[j+k*n]-=aux[j+i*n]*aux[k+i*n];
      }
    }
  }
  
  /*    printf("L\n"); */
  /*    printmatrix(aux, n, n); */
  
  /*    printf("A\n"); */
  /*    printmatrix(a, n, n); */

  /* calculate solution */

  for(i=0;i<n;i++){ /* from top to bottom*/
    if(J[i]){
      auxv[i]=0.0;
    } else {
      auxv[i]=b[i];
      for(j=0;j<i;j++)
	auxv[i]-=auxv[j]*(aux[j*n+i]);
      if(!aux[i+i*n]){
	*singular=1;
#if !defined __linux__ && !defined linux
	free(aux);
	free(auxv);
	free(J);
#endif
	return;
      }
      auxv[i]/=aux[i+i*n];
    }
  }


  for(i=n-1;i>=0;i--){ /* from bottom to top */
    if(J[i]){
      x[i]=0.0;
    } else {
      x[i]=auxv[i];
      for(j=n-1;j>i;j--)
	x[i]-=x[j]*(aux[j+i*n]);
    if(!aux[i+i*n]){ /* just in case */
      *singular=1;
#if !defined __linux__ && !defined linux
	free(aux);
	free(auxv);
	free(J);
#endif
      return;
    }
    x[i]/=aux[i+i*n];
    }
  }
#if !defined __linux__ && !defined linux
	free(aux);
	free(auxv);
	free(J);
#endif
}


/*  
main()
{
  double a[]={ 80, 0, 30, 10, 0, 80, 10, 10, 16, 20, 60, 72, 4, 0, 0, 8};
  double a[]={ 4, 1, 1, 1, 1, 4, 1, 1, 1, 1, 4, 1, 1, 1, 1, 4};
  double b[]={ 40, 27, 31, 2};
  double x[4];
  int sing;

  cholesky(4, a, b, x, &sing);

  printf("singular %d\n", sing);

  printf("x=%f, %f, %f, %f\n", x[0], x[1], x[2], x[3]);

}
*/


