#include "funcadd.h"

double blend(int i, int k, double x, double *knots);
double dblend(int j, int i,int k,double x,double *knots);
double alpha(int n, int k,int j, int r, double *ctl, double *knots);
double dalpha(int i, int n, int k, int j, int r, double *ctl, double *knots);

double bspline(arglist *al)
{
  double x, res;
  double *d, *h, *ra, *ctl, *knots;
  int *at, i, j, k, n;
  AmplExports *ae = al->AE;
    
  at = al->at;
  ra = al->ra;
  d = al->derivs;
  h = al->hes;

  if (al->n < 3){
    fprintf(Stderr, "Invalid number of args to bspline expected >=3 and obtained %d\n", al->n);
    return 0;
  }
  
  x=ra[0]; k=(int)ra[1]; n=ra[2];
  ctl=ra+3;
  knots=ra+n+3;

  /*  for(i=0;i<n;i++)
    fprintf(Stderr, "ctl%i=%f\n", i, ctl[i]);

  for(i=0;i<k+n;i++)
    fprintf(Stderr, "Knots%i=%f\n", i, knots[i]);

    fprintf(Stderr,"t=%f\n", x);*/
  res=0.0;
  for(j=0;j<n;j++){
    if(d){
      d[j+1]=blend(j,k,x,knots);
/*        fprintf(Stderr,"d%i=%f\n", j+1, d[j+1]); */
      res+=ctl[j]*d[j+1];
    } else {
      res+=ctl[j]*blend(j,k,x,knots);
    }
  }
/*    fprintf(Stderr,"sp=%f\n", res); */

  if(d){
    /*    fprintf(Stderr, "first derivative\n");*/
    /* bspline first derivative with respect to the t parameter*/
    d[0]=0.0;
    for(i=0;i<n+1;i++)
      d[0]+=alpha(n, k, 2, i, ctl, knots)*blend(i, k-1, x, knots);
  }
/*    fprintf(Stderr,"spd=%f\n", d[0]); */
  if(h){
    /*    fprintf(Stderr, "second derivative\n");*/
    /* bspline second derivative with respect to the t parameter*/
    h[0]=0.0;
    for(i=0;i<n+2;i++)
      h[0]+=alpha(n, k, 3, i, ctl, knots)*blend(i, k-2, x, knots);
  }

  return res;
}



double dbspline(arglist *al)
{
  double x, res;
  double *d, *h, *ra, *ctl, *knots;
  int *at, i, j, k, n;
  AmplExports *ae = al->AE;
    
  at = al->at;
  ra = al->ra;
  d = al->derivs;
  h = al->hes;

  if (al->n < 4){
    fprintf(Stderr, "Invalid number of args to bspline expected >=4 and obtained %d\n", al->n);
    return 0;
  }
  
  x=ra[0]; j=(int)ra[1]; k=(int)ra[2]; n=ra[3];
  ctl=ra+4;
  knots=ra+n+4;

/*    for(i=0;i<n;i++) */
/*      fprintf(Stderr, "ctl%i=%f\n", i, ctl[i]); */

/*    for(i=0;i<k+n;i++) */
/*      fprintf(Stderr, "Knots%i=%f\n", i, knots[i]); */

/*    fprintf(Stderr,"t=%f\n", x); */
  
  res=0.0;
  for(i=0;i<n+j;i++)
    res+=alpha(n, k, j+1, i, ctl, knots)*blend(i, k-j, x, knots);

/*    fprintf(Stderr,"sp=%f\n", res); */

  if(d){
    d[0]=0.0;
    for(i=0;i<n+j+1;i++)
      d[0]+=alpha(n, k, j+2, i, ctl, knots)*blend(i, k-j-1, x, knots);
/*      fprintf(Stderr,"spd=%f\n", d[0]); */

    for(i=0;i<n;i++){
      d[i+1]=dblend(j, i, k, x, knots);
/*        fprintf(Stderr,"d%i=%f\n", i+1, d[i+1]); */
    }
  }

  if(h){
    h[0]=0.0;
    for(i=0;i<n+j+2;i++)
      h[0]+=alpha(n, k, j+3, i, ctl, knots)*blend(i, k-j-2, x, knots);
  }
      
  return res;
}



double alpha(int n, int k, int j, int r, double *ctl, double *knots)
{
  double res;
  double denom;

  if(r<0 || r>n)return 0.0;

  j--;

  if(j>0){
    denom=knots[r+k-j]-knots[r];
    if(denom){
      res=(k-j)*((alpha(n, k, j, r, ctl, knots)-alpha(n, k, j, r-1, ctl, knots))/denom);
    } else {
      res=0;
    }
  } else {
    res=ctl[r];
  }

  return res;
}

double dalpha(int i, int n, int k, int j, int r, double *ctl, double *knots)
{
  double res;
  double denom;

  if(r<0 || r>n)return 0.0;
  if(r+k<j)return 0.0;
  
  j--;

  if(j>0){
    denom=knots[r+k-j]-knots[r];
    if(denom){
      res=(k-j)*((dalpha(i, n, k, j, r, ctl, knots)-dalpha(i, n, k, j, r-1, ctl, knots))/denom);
    } else {
      res=0.0;
    }
  } else {
    if(i==r){
      res=1.0;
    } else {
      res=0.0;
    }
  }

  return res;
}



double blend(int i,int k,double x,double *knots)
{
  double denom1,denom2,temp1,temp2;


  if(k>1){
    denom1=knots[i+k-1]-knots[i];
    if(denom1){
      temp1=(x-knots[i])*blend(i,k-1,x,knots)/denom1;
    } else {
      temp1=0.;
    }
    denom2=knots[i+k]-knots[i+1];
    if(denom2){
      temp2=(knots[i+k]-x)*blend(i+1,k-1,x,knots)/denom2;
    } else {
      temp2=0.;
    }
    return (temp1+temp2);
  } else {
    if(knots[i]<=x && x<knots[i+1]){
      return 1.;
    } else {
      return 0.;
    }
  }
}


double dblend(int j, int i,int k,double x,double *knots)
{
  double denom1, denom2, res;

  if(j<=1){
    denom1=knots[i+k]-knots[i+1];
    denom2=knots[i+k-1]-knots[i];
    res=0;
    if(denom1)
      res+=-blend(i+1, k-1, x, knots)/denom1;
    if(denom2)
      res+=blend(i, k-1, x, knots)/denom2;
    res*=(k-1);
  } else {
    denom1=knots[i+k]-knots[i+1];
    denom2=knots[i+k-1]-knots[i];
    res=0;
    if(denom1)
      res+=-dblend(j-1, i+1, k-1, x, knots)/denom1;
    if(denom2)
      res+=dblend(j-1, i, k-1, x, knots)/denom2;
    res*=(k-1);
  }

return res;
}



void
funcadd(AmplExports *ae){
  /* Insert calls on addfunc here... */
  /* Arg 3, called argtype, can be 0 or 1:
   *	0 ==> force all arguments to be numeric
   *	1 ==> pass both symbolic and numeric arguments.
   *
   * Arg 4, called nargs, is interpretted as follows:
   *	>=  0 ==> the function has exactly nargs arguments
   *	<= -1 ==> the function has >= -(nargs+1) arguments.
   *
   * Arg 5, funcinfo, is passed to the functions in struct arglist;
   *	it is not used in these examples, so we just pass 0.
   */
  
  addfunc("bspline", (ufunc*)bspline, 0, -4, 0);
  addfunc("dbspline", (ufunc*)dbspline, 0, -5, 0);

}

