/*******************************************************************
Copyright (C) 2000 Universidade do Minho

nsips = Nonlinear Semi-Infinite Programming Solver

Ismael Vaz -- aivaz@dps.uminho.pt
********************************************************************/

#include <asl.h>
#include <getstub.h>
#include "sip.h"
#include "nsips.h"



extern void sip_option(void);
extern void sip_solver();
extern struct Option_Info Oinfo;

/* Some definitions */
ASL *asl;
STATS stats;



int main(int argc, char **argv)
{
  char *stub, *s;
  fint m, n, no, nz, mxr, mxc;
  real *X, *T;
  FILE *nl;
  int guess;


  if(argc<=1){
    usage(argc, argv);
    exit(1);
  }

  

  asl=ASL_alloc(ASL_read_pfgh);

  /* set default options before user options */
  sip_option();

  stub=getstops(argv, &Oinfo); /*  argv[1]; */

  want_xpi0=1;

  /*jacpdim(stub, &m, &n, &no, &nz, &mxr, &mxc, (fint)strlen(stub));*/

  /* don't call jacpdim or it will allocate X0 and you won't know if
     there is a initial guess */
  
  nl=jac_dim_ASL(asl, stub, &m, &n, &no, &nz, &mxr, &mxc, (fint)strlen(stub));
  if (!nl)
    printf("Can't read problem\n");

  pfgh_read_ASL(asl, nl, 0);


  if (mxr == 0 || mxc == 0) {
    printf("%s\n%s\n",
	   "Semi-Infinite Programming interface needs .row and .col files.",
	   "In order to provide .row and .col files, in your AMPL session say");
    if ((s = getenv("solver")))
      printf("\toption %s_auxfiles rc;\n%s", s,
	     "before saying\n\tsolve;\nor\n");
    printf("\toption auxfiles rc;\nbefore saying\n\twrite ...\n");
    exit(1);
  }
  

  sip_init(asl);


  /* reset stats */
  stats.nsip_objval=0;
  stats.nsip_objgrd=0;
  stats.nsip_conval=0;
  stats.nsip_jacval=0;
  stats.nsip_pfval=0;
  stats.nsip_gpfval=0;
  stats.nsip_conxval=0;
  stats.nsip_contval=0;
  stats.nsip_conxgrd=0;
  stats.nsip_contgrd=0;
  stats.nnlpc=0;
  stats.nnlp=0;


  X=REAL(nxsip); /*allocate array for starting X*/

  if(X0 && (!havex0 || (havex0 && *havex0))){
    /*    printf("X0\n");
	  printarray(X0,n_var);*/
    sip_extractx(X0, X);
    guess=1;
  } else {
    X0=REAL(n_var); /* necessario para a solucao final */
    X=REAL(nxsip);
    guess=0;
  }

  sip_solver(nxsip, XBU, XBL, ntsip, TBU, TBL,
	     nxsipc, XCBU, XCBL, ntsipc, TCBU, TCBL,
	     X, guess, &sip_eval);

  /* lets write the solution */
  T=REAL(ntsip); /*allocate array for starting X*/
  memset(T, 0, ntsip*sizeof(real));
  sip_joinxt(X,T,X0);
  write_sol("End\n", X0, NULL, NULL);
  
  printf("Some statistics\n");
  printf("nsip_objval=%d\n",stats.nsip_objval);
  printf("nsip_objgrd=%d\n",stats.nsip_objgrd);
  printf("nsip_conval=%d\n",stats.nsip_conval);
  printf("nsip_jacval=%d\n",stats.nsip_jacval);
  printf("nsip_pfval=%d\n",stats.nsip_pfval);
  printf("nsip_gpfval=%d\n",stats.nsip_gpfval);
  printf("nsip_conxval=%d\n",stats.nsip_conxval);
  printf("nsip_contval=%d\n",stats.nsip_contval);
  printf("nsip_conxgrd=%d\n",stats.nsip_conxgrd);
  printf("nsip_contgrd=%d\n",stats.nsip_contgrd);
  printf("nnlpc/nnlp=%f\n",((double)stats.nnlpc)/((double)(stats.nnlp-1)));
  printf("nnlp=%d\n",stats.nnlp);

  sip_free();
  ASL_free(&asl);
  return 0;
}


void usage(int argc, char** argv)
{
  printf("Usage: %s stub -AMPL\n",argv[0]);
}


void printarray(real *a,int n)
{
  int i;

  for(i=0;i<n;i++)
    printf("%.10lf ", a[i]);
  printf("\n");
}

void printmatrix(real *m, int n1, int n2)
{
  int i;

  for(i=0;i<n1;i++)
    printarray(m+n2*i, n2);
}

real sip_eval(int mode, int n, real *X, real *T, real *RX, real *RT, real *RXT,
	      fint *nerror)
{
  real val;

  switch (mode){
  case(SIP_OBJVAL):
    /* only one objective function, for the moment */
    val=sip_objval(0, X, nerror);
    stats.nsip_objval++;
    break;
  case(SIP_OBJGRD):
    sip_objgrd(0, X, RX, nerror);
    val=0;
    stats.nsip_objgrd++;
    break;
  case(SIP_OBJHES):
    sip_objhes(0, RX, nerror);
    val=0;
    stats.nsip_objhes++;
    break;
  case(SIP_CONVAL):
    sip_conval(X, T, RX, RT, nerror);
    val=0;
    stats.nsip_conval++;
    break;
  case(SIP_JACVAL):
    sip_jacval(X, T, RX, RT, RXT, nerror);
    val=0;
    stats.nsip_jacval++;
    break;
  case(SIP_CONXVAL):
    val=sip_conxval(n, X, nerror);
    stats.nsip_conxval++;
    break;
  case(SIP_CONTVAL):
    val=sip_contval(n, X, T, nerror);
    stats.nsip_contval++;
    break;
  case(SIP_CONXGRD):
    sip_conxgrd(n, X, RX, nerror);
    val=0;
    stats.nsip_conxgrd++;
    break;
  case(SIP_CONTGRD):
    sip_contgrd(n, X, T, RX, RT, nerror);
    val=0;
    stats.nsip_contgrd++;
    break;
  case(SIP_CONXHES):
    sip_conxhes(n, RX, nerror);
    val=0;
    stats.nsip_conxhes++;
    break;
  case(SIP_CONTHES):
    sip_conthes(n, RX, RT, RXT, nerror);
    val=0;
    stats.nsip_conthes++;
    break;
  }

  return val;
}

