#include "pfs_dual.h"
#include "int.h"


void penaltyfs_dual()
{
  double f0, f1, dummy, norg;
  int ko, ki, ieps, l, tki, i;
  double miu,  pt;  /* penalty parameter */
  FILE *fp; /* for results file */

#if defined __linux__ || defined linux
  double tsample[ntsip];
  double dir[nxsip+ntsipc];
  double hss[nxsip*nxsip];
  double x0[nxsip], x1[nxsip], g1[nxsip], g0[nxsip], y[nxsip];
  double yeps[nxsip];
  double lag0[ntsipc], lag1[ntsipc], lagy[ntsipc];
  double lg0[ntsipc], lg1[ntsipc], jac[nxsip*ntsipc];
#else
  double *tsample, *dir, *hss, *x0, *x1, *g1, *g0, *y, *yeps;
  double *lag0, *lag1, *lagy, *lg0, *lg1, *jac;
  tsample=mymalloc(ntsip*sizeof(double));
  dir=mymalloc((nxsip+ntsipc)*sizeof(double));
  hss=mymalloc(nxsip*nxsip*sizeof(double));
  x0=mymalloc(nxsip*sizeof(double));
  x1=mymalloc(nxsip*sizeof(double));
  g1=mymalloc(nxsip*sizeof(double));
  g0=mymalloc(nxsip*sizeof(double));
  y=mymalloc(nxsip*sizeof(double));
  yeps=mymalloc(nxsip*sizeof(double));
  lag0=mymalloc(ntsipc*sizeof(double));
  lag1=mymalloc(ntsipc*sizeof(double));
  lagy=mymalloc(ntsipc*sizeof(double));
  lg0=mymalloc(ntsipc*sizeof(double));
  lg1=mymalloc(ntsipc*sizeof(double));
  jac=mymalloc(nxsip*ntsipc*sizeof(double));
#endif



  miu=Sip_Opt.startmiu;
  tki=0;
  ieps=0;

  memcpy(x1, X, nxsip*sizeof(double)); /* x1 with initial guess */
  memcpy(yeps, X, nxsip*sizeof(double));

  /* lagrange multipliers */
  for(l=0;l<ntsipc;l++)
    lag1[l]=1.0;


  for(ko=0;ko<Sip_Opt.maxitero;ko++){


    /* y with initial guess for inner iter */
    memcpy(y, x1, nxsip*sizeof(double));
    memcpy(lagy, lag1, ntsipc*sizeof(double));


    printf("STARTING INNER ITERATIONS ********** miu=%f, epsilon=%f\n",
	   miu, Sip_Opt.pf_epsilon);


    /* Get new penalty value, miu has changed */
    gpenaltyf_dual(nxsip, x1, ntsipc, lag1, ntsip, tsample, miu, g1, lg1,
		   &f1, &pt, jac);
    
    for(ki=0;ki<Sip_Opt.maxiteri;ki++){ /* Quasi-Newton inner iteration */

      /* Set Hessian */
      set_hss(hss, ki, nxsip, x1, x0, g1, g0);
      /*        printf("Hessian\n"); */
      /*        printmatrix(hss, nxsip, nxsip); */

    
      memcpy(x0, x1, nxsip*sizeof(double));
      memcpy(g0, g1, nxsip*sizeof(double));
      memcpy(lag0, lag1, ntsipc*sizeof(double));
      memcpy(lg0, lg1, ntsipc*sizeof(double));
      f0=f1;

      get_dir_dual(nxsip, x0, ntsipc, lag0, hss, jac, g0, lg0, dir);

      printf("X: ");
      printarray(x0, nxsip);
      printf("LAG: ");
      printarray(lag0, ntsipc);
      printf("Penalty %f\n", f0);
      printf("Gradient: ");
      printarray(g0, nxsip);
      printf("LAG Grad: ");
      printarray(lg0, ntsipc);
      printf("Direction: ");
      printarray(dir, nxsip+ntsipc);
      
      if(Sip_Opt.ScaleDir)
	scale_dir(nxsip, dir);

      switch (Sip_Opt.linesearch){
      default:
	printf("Unknown LineSearch\nUsing Armijo\n");
      case ARMIJO:
	armijo_dual(nxsip, x0, ntsipc, lag0, dir, x1, g0, lag1, lg0, miu);
	break;
      };


      gpenaltyf_dual(nxsip, x1, ntsipc, lag1, ntsip, tsample, miu, g1, lg1,
		   &f1, &pt, jac);

      printf("Penalty terms %f %f constraints %f\n", f0, f1, pt);


      /* inner stopping criteria */
      if(pf_i_stop(nxsip, x0, x1, g1, pt)){
	printf("INNER STOPPING IN ITERATION ********* %d\n", ki+1);
	break;
      }
    }

    tki+=ki+1;

    if(pt>Sip_Opt.zero){
      printf("Increasing penalty\n");
      miu*=Sip_Opt.miufactor;
      if(miu>1e+8)
	break;
    } else {
      if(Sip_Opt.pf_type==PF_GXTPLUS2)
	break;
      if(ieps>0){
	if(pf_eps_stop(nxsip, yeps, x1)){
	  break;
	}
      }
      ieps++;
      memcpy(yeps, x1,    nxsip*sizeof(double));
      printf("Decreasing epsilon\n");
      Sip_Opt.pf_epsilon/=10;
      Sip_Opt.pf_tau/=10;
      if(Sip_Opt.pf_epsilon<1e-8)
	break;
    }
    
    
    /* outer stopping criteria */
    /*      if(pf_o_stop(nxsip, y, x1, g1, pt)){ */
    /*        printf("Outer Stopping in iteration %d\n", ko+1); */
    /*        break; */
    /*      } */
  }
  
  memcpy(X, x1, nxsip*sizeof(double));
  printf("Solution found\n");
  printarray(X, nxsip);
  printf("Penalty function value\n");
  printf("%f\n", f1);


  fp=fopen("./results", "a+");
  f0=eval_sip(SIP_OBJVAL, 0, X, NULL, NULL, NULL, NULL, NULL);

  printf("Gradient of penalty:\n");
  printarray(g1, nxsip);

  norg=0.0;
  for(i=0;i<nxsip;i++)
    norg+=g1[i]*g1[i];
  norg=sqrt(norg);

  dummy=eval_sip(SIP_OBJGRD, 0, x1, NULL, g1, NULL, NULL, NULL);

  printf("Gradient of objective:\n");
  printarray(g1, nxsip);
  

/*    fprintf(fp, " & %d & %d & %d & %d & %d & %d & %.4e & %.4e & %.2e\\\\\n", */
/*  	  tki, ko+1, stats.nsip_objval, stats.nsip_objgrd, stats.nsip_contval, */
/*  	  stats.nsip_contgrd, f0, f1, norg); */


  fprintf(fp, " & %d & %d & %d & %d & %.6e & %.1e & %.1e \\\\\n",
	  tki, ko+1, stats.nsip_pfval, stats.nsip_gpfval, f0, miu, Sip_Opt.pf_epsilon);
  fclose(fp);

#if !defined __linux__ && linux
  free(tsample);
  free(dir);
  free(h);
  free(x0);
  free(x1);
  free(g1);
  free(g0);
  free(y);
  free(yeps);
  free(lag0);
  free(lag1);
  free(lagy);
  free(lg0);
  free(lg1);
  free(jac);
#endif

}



/*********************************************************
Evaluate the penalty function
*********************************************************/
double penaltyf_dual(int nx, double *x, int nl, double *lag, int nt, double *t,
		     double miu, double *pt, int dlevel, double *grad,
		     double *lgrad, double *jac)
{
  double pfval, lagnum;
  double num, den, dummy;
  int i, k;

#if defined __linux || defined linux
  double gd[nx], laggd[nx];
#else
  double *gd, *laggd;
  gd=mymalloc(nx*sizeof(double));
  laggd=mymalloc(nx*sizeof(double));
#endif

  stats.nsip_pfval++;

  pfval=0.0;
  *pt=0.0;
  if(dlevel==FIRST_DERIV){
    memset(grad, 0, sizeof(double)*nx);
    memset(lgrad, 0, sizeof(double)*nl);
  }


  for(i=0; i<ntsipc; i++){

    if(Sip_Opt.pf_type!=PF_GXTTEO) /* for Lagrangian */
      adapt_trap(&gxtteo, TBL, TBU, nx, x, nt, t,
		 Sip_Opt.adapt_int_n, i, &lagnum, &den,
		 Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		 dlevel, laggd, miu);
    
    switch (Sip_Opt.pf_type){
    case PF_GXTPLUS:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtplus, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtplus, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTPLUS2:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtplus2, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEPSILON:
      Sip_Opt.pf_epsilon=miu;
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtepsilon, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEXPPLUS:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtexpplus, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtexpplus, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtexpplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtexpplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEXPEPSILON:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtexpepsilon, TBL, TBU, nx, x, nt, t, i,
		 Sip_Opt.adapt_int_n, &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtexpepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtexpepsilon, TBL, TBU, nx, x, nt, t,
		   Sip_Opt.adapt_int_n, i, &num, &den,
		   Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		   dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtexpepsilon, TBL, TBU, nx, x, nt, t,
		    Sip_Opt.adapt_int_n, i, &num, &den,
		    Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		    dlevel, gd, miu);
	break;

      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTTEO:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtteo, TBL, TBU, nx, x, nt, t, i,
		 Sip_Opt.adapt_int_n, &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtteo, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtteo, TBL, TBU, nx, x, nt, t,
		   Sip_Opt.adapt_int_n, i, &num, &den,
		   Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		   dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtteo, TBL, TBU, nx, x, nt, t,
		    Sip_Opt.adapt_int_n, i, &num, &den,
		    Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		    dlevel, gd, miu);
	break;

      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEXPTEO:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtexpteo, TBL, TBU, nx, x, nt, t, i,
		 Sip_Opt.adapt_int_n, &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtexpteo, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtexpteo, TBL, TBU, nx, x, nt, t,
		   Sip_Opt.adapt_int_n, i, &num, &den,
		   Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		   dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtexpteo, TBL, TBU, nx, x, nt, t,
		    Sip_Opt.adapt_int_n, i, &num, &den,
		    Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		    dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTHART:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxthart, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxthart, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxthart, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxthart, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTGROSS:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtgross, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtgross, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtgross, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtgross, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    default:
      printf("Unknown Penalty integral function\n");
    }

    *pt+=num-Sip_Opt.pf_tau;

    /*      cs[i]=num; */

    if(dlevel==FIRST_DERIV && jac!=NULL)
      memcpy(&(jac[i*nx]), gd, sizeof(double)*nx);

    switch (Sip_Opt.pf_type){
    case PF_GXTTEO:
      pfval+=lag[i]*(num-Sip_Opt.pf_epsilon)+0.5*miu*pow(num, 2.0);
      /* -miu*log(lag[i]); */
      if(lgrad)
	lgrad[i]=num-Sip_Opt.pf_epsilon; /*  -(miu/lag[i]); */
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=lag[i]*gd[k]+miu*gd[k]*num;
      }
      break;
    case PF_GXTPLUS2:
      pfval+=lag[i]*(lagnum-Sip_Opt.pf_epsilon)+0.5*miu*num;
      /*  -miu*log(lag[i]); */
      if(lgrad)
	lgrad[i]=lagnum-Sip_Opt.pf_epsilon; /*  -miu*(1/lag[i]); */
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=lag[i]*laggd[k]+miu*gd[k];
      }
      break;
    case PF_GXTEXPTEO:
      pfval+=lag[i]*(lagnum-Sip_Opt.pf_epsilon)+0.5*miu*num*num
	-miu*log(lag[i]);
      if(lgrad)
	lgrad[i]=lagnum-Sip_Opt.pf_epsilon-miu*(1/lag[i]);
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=lag[i]*laggd[k]+miu*num*gd[k];
      }
      break;
    case PF_GXTHART:
    case PF_GXTGROSS:
      pfval+=-lag[i]*(num)+0.5*num/miu;
      if(lgrad)
	lgrad[i]=num;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=-lag[i]*gd[k]+0.5*gd[k]/miu;
/*  	lgrad[i]=num; */
      }
      break;
    default:
      pfval+=-lag[i]*(num)+miu*num;
      if(lgrad)
	lgrad[i]=num;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=-lag[i]*gd[k]+miu*gd[k];
/*  	lgrad[i]=num; */
      }
      break;
    }
  }


  pfval+=eval_sip(SIP_OBJVAL, 0, x, NULL, NULL, NULL, NULL, NULL);
  
  if(dlevel==FIRST_DERIV){
    dummy=eval_sip(SIP_OBJGRD, 0, x, NULL, gd, NULL, NULL, NULL);
    for(k=0;k<nx;k++)
      grad[k]+=gd[k];
  }

#if !defined __linux__ && linux
  free(gd);
  free(laggd);
#endif

  return pfval;
}


/****************************************************
gradient of penalty with respect to x
Arguments are similar of the penalty function
dx   - gradient
dl   - gradient w.r.t. lagrange multipliers
****************************************************/
void gpenaltyf_dual(int nx, double *x, int nl, double *lag, int nt, double *t,
		    double miu, double *dx, double *dl, double *fx,
		    double *pt, double *jac)
{
  double fx1;    /*  , fx2; */
  int i;

#if defined __linux || defined linux
  double x1[nx];
#else
  double *x1;
  x1=mymalloc(nx*sizeof(double));
#endif

  stats.nsip_gpfval++;

  /* dl is always set */
  *fx=penaltyf_dual(nx, x, nl, lag, nt, t, miu, pt, Sip_Opt.derivatives,
		    dx, dl, jac);
  if(Sip_Opt.derivatives==FIRST_DERIV){
    printf("Gradient:\n");
    printarray(dx,nx);
    printf("LAG Gradient:\n");
    printarray(dl,nl);
  }

  if(Sip_Opt.derivatives==NO_DERIV){
    /* forward differences */
    for(i=0; i<nx;i++){
      memcpy(x1, x, nx*sizeof(double));
      x1[i]+=Sip_Opt.fdiff;
      fx1=penaltyf_dual(nx, x1, nl, lag, nt, t, miu, pt, NO_DERIV, NULL, NULL, NULL);
      dx[i]=(fx1- *fx)/(Sip_Opt.fdiff);
    } 
    printf("Gradient with forward differences:\n");
    printarray(dx,nx);
    printf("LAG Gradient with forward differences:\n");
    printarray(dl,nl);
  }
#if !defined __linux__ && linux
  free(x1);
#endif

}


/******************************************************
Set the inverse of the Hessian
Similar to set_h but with lagrange multipliers
******************************************************/
void set_h_dual(double *h, int k, int nx , double *x1, double *x0,
		int nl, double *lag1, double *lag0, double *g1, double *g0,
		double *lg1, double *lg0)
{

#if defined __linux || defined linux
  double xlg1[nx+nl], xlg0[nx+nl], glg1[nx+nl], glg0[nx+nl];
#else
  double *xlg1, *xlg0, *glg1, *glg0;
  xlg1=mymalloc((nx+nl)*sizeof(double));
  xlg0=mymalloc((nx+nl)*sizeof(double));
  glg1=mymalloc((nx+nl)*sizeof(double));
  glg0=mymalloc((nx+nl)*sizeof(double));
#endif



  if((!(k%nxsip) && Sip_Opt.reset_h) || !k){

    identity(h, nx+nl);
/*      printf("Identity Hessian Inverse\n"); */
 
  } else {
    memcpy(xlg1, x1, nx*sizeof(double));
    memcpy(xlg1+nx, lag1, nl*sizeof(double));
    memcpy(xlg0, x0, nx*sizeof(double));
    memcpy(xlg0+nx, lag0, nl*sizeof(double));
    
    memcpy(glg1, g1, nx*sizeof(double));
    memcpy(glg1+nx, lg1, nl*sizeof(double));
    memcpy(glg0, g0, nx*sizeof(double));
    memcpy(glg0+nx, lg0, nl*sizeof(double));
    
    set_h(h, k, nx+nl, xlg1, xlg0, glg1, glg0);
  }
#if !defined __linux__ && linux
  free(xlg1);
  free(xlg0);
  free(glg1);
  free(glg0);
#endif
}


/*****************************************************
Set the Hessian by a BFGS formula
*****************************************************/
void set_hss(double *hss, int k, int nx , double *x1, double *x0,
	     double *g1, double *g0)
{
  int i, j, l;
  double den, teta, aux1, nors, nory;

#if defined __linux || defined linux
  double y[nx], s[nx], mat[nx*nx], auxv[nx];
#else
  double *y, *s, *auxv, *mat;
  y=mymalloc(nx*sizeof(double));
  s=mymalloc(nx*sizeof(double));
  auxv=mymalloc(nx*sizeof(double));
  mat=mymalloc(nx*nx*sizeof(double));
#endif


  if((!(k%nxsip) && Sip_Opt.reset_h) || !k){
    /*    if(1){ */

    identity(hss, nx);
/*      printf("Identity Hessian Inverse\n"); */
    
  } else {

      for(i=0;i<nx;i++){
	y[i]=g1[i]-g0[i];
	s[i]=x1[i]-x0[i];
      }

    if(Sip_Opt.dampedBFGS){
      /* Damped BFGS formula */
/*        printf("Damped BFGS formula\n"); */
      den=0.0;
      for(l=0;l<nx;l++)
	den+=s[l]*y[l];

      for(i=0;i<nx;i++){
	auxv[i]=0.0;
	for(j=0;j<nx;j++)
	  auxv[i]+=s[j]*hss[j*nx+i];
      }

      aux1=0.0;
      for(i=0;i<nx;i++)
	aux1+=auxv[i]*s[i];

      if(den>=0.2*aux1){
	teta=1; /* r=s */
      } else {
	teta=(0.8*aux1)/(aux1-den);
      }

/*        printf("Teta=%f\n", teta); */
      for(i=0;i<nx;i++)
	y[i]=teta*y[i]+(1-teta)*auxv[i];

    }

    den=0.0;
    nors=0.0;
    nory=0.0;
    for(l=0;l<nx;l++){
      den+=s[l]*y[l];
      nors+=s[l]*s[l];
      nory+=y[l]*y[l];
    }

    if(fabs(den)<=Sip_Opt.reseth){
      if(Sip_Opt.scaleBFGS){
	teta=(sqrt(Sip_Opt.reseth))/(sqrt(sqrt(nors)*sqrt(nory)));
	for(l=0;l<nx;l++){
	  s[l]*=teta;
	  y[l]*=teta;
	}
	den=0.0;
	for(l=0;l<nx;l++)
	  den+=s[l]*y[l];

/*  	printf("Hessian scaled by %f\n", teta); */
      } else {
/*  	printf("Not changing Hessian den=%.20f\n", den); */

#if !defined __linux__ && linux
	free(y);
	free(s);
	free(auxv);
	free(mat);
#endif
	return;
      }
    }


    for(i=0;i<nx;i++){
      for(j=0;j<nx;j++)
	mat[i*nx+j]=(y[i]*y[j])/den;
    }

    memset(auxv, 0, nx*sizeof(double));
    for(i=0;i<nx;i++) /* coluna da matriz */
      for(j=0;j<nx;j++)
	auxv[i]+=s[j]*hss[i+j*nx];

    den=0.0;
    for(i=0;i<nx;i++)
      den+=auxv[i]*s[i];


    for(i=0;i<nx;i++)
      for(j=0;j<nx;j++)
	hss[i*nx+j]+=mat[i*nx+j]-(auxv[i]*auxv[j]/den);
    
  }
#if !defined __linux__ && linux
  free(y);
  free(s);
  free(auxv);
  free(mat);
#endif
}





void get_dir_dual(int nx, double *x, int nl, double *lag, double *hss,
		  double *jac, double *grad, double *lgrad, double *dir)
{
  int i, j, sing;

#if defined __linux || defined linux
  double fullh[(nxsip+ntsipc)*(nxsip+ntsipc)];
  double ind[nxsip+ntsipc];
#else
  double *fullh, *ind;
  fullh=mymalloc((nxsip+ntsipc)*(nxsip+ntsipc)*sizeof(double));
  ind=mymalloc((nxsip+ntsipc)*sizeof(double));
#endif

  
  for(i=0;i<nx;i++)
    memcpy(&fullh[i*(nx+nl)], &hss[i*nx], nx*sizeof(double));

  for(i=0;i<nx;i++)
    for(j=0;j<nl;j++)
      fullh[j+nx+i*(nx+nl)]=jac[j*nx+i];

  for(i=0;i<nl;i++){
    memcpy(&fullh[(nx+i)*(nx+nl)], &jac[i*nx], nx*sizeof(double));
    memset(&fullh[(nx+i)*(nx+nl)+nx], 0, nl*sizeof(double));
  }

  for(i=0;i<nx;i++)
    ind[i]=-grad[i];
  for(i=0;i<nl;i++)
    ind[i+nx]=-lgrad[i];

  printf("HSS:\n");
  printmatrix(hss, nx, nx);
  printf("Jac:\n");
  printmatrix(jac, nl, nx);
  printf("Fullh:\n");
  printmatrix(fullh, (nx+nl), (nx+nl));
  printf("Ind:\n");
  printarray(ind, nx+nl);


  egpp(nx+nl, fullh, ind, dir, &sing);


  if(sing){
    printf("Full Hessian is singular\n");
    for(i=0;i<nx;i++)
      dir[i]=-grad[i];
    for(i=0;i<nl;i++)
      dir[i+nx]=-lgrad[i];
  }
#if !defined __linux__ && linux
  free(fullh);
  free(ind);
#endif

}


void armijo_dual(int nx, double *x0, int nl, double *lag0, double *dir,
		 double *x1, double *grad, double *lag1, double *glag,
		 double miu)
{
  int l;
  double alpha, dif, phi0, phiomega, nor, gd, pt;


#if defined __linux || defined linux
  double tsample[ntsip];
#else
  double *tsample;
  tsample=mymalloc(ntsip*sizeof(double));
#endif


  phi0=penaltyf_dual(nx, x0, nl, lag0, ntsip, tsample, miu, &pt,
		     NO_DERIV, NULL, NULL, NULL);
  /*  printf("phi0=%f\n", phi0);*/

  gd=0.0;
  nor=0.0;
  for(l=0;l<nx;l++){
    gd+=grad[l]*dir[l];
    nor+=dir[l]*dir[l];
  }

  /*    for(l=0;l<nl;l++){ */
  /*      gd+=glag[l]*dir[nx+l]; */
  /*      nor+=dir[nx+l]*dir[nx+l]; */
  /*    } */
  
  nor=sqrt(nor);

  alpha=1.0;
  for(l=0;l<nl;l++)
    if(dir[nx+l]<0 && -lag0[l]/dir[nx+l]<alpha)
      alpha=-lag0[l]/dir[nx+l];

  if(alpha<0)
    alpha=0;

  if(alpha!=1)
    alpha=0.95*alpha;


  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    for(l=0;l<nl;l++)
      lag1[l]=lag0[l]+alpha*dir[nx+l];
    phiomega=penaltyf_dual(nx, x1, nl, lag1, ntsip, tsample, miu, &pt,
			   NO_DERIV, NULL, NULL, NULL);
    printf("alpha=%f phiomega=%f\n",alpha, phiomega);

    dif=phi0-phiomega;
    if(dif<-(Sip_Opt.armijo)*alpha*gd){
      alpha=alpha/2;
      if(alpha*nor<=1e-8){
	for(l=0;l<nx;l++)
	  x1[l]=x0[l]+alpha*dir[l];
	for(l=0;l<nl;l++)
  	  lag1[l]=lag0[l]+alpha*dir[nx+l];
	printf("Stepzero %.20f\n", alpha);
#if !defined __linux__ && linux
	free(tsample);
#endif
	return;
      }
    } else {
/*        for(l=0;l<nl;l++) */
/*  	lag1[l]=lag0[l]+alpha*dir[nx+l]; */
      printf("Step=%f\n",alpha);
#if !defined __linux__ && linux
      free(tsample);
#endif
      return;
    }
  }
#if !defined __linux__ && linux
  free(tsample);
#endif

}
