###############################################################################
# Objective: Linear
# Constraints: Generic
###############################################################################
#
#   As described by J.W. Blankenship and J.E. Falk in "Infinitely Constrained
#                 Optimization Problems",
#    Journal of Optimization Theory and Applications (JOTA), Vol. 19, No. 2,
#                 June 1976
#
#   Example 3.1 (seesaw example)
#    Coded by A. Ismael F. Vaz 03/03/2001  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

param pi := 4*atan(1);

# x variable
var x := -(pi/2)+0.05;# very simple and tricky problem if we start near -(pi/2)
# t variable
var t := 0;
# w variable
var w := 1;

minimize fx:
	w;

subject to tcons:
	t*sin(x)-w <= 0;
subject to bounds1:
	-1 <= t <= 1;
subject to bounds2:
	-(pi/2) <= x <= (pi/2);


# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=%lf fx=%lf\n", 0.0, 0.0;
