###############################################################################
# Objective: Quadratic
# Constraints: Linear
###############################################################################
#
#   As described in C.J. Price and I.D. Coope
#    "Numerical Experiments in Semi-infinite Programming",
#    Computational Optimization and Applications 6, 169-189 (1996)
#    Kluwer Academic Publishers.
#
#   Problem M
#    Coded by A. Ismael F. Vaz 11/02/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

# PI
param pi := atan(1)*4;

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	(x[1]-2)^2+x[2]^2;

subject to tcons:
	x[1]*cos(t[1])+x[2]*sin(t[1])-1 <= 0;
subject to xcons:
	max(x[1],x[2])<=1;
subject to bounds:
	0 <= t[1] <= pi;

data;

param nx := 2;
param nt := 1;

# Price starting point
param xinit :=
	1 0
	2 0.1;

param tinit :=
	1 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf) f=%lf\n", 1, 0, 1;
