###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#   As described by M.C. Ferris and A.B. Philpott
#	in "An Interior Point Algorithm for Semi-Infinite
#		  Linear Programming"
#    Mathematical Programming vol. 43, pp. 257-276, 1989
#
#   Problem of section 2.3
#    Coded by A. Ismael F. Vaz 27/02/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars (polynomial degree), results for n=3..8
param nx;
#number of t
param nt;

# initial guess
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];
# distance d
var d;


minimize fx:
	d;

subject to tcons1:
	-(t[1]^nx-(sum {i in 1..nx} (x[i]*t[1]^(i-1))))-d <=0;

subject to tcons2:
	(t[1]^nx-(sum {i in 1..nx} (x[i]*t[1]^(i-1))))-d <=0;

subject to bounds:
	0 <= t[1] <= 1;


data;

param nx := 6;
param nt := 1;

# inital guess
param xinit:=
	1 1
	2 1
	3 1
	4 1
	5 1
	6 1;
#	7 1
#	8 1;

param tinit:=
	1 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
#printf "f=%lf\n", 3.125E-2; # n=3
#printf "f=%lf\n", 7.809E-3; # n=4
#printf "f=%lf\n", 1.95E-3; # n=5
printf "f=%lf\n", 4.88E-4; # n=6
#printf "f=%lf\n", 1.22E-4; # n=7
#printf "f=%lf\n", 3.1E-5; # n=8
