###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#   As described by Stephan Goerner in "Ein Hybridverfahren zur Loesung
#                         nichtlinearer semi-infiniter Optimierungsprobleme",
#   PhD. Dissertation, Berlin University, 1997
#
#   Example 20
#    Coded by A. Ismael F. Vaz 16/01/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	x[7];

subject to tcons1:
	t[1]*exp(t[2])-(x[1]+x[2]*t[1]+x[3]*t[2]+x[4]*t[1]^2+x[5]*t[1]*t[2]+x[6]*t[2]^2)-x[7] <=0;
subject to tcons2:
	-t[1]*exp(t[2])+(x[1]+x[2]*t[1]+x[3]*t[2]+x[4]*t[1]^2+x[5]*t[1]*t[2]+x[6]*t[2]^2)-x[7] <=0;
subject to bounds {i in 1..nt}:
	0 <= t[i] <= 1;

data;

param nx := 7;
param nt := 2;

# watson starting point
param xinit :=
	1 1
	2 1
	3 1
	4 1
	5 1
	6 1
	7 1;

param tinit :=
	1 0
	2 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf,%lf,%lf,%lf,%lf,%lf) f=%lf\n", 0.053324, 0.943862,
	 -0.426591, -0.050510, 1.718282, 0.426591, 0.053324, 0.053324;
