###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#  Fractional semi-infinite programming problem
#  As described by Martin Gugat, PhD Thesis,
#   submitted to Trier University, "Fractional Semi-Infinite Programming", 1994
#
#   Example 8
#    Coded by A. Ismael F. Vaz 19/03/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of z vars
param nz := 8;

# the gamma parameter
param gamma := 10^5;

# z variables
var z {1..nz};
# infinite variable
var t;
# d distance
var d;


# The functions
var f = t;
var v = z[5]*t^32+z[4]*t^24+z[3]*t^16+z[2]*t^8+z[1];
var w = z[6]*t^16+z[7]*t^8+z[8];



minimize dp:
	d;

subject to tcons1:
	-(f-v/w)-d <= 0;
subject to tcons2:
	(f-v/w)-d <= 0;
subject to tcons3:
	-w <= -1;
subject to tcons4:
	w <= gamma;
subject to bounds:
	0 <= t <= 1;


###################
# End of Problem  #
###################


# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display d;
display z;

################################################
# solution of problem presented by the authors #
################################################

# Gamma=10^5
printf "Original Solution\n";
printf "%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf\n", 0.78691117e-1, 0.22620654e5,
	0.21584729e6, -0.37423581e6, 0.2436369e6, 0.50755895e5, 0.49243105e5,
	0.1e1;
