###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#  Fractional semi-infinite programming problem
#  As described by Martin Gugat, PhD Thesis,
#   submitted to Trier University, "Fractional Semi-Infinite Programming", 1994
#
#   Example 9 Function F
#    Coded by A. Ismael F. Vaz 19/03/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of z vars
param nz := 8;

# the gamma parameter
param gamma := 10^3;

# z variables
var z {1..nz};
# infinite variable
var t;
# d distance
var d;


# The functions
var f = cos(t/2);
var v = z[5]*t^4+z[4]*t^3+z[3]*t^2+z[2]*t+z[1];
var w = z[6]*t^2+z[7]*t+z[8];



minimize dp:
	d;

subject to tcons1:
	-(f-v/w)-d <= 0;
subject to tcons2:
	(f-v/w)-d <= 0;
subject to tcons3:
	-w <= -1;
subject to tcons4:
	w <= gamma;
subject to bounds:
	0 <= t <= 3;


###################
# End of Problem  #
###################


# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display d;
display z;

################################################
# solution of problem presented by the authors #
################################################

# Gamma=10^3
printf "Original Solution\n";
printf "%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf\n", 0.10035455e1, -0.12050977e-1,
	-0.11513578, 0.15909333e-2, 0.12455960e-2, 0.10351184e-1,
	-0.12059466e-1, 0.10035458e1;
