###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#  Fractional semi-infinite programming problem
#  As described by Martin Gugat, PhD Thesis,
#   submitted to Trier University, "Fractional Semi-Infinite Programming", 1994
#
#   Example 10 Function E
#    Coded by A. Ismael F. Vaz 19/03/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of z vars
param nz := 6;

# the gamma parameter
param gamma := 10^3;

# z variables
var z {1..nz};
# infinite variable
var t;
# d distance
var d;


# The functions
var f = if t<=1 then t else 0.5*t+0.4;
var v = z[3]*t^2+z[2]*t+z[1];
var w = z[6]*t^2+z[5]*t+z[4];



minimize dp:
	d;

subject to tcons1:
	-(f-v/w)-d <= 0;
subject to tcons2:
	(f-v/w)-d <= 0;
subject to tcons3:
	-w <= -1;
subject to tcons4:
	w <= gamma;
subject to bounds:
	0 <= t <= 2;


###################
# End of Problem  #
###################


# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# problems with diferentiability in t=1
#option nsips_options "method=disc_hett disc_h=0.125 disc_k=5";
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display d;
display z;

################################################
# solution of problem presented by the authors #
################################################

# Gamma=10^3
printf "Original Solution\n";
printf "%lf,%lf,%lf,%lf,%lf,%lf\n", 0.41151422, -0.27005298e2, 0.34984090e3,
	0.75875764e1, 0.17218642e3, 0.16200985e3;
