###############################################################################
# Objective: Linear
# Constraints: Quadratic
###############################################################################
#
#  Hettich set of SIP problems (Chebyshev approximation problems)
#   As described by Craig T. Lawrence and Andre L. Tits, "Feasible Sequential
#              Quadratic Programming for Finely Discretized Problems from SIP",
#   in R. Reemtsen and Jan-J. Ruckmann (eds.), "Semi-infinite Programming",
#    pp. 159-193, Kluwer Academic Publishers, 1998
#
#   Example hz_1, pp. 183
#    Coded by A. Ismael F. Vaz 20/05/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of t
param nt := 1;
#number of x
param nx := 2;

# t variable
var t {1..nt};
# x variables
var x {1..nx};

minimize fx:
	x[2];

subject to tcons1:
	-(1-t[1]^2)+(0.5*x[1]^2-2*x[1]*t[1])-x[2] <=0;
subject to tcons2:
	 (1-t[1]^2)+(0.5*x[1]^2-2*x[1]*t[1])-x[2] <=0;
subject to bounds {i in 1..nt}:
	-1 <= t[i] <= 1;


###################
# End of Problem  #
###################

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;
