###############################################################################
# Objective: Linear
# Constraints: Quadratic
###############################################################################
#
#  Hettich set of SIP problems (Chebyshev approximation problems)
#   As described by R. Hettich, "A comparison of some numerical methods
#                                for semi-infinite programming",
#   in R. Hettich (eds.), "Semi-infinite Programming",
#    Lecture notes in Control and Information Sciences no. 15, pp. 112-125
#    Springer Verlag, Berlin (1979)
#
#   Example 3
#    Coded by A. Ismael F. Vaz 27/12/1999  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of p vars
param np := 3;
#number of t
param nt := 1;
# sigma can be -1 or 1
param sigma := 1;

# p variables
var p {0..np};
# t variable
var t {1..nt};
# d distance
var d;

minimize dp:
	d;

subject to tcons1:
	-(sqrt(t[1])-sigma*((p[0]*t[1]^2+p[1]*t[1]+p[2])^2+p[3]))-d <=0;
subject to tcons2:
	(sqrt(t[1])-sigma*((p[0]*t[1]^2+p[1]*t[1]+p[2])^2+p[3]))-d <=0;
subject to bounds {i in 1..nt}:
	0.25 <= t[i] <= 1;


###################
# End of Problem  #
###################

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display p;
display dp;
