###############################################################################
# Objective: Linear (for lambda!=0)
# Constraints: Linear
###############################################################################
#
#  Hettich set of SIP problems (Chebyshev approximation problems)
#   As described by R. Hettich, "A note on an implementation of a method for
#                                quadratic  Semi-Infinite Programming",
#   in Mathematical Programming 46(1990) 249-254
#
#   Example 1
#    Coded by A. Ismael F. Vaz 30/12/1999  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

# degree of polynomial
param d := 3;

#number of t
param nt := 1;

#lamdba on objective function
param lambda := 0.1;

# x variables
# extra variables for ex. x[0] are eliminated by presolver
var x {1..d+1};
# t variable
var t {1..nt};
# distance variable
var dist;

minimize distance:
	dist+lambda*(sum {i in 3..d+1} ((i-1)*x[i]^2));
subject to tcons1:
	-((sum {i in 1..d+1} (x[i]*t[1]^(i-1)))-sin(t[1]))-dist <= 0;
subject to tcons2:
	((sum {i in 1..d+1} (x[i]*t[1]^(i-1)))-sin(t[1]))-dist <= 0;
subject to bound1:
	0 <= t[1] <= 1;
#subject to xbound {i in 1..d+1}:
#	x[i]>=0;
	


###################
# End of Problem  #
###################

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display dist;

################################################
# solution of problem presented by the authors #
################################################
printf "\nWarning!!!!!!\nThis problem is not the original one.\nAn error in the original paper??\n\n";

