###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  As described by K.O. Kortanek and H. No in
#      "A central cutting plane algorithm for convex semi-infinite
#               programming problems"
#    SIAM Journal on Optimization, vol. 3, no. 4, pp. 901-918, 1993
#
#   Example 3
#
#    Coded by A. Ismael F. Vaz 03/06/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of x vars
param nx := 2;
#number of t vars
param nt := 2;

# x variables
var x {1..nx};
# t variable
var t {1..nt};

minimize obj:
	x[1]^2+x[2]^2;

subject to tcons:
	((x[1]-2)^2+(x[2]-2)^2-4)*t[1]+(x[1]^2+x[2]^2-4)*t[2] <= 0;
subject to xbound {i in 1..nx}:
	0 <= x[i] <= 2;
subject to bounds {i in 1..nt}:
	0 <= t[i] <= 1;


###################
# End of Problem  #
###################

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display x;
display obj;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf, %lf) obj=%lf\n", 0.585786438, 0.585786438, 0.686291501;
# Dual solution, l=6.62741700 when t=(0.0625, 0)
printf "Exact solution x=(2-sqrt(2), 2-sqrt(2))\n";
