###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  As described by T. Leon, S. Sanmatias and E. Vercher,
#      "On the numerical treatment of linearly constrained semi-infinite
#                                optimization problems"
#    European Journal of Operational Research 121 (2000) 78-91
#
#   Example LBV 1 and LBV 2, section 5.2
#
#    Coded by A. Ismael F. Vaz 04/03/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of x vars
param nx;
#number of t vars
param nt;
# initial guess
param x0 {0..nx-1};
param t0 {1..nt};

# x variables
var x {i in 0..nx-1} := x0[i];
# t variable
var t {i in 1..nt} := t0[i];

minimize obj:
	x[0];

subject to tcons1:
	t[1]-(sum {i in 1..nx-1} ((2*i*(2*i-1)*t[1]^(i-1)+(1+t[1])*(1-t[1]^i))*x[i])) <= 0;
subject to tcons2:
	(sum {i in 1..nx-1} ((2*i*(2*i-1)*t[1]^(i-1)+(1+t[1])*(1-t[1]^i))*x[i]))-t[1]-x[0] <= 0;
subject to bounds {i in 1..nt}:
	0 <= t[i] <= 1;


###################
# End of Problem  #
###################

data;

# nx := 3 for LBV 1
# nx := 4 for LBV 2
param nx := 3;
param nt := 1;

param x0 :=
	0	3
	1	1
	2	0;
#	3	0;

param t0 :=
	1	0;

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display x;
display obj;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "(LBV 1) obj=%lf\n", 0.0484144;
#printf "(LBV 2) obj=%lf\n", 0.00564715;
