###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  As described by T. Leon, S. Sanmatias and E. Vercher,
#      "On the numerical treatment of linearly constrained semi-infinite
#                                optimization problems"
#    European Journal of Operational Research 121 (2000) 78-91
#
#   Example PT, Appendix A
#
#    Coded by A. Ismael F. Vaz 04/03/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of x vars
param nx;
#number of t vars
param nt;
# initial guess
param x0 {1..nx};
param t0 {1..nt};

# x variables
var x {i in 1..nx} := x0[i];
# t variable
var t {i in 1..nt} := t0[i];

minimize obj:
	x[1];

subject to tcons1:
	t[1]-t[1]^2-x[1]-(1-t[1]-t[1]^2)*x[2]<=0;
subject to bounds {i in 1..nt}:
	0 <= t[i] <= 1;


###################
# End of Problem  #
###################

data;

param nx := 2;
param nt := 1;

param x0 :=
	1	2.5
	2	0;

param t0 :=
	1	0;

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display x;
display obj;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf) obj=%lf\n", sqrt(5)-2, 1-2*sqrt(5)/5, sqrt(5)-2;

