###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  As described by T. Leon, S. Sanmatias and E. Vercher,
#      "On the numerical treatment of linearly constrained semi-infinite
#                                optimization problems"
#    European Journal of Operational Research 121 (2000) 78-91
#
#   Example GU3, Appendix A
#
#    Coded by A. Ismael F. Vaz 04/03/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of x vars
param nx;
#number of t vars
param nt;
# initial guess
param x0 {1..nx};
param t0 {1..nt};

param epson := 1.5E-6;

# x variables
var x {i in 1..nx} := x0[i];
# t variable
var t {i in 1..nt} := t0[i];

minimize obj:
	x[1]+(1/2)*x[2]+((1/2)+epson/3)*x[3];

subject to tcons1:
	exp(t[1])-x[1]-t[1]*x[2]-(t[1]+epson*t[1]^2)*x[3]<=0;
subject to bounds {i in 1..nt}:
	0 <= t[i] <= 1;


###################
# End of Problem  #
###################

data;

param nx := 3;
param nt := 1;

param x0 :=
	1	2.71828182845904523536 # exp(1)
	2	0
	3	0;

param t0 :=
	1	0;

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display x;
display obj;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
# if epson != 0
printf "x=(%lf,%lf,%lf) obj=%lf\n", 1.027996,-587431.709682, 587432.518819, (1/4)*(3*exp(1/3)+exp(1));
# if epson = 0
#printf "obj=%lf\n", (1/2)*(1+exp(1));
