###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  As described by T. Leon, S. Sanmatias and E. Vercher,
#      "On the numerical treatment of linearly constrained semi-infinite
#                                optimization problems"
#    European Journal of Operational Research 121 (2000) 78-91
#
#   Example 1, section 3.2
#
#    Coded by A. Ismael F. Vaz 28/03/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of x vars
param nx;
#number of t vars
param nt;
# initial guess
param x0 {1..nx};
param t0 {1..nt};

# x variables
var x {i in 1..nx} := x0[i];
# t variable
var t {i in 1..nt} := t0[i];

minimize obj:
	(-3/4)*x[1];

subject to tcons1:
	(3-7*t[1])*x[1]-4*t[1]*x[2]<=4*t[1]^2-11*t[1]+7;

subject to bounds {i in 1..nt}:
	0 <= t[i] <= 1;


###################
# End of Problem  #
###################

data;

param nx := 2;
param nt := 1;

param x0 :=
	1	2.5
	2	-2.5;

param t0 :=
	1	0;

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display x;
display obj;
