###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  As described by T. Leon, S. Sanmatias and E. Vercher,
#      "On the numerical treatment of linearly constrained semi-infinite
#                                optimization problems"
#    European Journal of Operational Research 121 (2000) 78-91
#
#   Example LCA 7, section 5.2
#    Coded by A. Ismael F. Vaz 04/03/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of x vars
param nx;
#number of t vars
param nt;
# initial guess
param x0 {0..nx-1};
param t0 {1..nt};

# x variables
var x {i in 0..nx-1} := x0[i];
# t variable
var t {i in 1..nt} := t0[i];

minimize obj:
	x[0];

subject to tcons1:
	-(x[1]+x[2]*t[1]+x[3]*t[1]^2+x[4]*t[1]^3+x[5]*sin(t[1])+x[6]*cos(t[1]))+(1-exp(-t[1]^2))-x[0] <=0;
subject to tcons2:
	(x[1]+x[2]*t[1]+x[3]*t[1]^2+x[4]*t[1]^3+x[5]*sin(t[1])+x[6]*cos(t[1]))-(1-exp(-t[1]^2))-x[0] <=0;
subject to bounds {i in 1..nt}:
	-2 <= t[i] <= 2;


###################
# End of Problem  #
###################

data;

param nx := 7;
param nt := 1;

param x0 :=
	0	0.98168436111126581971 # 1-exp(-(2)^2)
	1	0
	2	0
	3	0
	4	0
	5	0
	6	0;

param t0 :=
	1	0;

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display x;
display obj;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "obj=%lf\n", 0.0542221;
