###############################################################################
# Objective: Quadratic
# Constraints: Linear
###############################################################################
#
#   As described by Y. Liu, K.L. Teo and S. Ito
#	in "A Dual Parametrization Approach to Linear-Quadratic
#                 Semi-Infinite Programming Problems",
#    Optimization Methods and Software, vol. 10, pp. 471-495
#
#   Example 6.2
#    Coded by A. Ismael F. Vaz 11/02/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

# PI
param pi := atan(1)*4;

# number of x vars
param nx;
#number of t
param nt;

# some extra data
param Q {1..2,1..2};


# x variables
var x {i in 1..nx};
# t variable
var t {j in 1..nt};

minimize fx:
	(Q[1,1]*x[1]^2+Q[2,2]*x[2]^2+(Q[1,2]+Q[2,1])*x[1]*x[2])-2*x[1]-4*x[2];

subject to tcons:
	cos(t[1])*x[1]+sin(t[1])*x[2] -1 <= 0;
subject to bounds:
	0 <= t[1] <= pi/2;

data;

param nx := 2;
param nt := 1;

param Q:
	1	2:=
1	2	-1
2	-1	2;



# solve problem
# no initial guess
option reset_initial_guesses 1;
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf) f=%lf\n", 0.57638, 0.81718, -3.36350;
