###############################################################################
# Objective: Linear
# Constraints: Quadratic
###############################################################################
#
#  Polak Generalized Semi-Infinite Min-Max Problem
#   As described by Polak, "First-Order Algorithms for Generalized
#                                Semi-Infinite Min-Max Problems"
#   in Computational Optimization and Applications, 13, 137-161(1999)
#
#   Example 1
#    Coded by A. Ismael F. Vaz 12/04/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of t
param nt := 2; #there are one infinite constraint for each constraint
#number of x
param nx := 2;
#number of z
param nz := 2;

# x variable
var x {1..nx};
# t variable
var t {1..nt};
# z variable
var z {1..nz};

minimize dp:
	z[1]+z[2];

subject to tcons1:
	(t[1]^2-(t[1]*x[1]+exp(t[1])*x[2])+(x[1]+x[2])^2+x[1]^2+x[2]^2)
	   -z[1]<=0;
subject to tcons2:
	((t[2]-1)^2+0.5*(x[1]+x[2])^2-2*t[2]*(x[1]+x[2])+0.5*(x[1]^2+x[2]^2))
	   -z[2]<=0;
subject to bound1:
	0 <= t[1] <= 2;
subject to bound2:
	-1 <= t[2] <= 1;


###################
# End of Problem  #
###################

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display z;
display dp;
