###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#  Design of FIR filters
#   As described by A.W. Potchinkov
#	"Design of optimal linear phase FIR filters by a semi-infinite
#	    programming technique"
#	 in Signal Processing 58 (1997) 165-180
#	
#   Design 1
#    Coded by A. Ismael F. Vaz 04/07/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################
param pi := atan(1)*4;
#param N := 2001; # original value
param N := 593;   # the student edition limit
param R := (N-1)/2;

var x {0..R}; # coefficients
var d;        # delta (distance)
var t{1..2};  # infinite variable w (one for Bp and other for Bs)


# D(w) and W(w) for Bp and Bs, respectively
param D1 := 1;
param D2 := 0;
param W1 := 1;
param W2 := 10;



minimize dp:
	d;

subject to t1:
	-W1*(sum {n in 0..R} (x[n]*cos(t[1]*n)))+D1*W1-d <=0;

subject to t2:
	-W2*(sum {n in 0..R} (x[n]*cos(t[2]*n)))+D2*W2-d <=0;

subject to t3:
	W1*(sum {n in 0..R} (x[n]*cos(t[1]*n)))-D1*W1-d <=0;

subject to t4:
	W2*(sum {n in 0..R} (x[n]*cos(t[2]*n)))-D2*W2-d <=0;

subject to bound1:
	0 <= t[1] <=0.2*pi;

subject to bound2:
	0.204*pi <= t[2] <=pi;


# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
#option reset_initial_guesses 1; # don't want a initial guess
option solver nsips;
solve;



display dp;

# Solution in MATLAB language
printf "x=\[";
for {i in 0..(R-1)} {
	printf "%f, ", x[i];
};
printf "%f\];\n", x[R];
