###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#  Design of FIR filters
#   As described by A.W. Potchinkov
#	"Design of optimal linear phase FIR filters by a semi-infinite
#	    programming technique"
#	 in Signal Processing 58 (1997) 165-180
#	
#   Design 4a
#    Coded by A. Ismael F. Vaz 06/07/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################
param pi := atan(1)*4;
param N := 129;
param R := (N-1)/2;

var x {0..(R+1)}; # coefficients
var d;            # delta (distance)
var t{1..1};      # infinite variable w (one for Bs)


# D(w) and W(w) for Bs
param D1 := 0;
param W1 := 1;

var lambda = (sum {i in 0..(R+1)} (x[i]*cos(t[1]*i)));

var alpha {k in 1..18} = (sum {n in 0..(R+1)} (x[n]*((-1)^k)*(n^(2*k))));
var alpha19            = (sum {n in 0..(R+1)} (x[n]));

minimize dp:
	d;

subject to t1:
	-W1*lambda-d <=0;
subject to t2:
	 W1*lambda-d <=0;

subject to c1 {k in 1..18}:
	alpha[k] <= 0;
subject to c2:
	alpha19+1 <= 0;
	

subject to bound:
	0.24*pi <= t[1] <= pi;



# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
#option reset_initial_guesses 1; # don't want a initial guess
option solver nsips;
solve;



display dp;

# Solution in MATLAB language
printf "x=\[";
for {i in 0..R} {
	printf "%f, ", x[i];
};
printf "%f\];\n", x[R+1];
