###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#  Design of FIR filters
#   As described by A.W. Potchinkov
#	"Design of optimal linear phase FIR filters by a semi-infinite
#	    programming technique"
#	 in Signal Processing 58 (1997) 165-180
#	
#   Design 4b
#    Coded by A. Ismael F. Vaz 06/07/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################
param pi := atan(1)*4;
param N := 129;
param R := (N-1)/2;

var x {0..R}; # coefficients
var d;            # delta (distance)
var t{1..1};      # infinite variable w (one for Bs)


param A {p in 0..R, q in 0..R} := if(p+q=0)	then 0.76*pi	else
				  if(p-q=0)	then (1/(2*(p+q)))*(sin((p+q)*pi)-sin((p+q)*0.24*pi))+0.5*0.76*pi 				       else
				  (1/(2*(p+q)))*(sin((p+q)*pi)-sin((p+q)*0.24*pi))+(1/(2*(p-q)))*(sin((p-q)*pi)-sin((p-q)*0.24*pi));


var alpha {k in 1..18} = (sum {n in 0..R} (x[n]*((-1)^k)*(n^(2*k))));
var alpha19            = (sum {n in 0..R} (x[n]));

minimize obj:
	(sum {i in 0..R, j in 0..R} (x[i]*x[j]*A[i,j])); #matrix A is symmetric

subject to t1:
	x[0]*sin(t[1]) <= x[0];

subject to c1 {k in 1..18}:
	alpha[k] = 0;
subject to c2:
	alpha19+1 = 0;
	

subject to bound:
	0.24*pi <= t[1] <= pi;



# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
#option reset_initial_guesses 1; # don't want a initial guess
option solver nsips;
solve;



display obj;

# Solution in MATLAB language
printf "x=\[";
for {i in 0..(R-1)} {
	printf "%f, ", x[i];
};
printf "%f\];\n", x[R];
