###############################################################################
# Objective: Quadratic
# Constraints: Linear
###############################################################################
#
#  Price PhD. Thesis
#   As described in C.J. Price 
#    "Nonlinear Semi-infinite Programming",
#    PhD. Thesis submited to University of Canterbury,
#    New Zealand, August 1992.
#
#   Problem K
#    Coded by A. Ismael F. Vaz 24/12/1999  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

# PI
param pi := atan(1)*4;

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	x[2]^2-4*x[2];

subject to tcons:
	x[1]*cos(t[1])+x[2]*sin(t[1])-1 <= 0;
subject to bounds:
	0 <= t[1] <= pi;

data;

param nx := 2;
param nt := 1;

# Price starting point
param xinit :=
	1 0.9
	2 0;

param tinit :=
	1 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf) f=%lf\n", 0.0, 1.0, -3.0;
