###############################################################################
# Objective: Quadratic
# Constraints: Generic
###############################################################################
#
#  Price PhD. Thesis
#   As described in C.J. Price 
#    "Nonlinear Semi-infinite Programming",
#    PhD. Thesis submited to University of Canterbury,
#    New Zealand, August 1992.
#
#   Problem T
#    Coded by A. Ismael F. Vaz 24/12/1999  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	sum {i in 1..4} (x[i]^2-x[i]);

subject to tcons:
	-sum {i in 1..4} (x[i]^2)+
	  (1/(1+sum{j in 1..nt} ((t[j]-x[1])^2)))+
	  (1/(1+sum{j in 1..nt} ((t[j]-x[2]*(-1)^j)^2)))+
	  (1/(1+sum{j in 1..nt} ((t[j]-x[3]*(-1)^(j div 2))^2)))+
	  (1/(1+sum{j in 1..nt} ((t[j]-x[4]*(-1)^((j+1) div 2))^2))) <= 0;
subject to bounds {i in 1..nt}:
	-3 <= t[i] <= 3;

data;

param nx := 4;
# can have several nt values
param nt := 3;

# Price starting point
param xinit :=
#	1 0.5
#	2 0.5
#	3 0.5
#	4 0.5;
	1 -2.25
	2 -2.5
	3 -2.75
	4 -3.0;

param tinit :=
	1 0
	2 0
	3 0;
#	4 0
#	5 0
#	6 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
# solution for nt=3
printf "Original Solution\n";
printf "x=(%lf,%lf,%lf,%lf) f=%lf\n", 0.659449, 0.659446, 0.659446, 0.659441, -0.898308;
# solution for nt=4
#printf "Original Solution\n";
#printf "x=(%lf,%lf,%lf,%lf) f=%lf\n", 0.659442, 0.659450, 0.659448, 0.659443, -0.898308;
# solution for nt=5
#printf "Original Solution\n";
#printf "x=(%lf,%lf,%lf,%lf) f=%lf\n", 0.636215, 0.636215, 0.636216, 0.636215, -0.925782;
# solution for nt=6
#printf "Original Solution\n";
#printf "x=(%lf,%lf,%lf,%lf) f=%lf\n", 0.617580, 0.617580, 0.617579, 0.617580, -0.944700;


