###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  Reemtsen set of SIP problems (Chebyshev approximation problems)
#   As described by R. Reemtsen, "Discretization Methods for the Solution of
#                                 Semi-Infinite Programming Problems",
#   in Journal of Optimization Theory and Applications, vol. 71, No. 1, 1991
#
#   Example 4.8
#    Coded by A. Ismael F. Vaz 27/12/1999  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

# degree of polynomial d=2..3 was tested by Reemtsen
param d := 2;

#number of t
param nt := 3;

# x variables
var x {0..d,0..d,0..d};
# t variable
var t {1..nt};
# distance variable
var dist;

minimize distance:
	dist;

subject to tcons1:
	-(sum {i in 0..d} (
            sum {j in 0..d-i} (
	      sum {k in 0..d-i-j} (
		x[i,j,k]*(t[1]^i)*(t[2]^j)*(t[3]^k) )))
          -(abs(log((t[1]*t[2]+1)/(t[1]+0.5)))*t[2]^((t[3]+1)/2)))-dist <= 0;
subject to tcons2:
	(sum {i in 0..d} (
            sum {j in 0..d-i} (
	      sum {k in 0..d-i-j} (
		x[i,j,k]*(t[1]^(i))*(t[2]^j)*(t[3]^k) )))
          -(abs(log((t[1]*t[2]+1)/(t[1]+0.5)))*t[2]^((t[3]+1)/2)))-dist <= 0;
subject to bound {i in 1..nt}:
	1e-20 <= t[i] <= 1;


###################
# End of Problem  #
###################

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display distance;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
# d=2
printf "distance=%lf\n", 8.893175e-2;
# d=3
#printf "distance=%lf\n", 4.811702e-2;
