###############################################################################
# Objective: Generic
# Constraints: Generic
###############################################################################
#
#  PID controller design problem,
#   As described by K.L. Teo and C.J. Goh in "A Simple Computational Procedure
#            for Optimization Problems with Functional Inequality Constraints",
#   in IEEE Transactions on Automatic Control,VOL. AC-32,No 10,pp. 940-941,1987
#   Originally proposed by C. Gonzaga, E. Polak and R. Trahan in "An Improved
#            Algorithm for Optimization Problems with Functional Inequality
#            Constraints"
#   in IEEE Transactions on Automatic Control, VOL AC-25, No 1,pp. 49-54, 1980
#
#   Example 1,
###############################################################################
#   THE OBJECTIVE FUNCTION WAS WRONGLY CODED IN TEO PAPER.
#   The objective function coded is the one in Gonzaga paper
#   See a more recent paper by K.L. Teo and V.Rehbock and L.S. Jennings in
#    "A New Computational Algorithm for Functional Inequality Constrained
#     Optimization Problems", Automatica, VOL 29, No 3,pp. 789-792, 1993
###############################################################################
#    Coded by A. Ismael F. Vaz 12/12/2000  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx := 3;
#number of t vars
param nt := 1;

# initial guess
param xinit {1..nx};
param tinit {1..nx};

# p variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {i in 1..nt} := tinit[i];


# auxiliary vars
var ts=t[1]+1e-6; # place a shift to avoid division by zero

var ImT = (-x[1]*(8*ts-ts^3)+(x[3]*ts-(x[2]/ts))*(6-5*ts^2))/
	((8*ts-ts^3)^2+(6-5*ts^2)^2);

var ReT = 1 + (x[1]*(6-5*ts^2)+(8*ts-ts^3)*(x[3]*ts-(x[2]/ts)))/
	((8*ts-ts^3)^2+(6-5*ts^2)^2);


minimize fx:
	(x[2]*(122+17*x[1]+6*x[3]-5*x[2]+x[1]*x[3])+180*x[3]-36*x[1]+1224)/
	(x[2]*(408+56*x[1]-50*x[2]+60*x[3]+10*x[1]*x[3]-2*x[1]^2));

subject to tconst:
	ImT-ReT^2+0.2 <= 0;

subject to xbounds1:
	0   <= x[1] <= 100;
subject to xbounds2:
	0.1 <= x[2] <= 100;
subject to xbounds3:
	0   <= x[3] <= 100;

subject to bounds {i in 1..nt}:
	0 <= t[i] <= 30-1e-6;

data;
param xinit :=
	1	1
	2	1
	3	1;

param tinit :=
	1	1e-6;

###################
# End of Problem  #
###################

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf,%lf) fx=%lf\n", 30.855, 100.0, 100.0, 0.12507;
