###############################################################################
# Objective: Generic
# Constraints: Generic
###############################################################################
#
#  Watson set of SIP problems
#   As described in Coope, I.D. and G.A. Watson
#    "A Projected Lagrangian Algoritm for Semi-Infinite Programming",
#    Math. Prog., vol. 32, pp 337-356(1985)
#
#   Example 14
#    Coded by A. Ismael F. Vaz 24/12/1999  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# this objective funcion has an extra parameter c
param c;

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	c^2*exp(x[1])+exp(x[2]);

subject to tcons:
	t[1]-exp(x[1]+x[2]) <= 0;

subject to bounds:
	0 <= t[1] <= 1;

data;

param nx := 2;
param nt := 1;

# has Coope and Watson used in the numerical experiments
param c := 1.1;

# starting point
param xinit :=
	1 0.8
	2 0.9;

param tinit :=
	1 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf) f=%lf\n", -log(abs(c)), log(abs(c)), 2*abs(c);
