###############################################################################
# Objective: Linear
# Constraints: Polynomial
###############################################################################
#  Watson set of SIP problems
#   As described by G.A. Watson in "Numerical Experiments with Globally
#                 Convergent Methods for Semi-Infinite Programming Problems",
#   Edited in Fiacco, A.V., and K.O. Kortanek (eds.),
#    "Semi-infinite Programming and Applications",
#    Lecture notes in Economics and Mathematical Systems no. 215, pp.193-205
#    Springer Verlag, Berlin (1983)
#
#   Example 4 with n=6
#    Coded by A. Ismael F. Vaz 24/12/1999  aivaz@ci.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	sum{i in 1..nx} (x[i]/i);

subject to tcons:
	tan(t[1])- sum{i in 1..nx} (x[i]*t[1]^(i-1)) <= 0;
subject to bounds:
	0 <= t[1] <= 1;

data;

# nx can be 3, 6, 8
param nx := 6;
param nt := 1;

# watson starting point
# starting point for n=6
param xinit :=
	1 0
	2 0
	3 0
	4 0
	5 0
	6 0;

param tinit :=
	1 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf,%lf,%lf,%lf,%lf) f=%lf\n", 0.002084, 0.963374, 0.203505, -0.026589, 0.082262, 0.332771, 0.616876;
