/***************************************************************
 * Select tool - help to find problems in the SIPAMPL database
 *   with specified options
 *
 * function:
 * main  printoptions  printtype  printbound_var  printbound
 * printinitial chgoptioninitial
 * chgoptiontype  chgoptionbound  choptionbound_var  mymalloc_sel
 * matchoptions  printfiles  myopen
 ***************************************************************/
#include <stdio.h>
#include <string.h>
#if defined __linux || defined linux
  #include <dirent.h>
  #include <unistd.h>
#else
  #include <io.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <ctype.h>
#include <asl.h>
#include <getstub.h>
#include "sip.h"
#include "select.h"



ASL *asl;

/* infinity */
#define INF 0x7ff00000

char version[]="v2.0";



/**************************************************************
 * The main routine
 *************************************************************/
void main(int argc, char **argv)
{
  char buff[256];
  int option, match=0, len;
#if defined __linux__ || defined linux
  struct dirent *file;
  DIR *directory;
  struct stat stat_file;
#else
  struct _stat stat_file;
  struct _finddata_t file;
  long filehdl;
#endif
  struct match_file *tmp;
  FILE *resfd;


  /* set default options */

  options.obj_type=ALL;
  options.cons_type=ALL;
  options.nfin_var_lb=0;
  options.nfin_var_ub=+INF;
  options.ninf_var_lb=0;
  options.ninf_var_ub=+INF;
  options.nfin_cons_lb=0;
  options.nfin_cons_ub=+INF;
  options.ninf_cons_lb=0;
  options.ninf_cons_ub=+INF;
  options.boundx=ANY;
  options.boundt=ANY;
  options.initial_guess=INI_BOTH;


  printf("Select %s tool for SIPAMPL\n\n", version);


  if(argc>1){
    if(!strncmp(argv[1], "-x", 2)){
      expert=1;
      printf("Expert mode enabled\n");
    } else {
      printf("Usage: %s -x for expert mode\n\n", argv[0]);
      exit(1);
    }
  }

  /* Ask the user for the database directory */

  printf("\nDefault database directory: %s\n", DATABASE);
  printf("New database directory [CR=Accept default]:");
  fgets(buff, 256, stdin);

  len=strlen(buff);
  if(strcmp(buff,"\n")){
    strncpy(DATABASE, buff, len-1);
    DATABASE[len-1]=0;
  }

  printf("Using database directory: %s\n", DATABASE);


  /* Some sanity check in the database directory */

#if defined __linux__ || defined linux
  if(stat(DATABASE, &stat_file)){
#else
  if(_stat(DATABASE, &stat_file)){
#endif
    printf("An error occured when stating %s directory (database)\n",
       DATABASE);
    printf("Unable to proceed... Aborting\n");
    perror("Stating database directory");
    exit(1);
  }

#if defined __linux__ || defined linux
  if(!(S_IFDIR & stat_file.st_mode)){ /*  S_ISDIR(stat_file.st_mode))){ */
#else
  if(!(_S_IFDIR & stat_file.st_mode)){
#endif
    printf("%s is not a directory... Aborting\n", DATABASE);
    exit(1);
  }




  if(expert){
    
    /* Ask the user for the database directory */
    
    printf("\nDefault deposit directory: %s\n", DEPOSIT);
    printf("New deposit directory [CR=Accept default]:");
    fgets(buff, 256, stdin);
    
    len=strlen(buff);
    if(strcmp(buff,"\n")){
      strncpy(DEPOSIT, buff, len-1);
      DEPOSIT[len-1]=0;
    }
    
    printf("Using deposit directory: %s\n", DEPOSIT);
    
    
    /* Some sanity check in the deposit directory */
    
#if defined __linux__ || defined linux
    if(stat(DEPOSIT, &stat_file)){
#else
      if(_stat(DEPOSIT, &stat_file)){
#endif
	printf("An error occured when stating %s directory (deposit)\n",
	       DEPOSIT);
	printf("Unable to proceed... Aborting\n");
	perror("Stating deposit directory");
	exit(1);
      }

#if defined __linux__ || defined linux
      if(!(S_IFDIR & stat_file.st_mode)){ /*  S_ISDIR(stat_file.st_mode))){ */
#else
	if(!(_S_IFDIR & stat_file.st_mode)){
#endif
	  printf("%s is not a directory... Aborting\n", DEPOSIT);
	  exit(1);
	}

  }

      /* Ask the user for the AMPL binary path.
	 We need the AMPL to provide .nl, .col and .row files */
      
  printf("\nFull path for AMPL binary: %s\n", AMPL);
  printf("New full path for AMPL binary [CR=Accept default]:");
  fgets(buff, 256, stdin);
  
  len=strlen(buff);
  if(strcmp(buff,"\n")){
    strncpy(AMPL, buff, len-1);
    AMPL[len-1]=0;
  }

  printf("Using %s when executing AMPL\n", AMPL);


  /* Again some sanity check on the AMPL command */

#if defined __linux__ || defined linux
  if(stat(AMPL, &stat_file)){
#else
  if(_stat(AMPL, &stat_file)){
#endif
    printf("An error occured when stating AMPL command:%s\n",AMPL);
    printf("Unable to proceed... Aborting\n");
    perror("Stating AMPL command");
    exit(1);
  }

#if defined __linux__ || defined linux
  if(!(S_ISREG(stat_file.st_mode))){
#else
  if(!(_S_IFREG & stat_file.st_mode)){
#endif
    printf("%s is not a regular file... Aborting\n", AMPL);
    exit(1);
  }

#if defined __linux__ || defined linux
  if(!(S_IXUSR & stat_file.st_mode)){
#else
  if(!(_S_IEXEC & stat_file.st_mode)){
#endif
    printf("You do not have permission to execute %s... Aborting\n", AMPL);
    exit(1);
  }


  /* Print the default options */
  printoptions();


  /* Cycle while the user wants to change options */

  while(1){

    printf("\nEnter option number to change option [CR=End]:");
    fgets(buff, 256, stdin);

    /* Stop in a newline */
    if(!strcmp(buff,"\n"))
      break;
      

    /* Find option from the user and call the appropriate routine */
    option=buff[0]-'0';
    if(option<1 || option >9){
      printf("**** Invalid option\n");
      printoptions();
    } else {
      printf("Processing option %d\n", option);
      switch(option){
      case 1:
    options.obj_type=chgoptiontype("New objective type");
    break;
      case 2:
    options.cons_type=chgoptiontype("New constraints type");
    break;
      case 3:
    chgoptionbound(&options.nfin_var_lb, &options.nfin_var_ub,
               "New number of finite variables");
    break;
      case 4:
    chgoptionbound(&options.ninf_var_lb, &options.ninf_var_ub,
               "New number of infinite variables");
    break;
      case 5:
    chgoptionbound(&options.nfin_cons_lb, &options.nfin_cons_ub,
               "New number of finite constraints");
    break;
      case 6:
    chgoptionbound(&options.ninf_cons_lb, &options.ninf_cons_ub,
               "New number of finite constraints");
    break;
      case 7:
    options.boundx=choptionbound_var("Finite variables simple bounds");
    break;
      case 8:
    options.boundt=choptionbound_var("Infinite variables simple bounds");
    break;
      case 9:
    options.initial_guess=chgoptioninitial("With or without initial guess");
    break;
      }
      /* Print the options */
      printoptions();
    }
  }

  /* All wanted options were set */

  /* Open the database directory */
#if defined __linux__ || defined linux
  directory=opendir(DATABASE);
  if(!directory){
    printf("Unable to open %s database directory... Aborting\n",DATABASE);
    perror("Database directory");
    exit(1);
  }
#else
  sprintf(buff, "%s\\*.mod",DATABASE);
  filehdl=_findfirst(buff, &file);
  if(filehdl==-1L){
    printf("Unable to open %s database directory... Aborting\n",DATABASE);
    perror("Database directory");
    exit(1);
    }
#endif


  


  /* Cycle for all files in the database directory */

#if defined __linux__ || defined linux
  while((file=readdir(directory))){
#else
  do{
#endif    
    /* File ends with .mod extension and matchs the options ? */
#if defined __linux__ || defined linux
    if(!strncmp(file->d_name+strlen(file->d_name)-4, ".mod", 4) &&
       matchoptions(file->d_name)){
#else
    if(matchoptions(file.name)){
#endif

      /* yep, one found!! */
      /* increment counter */
      match++;

#if 0
  #if defined __linux__ || defined linux
      printf("%-17s & %2d & %2d & %2d & %2d \\\\\n", file->d_name,
	     nxsip, ntsip, nxsipc, ntsipc);
  #else
      printf("%-17s & %2d & %2d & %2d & %2d \\\\\n", file.name,
	     nxsip, ntsip, nxsipc, ntsipc);
  #endif
#endif

      /* and put it in the list */
      if(!first_file){
        first_file=mymalloc_sel(sizeof(struct match_file));
#if defined __linux__ || defined linux
        first_file->filen=mymalloc_sel((strlen(file->d_name)+1)*sizeof(char));
        strcpy(first_file->filen, file->d_name);
#else
        first_file->filen=mymalloc_sel((strlen(file.name)+1)*sizeof(char));
        strcpy(first_file->filen, file.name);
#endif
        first_file->next=NULL;
      } else {
        tmp=mymalloc_sel(sizeof(struct match_file));
#if defined __linux__ || defined linux
        tmp->filen=mymalloc_sel((strlen(file->d_name)+1)*sizeof(char));
        strcpy(tmp->filen, file->d_name);
#else
        tmp->filen=mymalloc_sel((strlen(file.name)+1)*sizeof(char));
        strcpy(tmp->filen, file.name);
#endif
        tmp->next=first_file;
        first_file=tmp;
      }
    }
#if defined __linux__ || defined linux
  }
  /* Close the directory */
  closedir(directory);
#else
  } while (!_findnext(filehdl, &file));
  /* Close the directory */
  _findclose(filehdl);
#endif



  /* Print the matched files */
  if(match==0){
    printf("\n\nSorry, no files found with specified options\n\n");
  } else {
    while(1){
      printf("\n\n  %d file(s) found with specified options\n\n", match);
      
      /* Ask if want to save the file names to a file */
      
      printf("Do you want me to:\n");
      printf("  1) Save results to file ""select.res""\n");
#if defined __linux__ || defined linux
      printf("  2) Save results to a bash script ""select.run""\n");
#else
      printf("  2) Save results to a batch file ""select.bat""\n");
#endif
      printf("  3) Save results to a M-file ""sip_run.m""\n");
      printf("  4) Print results to stdout\n");
      printf("  5) Just quit\n");
      
      printf("Option:");
      fgets(buff, 20, stdin);
      
      option=buff[0]-'0';
      
      if(option>=1 && option<=5){
	switch (option){
	case 1:
	  /* Save results to file */
	  resfd=myopen("select.res");
	  if(resfd){
	    printfiles(first_file, resfd,  "   %s\n", TRUE);
	  }
	  fclose(resfd);
	  break;
	case 2:
#if defined __linux__ || defined linux
	  /* Save results and produce bash script */
	  resfd=myopen("select.run");
	  if(resfd){
	    fprintf(resfd, "#/bin/bash\nexport ");
	    fprintf(resfd, "AMPLFUNC=./bspline.dll\nfor i in");
	    printfiles(first_file, resfd,  " %s", FALSE);
	    fprintf(resfd, "\ndo\necho -n $i >> results\n");
	    fprintf(resfd, "echo #############\necho $i\n");
	    fprintf(resfd, "echo #############\n");
	    fprintf(resfd, "%s %s$i.mod\ndone", AMPL, DATABASE);
	    fclose(resfd);
	    chmod("select.run",
		  S_IRUSR | S_IWUSR| S_IXUSR | S_IRGRP
		  | S_IXGRP | S_IROTH | S_IXOTH);
	  }
#else
	  /* Save results and produce batch file */
	  resfd=myopen("select.bat");
	  if(resfd){
	    fprintf(resfd, "@echo off\nset ");
	    fprintf(resfd, "AMPLFUNC=.\\bspline.dll\n");
	    sprintf(buff,"set i=%%s\necho %%%%i%%%% >> results\necho #############\n"\
		    "echo %%%%i%%%%\n%s %s\\%%%%i%%%%.mod\necho #############\n", AMPL,
		    DATABASE);
	    printfiles(first_file, resfd,  buff, FALSE);
	    fprintf(resfd, "set i=\n");
	    fclose(resfd);
	  }
#endif
	  break;
	case 3:
	  /* Save results and produce M-file */
	  resfd=myopen("sip_run.m");
	  if(resfd){
	    fprintf(resfd, "function sip_run\n");
	    fprintf(resfd, "%% sip_run.m was automatically generated by the SIPAMPL\n");
	    fprintf(resfd, "%%   select tool\n\n");
	    sprintf(buff,"sip_solve('%s/%%s');\n", DEPOSIT);
	    printfiles(first_file, resfd,  buff, FALSE);
	    fclose(resfd);
	  }
	  break;
	case 4:
	  /* Print results to stdout */
	  printfiles(first_file, stdout, "    %s\n", TRUE);
	  break;
	case 5:
	  exit(0);
	}
      }
    }
  }
  

}


/***********************************************************
 * Print the active options
 **********************************************************/
void printoptions()
{

  printf("\nSpecified Options:\n");

  printf("1)");
  printtype(options.obj_type, "Objective Type");

  printf("2)");
  printtype(options.cons_type, "Constraints Type");

  printf("3)");
  printbound(options.nfin_var_lb, options.nfin_var_ub,
         "Number of finite variables");

  printf("4)");
  printbound(options.ninf_var_lb, options.ninf_var_ub,
         "Number of infinite variables");

  printf("5)");
  printbound(options.nfin_cons_lb, options.nfin_cons_ub,
         "Number of finite constraints");

  printf("6)");
  printbound(options.ninf_cons_lb, options.ninf_cons_ub,
         "Number of infinite constraints");

  printf("7)");
  printbound_var(options.boundx, "Finite variables:");

  printf("8)");
  printbound_var(options.boundt, "Infinite variables:");

  printf("9)");
  printinitial(options.initial_guess, "Initial guess:");

}

/***************************************************************
 * print the type of an option: linear, quadratic, etc...
 **************************************************************/
void printtype(int opt, char *str)
{
  printf("  %-20s:", str);
  switch (opt){
  case LINEAR:
    printf("   %-15s\n", "Linear");
    break;
  case QUADRATIC:
    printf("   %-15s\n", "Quadratic");
    break;
  case POLYNOMIAL:
    printf("   %-15s\n", "Polynomial");
    break;
  case GENERIC:
    printf("   %-15s\n", "Generic");
    break;
  case ALL:
    printf("   %-15s\n", "All type");
    break;
  default:
    printf("   %-15s\n", "Unknown");
    break;
  }

}


/**************************************************************
 * Print the bounds of a variable (finite or infinite)
 *************************************************************/
void printbound_var(int type, char *str)
{

  switch(type){
  case BOTH:
    printf("  %-20sBoth limits finite\n", str);
    break;
  case LOWER:
    printf("  %-20sLower limit finite\n", str);
    break;
  case UPPER:
    printf("  %-20sUpper limit finite\n", str);
    break;
  case NONE:
    printf("  %-20sNone limits finite\n", str);
    break;
  case ANY:
    printf("  %-20sEverything\n", str);
    break;
  }
}

  
/*************************************************************
 * Print the bounds (numerically)
 ************************************************************/
void printbound(int lb, int ub, char *str)
{
  char lower[20], upper[20];

  if(lb <= -INF){
    sprintf(lower,"-Infinity");
  } else {
    sprintf(lower, "%d", lb);
  }
  
  if(ub >= +INF){
    sprintf(upper,"+Infinity");
  } else {
    sprintf(upper, "%d", ub);
  }
  
  printf("  %-15s <= %-31s <= %15s\n",  lower, str, upper);
}


/***************************************************************
 * print the initial guess type
 **************************************************************/
void printinitial(int opt, char *str)
{
  printf("  %-20s:", str);
  switch (opt){
  case INI_NO:
    printf("   %-30s\n", "Without initial guess");
    break;
  case INI_YES:
    printf("   %-30s\n", "With initial guess");
    break;
  case INI_BOTH:
    printf("   %-30s\n", "With or without initial guess");
    break;
  default:
    printf("   %-30s\n", "Unknown");
    break;
  }

}




/************************************************************
 * Change an type option: linear, quadratic, etc...
 ***********************************************************/
int chgoptiontype(char *str)
{
  int opt;
  char buff[20];

  printf("  %s\n", str);
  printf("  1) Linear\n");
  printf("  2) Quadratic\n");
  printf("  3) Polynomial\n");
  printf("  4) Generic\n");
  printf("  5) All type\n");

  printf("Option:");
  fgets(buff, 20, stdin);

  opt=buff[0]-'0';

  if(opt>=1 && opt<=5){
    switch (opt){
    case 1:
      return LINEAR;
    case 2:
      return QUADRATIC;
    case 3:
      return POLYNOMIAL;
    case 4:
      return GENERIC;
    case 5:
      return ALL;
    }
    return ALL;
  } else {
    return ALL;
  }
}


/***********************************************************
 * Change an bound option (numerically)
 **********************************************************/
void chgoptionbound(int *lb, int *ub, char *str)
{
  char buff[256];

  printf("%s\n", str);

  printf("Lower bound [CR=keep value]:");
  fgets(buff, 256, stdin);

  if(strcmp(buff, "\n")){
    sscanf(buff, "%d", lb);
    if(*lb<0)*lb=0;
  }

  printf("Upper bound [CR=keep value, INF=+Infinity]:");
  fgets(buff, 256, stdin);

  if(!strncmp(buff, "INF", 3)){
    *ub=+INF;
  } else {
    if(strcmp(buff, "\n")){
      sscanf(buff, "%d", ub);
      if(*ub<*lb)*ub=*lb;
    }
  }

}


/**************************************************************
 * Change an bound option of an variable (finite or infinite)
 *************************************************************/
int choptionbound_var(char *str)
{
  char buff[20];
  int opt;


  printf("  %s\n", str);
  printf("  1) Both limits finite\n");
  printf("  2) Lower limit finite\n");
  printf("  3) Upper limit finite\n");
  printf("  4) None limit finite\n");
  printf("  5) Everything\n");

  printf("Option:");
  fgets(buff, 20, stdin);

  opt=buff[0]-'0';

  if(opt>=1 && opt<=5){
    switch (opt){
    case 1:
      return BOTH;
    case 2:
      return LOWER;
    case 3:
      return UPPER;
    case 4:
      return NONE;
    case 5:
      return ANY;
    }
    return ANY;
  } else {
    return ANY;
  }
}



/************************************************************
 * Change the initial guess type
 ***********************************************************/
int chgoptioninitial(char *str)
{
  int opt;
  char buff[20];

  printf("  %s\n", str);
  printf("  1) With initial guess\n");
  printf("  2) Without initial guess\n");
  printf("  3) With or Without initial guess\n");

  printf("Option:");
  fgets(buff, 20, stdin);

  opt=buff[0]-'0';

  if(opt>=1 && opt<=5){
    switch (opt){
    case 1:
      return INI_YES;
    case 2:
      return INI_NO;
    case 3:
      return INI_BOTH;
    }
    return INI_BOTH;
  } else {
    return INI_BOTH;
  }
}




/**************************************************************
 * A malloc which produces an error if it can not allocate the
 *   memory requested
 *************************************************************/
void *mymalloc_sel(int size)
{
  void *tmp;


  tmp=malloc(size);

  if(!tmp){
    printf("Failed to allocate memory... Aborting\n");
    perror("Allocating memory");
    exit(1);
  }

  return tmp;
}


/*************************************************************
 * Check if a given filename (with .mod extension) matches the
 *   options set by the user
 ************************************************************/
int matchoptions(char *filen)
{
  FILE *original, *copy;
  char tmpfile[256];
#if !defined __linux__ && !defined linux
  char *tmpname;
#endif
  char tmpline[256];
  char tmpclean[256];
  char token[256];
  char cmdline[256];
  int objective_check;
  int constraints_check;
  int len, i, res;
  fint m, n, no, nz, mxr, mxc;
  FILE *nl;
  double lower, upper;


  /* Control which options were already checked */
  objective_check=0;
  constraints_check=0;

  sprintf(tmpfile, "%s/%s", DATABASE, filen);
  original=fopen(tmpfile, "r");

  if(!original){
    fprintf(stderr, "Unable to open file %s\n", tmpfile);
    fprintf(stderr, "File ignored\n");
    perror("Unable to open");
    return 0;
  }

  if(expert){
    len=strlen(filen);
    strncpy(tmpline, filen, len-4);
    tmpline[len-4]=0;
    sprintf(tmpfile, "%s/%s", DEPOSIT, tmpline);
  } else {
#if defined __linux__ || defined linux
    if(!tmpnam(tmpfile)){
      fprintf(stderr, "Unable to get temporary file name\n");
      fprintf(stderr, "File %s ignored\n", filen);
      perror("Unable to get temporary file");
      return 0;
    }
#else
    tmpname=_tempnam(TMP, NULL);
    if(!tmpname){
      fprintf(stderr, "Unable to get temporary file name\n");
      fprintf(stderr, "File %s ignored\n", filen);
      perror("Unable to get temporary file");
      return 0;
    }
    strcpy(tmpfile,tmpname);
#endif
  }

  copy=fopen(tmpfile, "w");

  if(!copy){
    fprintf(stderr, "Unable to open file %s\n", tmpfile);
    fprintf(stderr, "File ignored\n");
    perror("Unable to open");
    return 0;
  }

  while(fgets(tmpline, 256, original)){
    
    if((options.obj_type!=ALL) && sscanf(tmpline, "# Objective: %s", token)){

      /* convert to lower case, just in case */
      objective_check=1;
      tolowers(token);
      if(options.obj_type==LINEAR && strcmp(token, "linear")){
        fclose(original);
        fclose(copy);
        unlink(tmpfile);
        return 0;
      }
      if(options.obj_type==QUADRATIC && strcmp(token, "quadratic")){
        fclose(original);
        fclose(copy);
        unlink(tmpfile);
        return 0;
      }
      if(options.obj_type==POLYNOMIAL && strcmp(token, "polynomial")){
        fclose(original);
        fclose(copy);
        unlink(tmpfile);
        return 0;
      }
      if(options.obj_type==GENERIC && strcmp(token, "generic")){
        fclose(original);
        fclose(copy);
        unlink(tmpfile);
        return 0;
      }

    }

    if((options.cons_type!=ALL) && sscanf(tmpline, "# Constraints: %s",
                      token)){

      /* convert to lower case, just in case */
      constraints_check=1;
      tolowers(token);
      if(options.cons_type==LINEAR && strcmp(token, "linear")){
        fclose(original);
        fclose(copy);
        unlink(tmpfile);
        return 0;
      }
      if(options.cons_type==QUADRATIC && strcmp(token, "quadratic")){
        fclose(original);
        fclose(copy);
        unlink(tmpfile);
        return 0;
      }
      if(options.cons_type==POLYNOMIAL && strcmp(token, "polynomial")){
        fclose(original);
        fclose(copy);
        unlink(tmpfile);
        return 0;
      }
      if(options.cons_type==GENERIC && strcmp(token, "generic")){
        fclose(original);
        fclose(copy);
        unlink(tmpfile);
        return 0;
      }

    }

    /* remove all white spaces and return lower cases*/
    rmlowhite(tmpline, tmpclean);
    if(!strcmp(tmpclean, "solve;")){
      fprintf(copy, "option auxfiles rc;\nwrite g%s;\n", tmpfile);
    } else {
      fprintf(copy, tmpline);
    }
  }

  if(options.obj_type!=ALL && !objective_check){
    fprintf(stderr, "Option on objective function and "
        "no objective type found\n");
    fprintf(stderr, "File ignored: %s\n", filen);
    fclose(original);
    fclose(copy);
    unlink(tmpfile);
    return 0;
  }

  if(options.cons_type!=ALL && !constraints_check){
    fprintf(stderr, "Option on constraints function and "
        "no constraints type found\n");
    fprintf(stderr, "File ignored: %s\n", filen);
    fclose(original);
    fclose(copy);
    unlink(tmpfile);
    return 0;
  }

  fclose(original);
  fclose(copy);

  /* call ampl to obtain .col, .row and .nl files */
#if defined __linux__ || defined linux
  sprintf(cmdline, "%s %s 2>&1 > /dev/null", AMPL, tmpfile);
#else
  sprintf(cmdline, "%s %s > nul 2> nul ", AMPL, tmpfile);
#endif
  system(cmdline);


  asl=ASL_alloc(ASL_read_pfgh);
  

  want_xpi0=1;

  nl=jac_dim_ASL(asl, tmpfile, &m, &n, &no, &nz, &mxr, &mxc,
         (fint)strlen(tmpfile));

  if(!nl){
    fprintf(stderr, "Can't read problem %s\n", tmpfile);
    perror("Reading problem");
  }

  pfgh_read_ASL(asl, nl, 0);


  sip_init(asl);
  

  res=1;

  if(options.nfin_var_lb>nxsip || options.nfin_var_ub<nxsip)
    res=0;

  if(options.ninf_var_lb>ntsip || options.ninf_var_ub<ntsip)
    res=0;

  if(options.nfin_cons_lb>nxsipc || options.nfin_cons_ub<nxsipc)
    res=0;

  if(options.ninf_cons_lb>ntsipc || options.ninf_cons_ub<ntsipc)
    res=0;

  if(options.initial_guess != INI_BOTH){
    if(X0 && (!havex0 || (havex0 && *havex0))){
      /* problem has initial guess */
      if(options.initial_guess == INI_NO)
	res=0;
    } else {
      /* problem has not initial guess */
      if(options.initial_guess == INI_YES)
	res=0;
    }
  }
    

  if(options.boundt!=ANY){
    /* Infinite variables bounds */
    lower=TBL[0];
    upper=TBU[0];
    for(i=1;i<ntsip;i++){
      if(TBL[i]>lower)
    lower=TBL[i];
      if(TBU[i]<upper)
    upper=TBU[i];
    }
    switch(options.boundt){
    case BOTH:
      if(lower<=-INF || upper >=+INF)
    res=0;
      break;
    case LOWER:
      if(lower<=-INF || upper <+INF)
    res=0;
      break;
    case UPPER:
      if(lower>-INF || upper >=+INF)
    res=0;
      break;
    case NONE:
      if(lower>-INF || upper <+INF)
    res=0;
      break;
    }
  }
  
  /* Finite variables bounds */
  if(options.boundx!=ANY){
    /* Finite variables bounds */
    lower=XBL[0];
    upper=XBU[0];
    for(i=1;i<nxsip;i++){
      if(XBL[i]>lower)
    lower=XBL[i];
      if(XBU[i]<upper)
    upper=XBU[i];
    }
    switch(options.boundx){
    case BOTH:
      if(lower<=-INF || upper >=+INF)
    res=0;
      break;
    case LOWER:
      if(lower<=-INF || upper <+INF)
    res=0;
      break;
    case UPPER:
      if(lower>-INF || upper >=+INF)
    res=0;
      break;
    case NONE:
      if(lower>-INF || upper <+INF)
    res=0;
      break;
    }
  }
  

  sip_free();

#if !defined __linux__ && !defined linux
  /* If we free the memory under Linux a segmentation fault happens, 
     if not a segmentation fault happens in MSDOS !!!!!!
     As far as I know the memory should be released */
  ASL_free(&asl);
#endif


  unlink(tmpfile);

  if(expert && !res){ /* remove all unmatched problems */
    len=strlen(tmpfile);
    strcpy(tmpfile+len, ".col");
    unlink(tmpfile);
    strcpy(tmpfile+len, ".row");
    unlink(tmpfile);
    strcpy(tmpfile+len, ".nl");
    unlink(tmpfile);
  }

  if(!expert){  
    /* remove all tmp files */
    len=strlen(tmpfile);
    strcpy(tmpfile+len, ".col");
    unlink(tmpfile);
    strcpy(tmpfile+len, ".row");
    unlink(tmpfile);
    strcpy(tmpfile+len, ".nl");
    unlink(tmpfile);
  }
  

  return res;
}


/************************************************************
 * Recursevelly print the matched filenames
 * It accepts an file descriptor for eventually write to a
 *   file.
 ***********************************************************/
void printfiles(struct match_file *tmp, FILE *desc, char *fmt, int ext)
{
  char file[256];
  int len;

  if(!tmp)return;

  if(tmp->filen)
    if(ext){
      fprintf(desc, fmt, tmp->filen);
    } else {
      len=strlen(tmp->filen);
      strncpy(file, tmp->filen, len-4);
      file[len-4]=0;
      fprintf(desc, fmt, file);
    }

  if(tmp->next)
    printfiles(tmp->next, desc, fmt, ext);

  return;

}


/*************************************************************
 * Get all char from a string to lower letters
 ************************************************************/
void tolowers(char *s)
{
  int len=strlen(s), i;
  
  for(i=0;i<len;i++)
    s[i]=tolower(s[i]);
}


/*************************************************************
 * Remove all white spaces from the string and lower cases
 ************************************************************/
void rmlowhite(char *tmpline, char *tmpclean)
{
  int len=strlen(tmpline);
  int i,j;

  for(i=0, j=0;i<len;i++)
    if(tmpline[i]!=' ' && tmpline[i]!='\n' && tmpline[i]!='\r'){
      tmpclean[j]=tolower(tmpline[i]);
      j++;
    }
  tmpclean[j]=0;
}


FILE *myopen(char *filen)
{
  FILE *resfd;
  char answer[256];
  int option;
  struct stat stat_file;

  /* First some sanity check */
  if(stat(filen, &stat_file)){
    /* Error, good if file does not exists */
    if(errno==ENOENT){
      resfd=fopen(filen, "w");
    } else {
      perror("Can not stat file");
      return NULL;;
    }
  } else {
    /* No error, so file exists */
    printf("File %s exists\n", filen);
    printf("Do you want to overwrite it?[Y/N]");
    fgets(answer, 256, stdin);
    option=(int)answer[0];
    if(option=='y' || option=='Y'){
      resfd=fopen(filen, "w");
    } else {
      return NULL;
    }
  }
  
  if(!resfd){
    perror("Opening file");
    return NULL;
  }

  return resfd;
}
