#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "funcadd.h"

double blend(int i, int k, double x, double *knots);
double dblend(int j, int i,int k,double x,double *knots);
double alpha(int n, int k,int j, int r, double *ctl, double *knots);
double dalpha(int i, int n, int k, int j, int r, double *ctl, double *knots);
int tridiagonal(int n, double *inf, double *diag, double *sup, double *ind,
		double *sol, double *multi, int warm);

static double *M=NULL, *previous_space=NULL, *previous_knots=NULL;
static double *dM;
static int Mdim=0, dcomputed=0;


#define FALSE 0
#define TRUE  1


void printarray(AmplExports *ae, real *a,int n)
{
  int i;

  for(i=0;i<n;i++)
    fprintf(Stderr,"%.3lf ", a[i]);
  fprintf(Stderr,"\n\n");
}

void printmatrix(AmplExports *ae, real *m, int n1, int n2)
{
  int i;

  for(i=0;i<n1;i++)
    printarray(ae, m+n2*i, n2);
}




double bspline(arglist *al)
{
  double x, res;
  double *d, *h, *ra, *ctl, *knots;
  int *at, i, j, k, n;
  AmplExports *ae = al->AE;
    
  at = al->at;
  ra = al->ra;
  d = al->derivs;
  h = al->hes;

  if (al->n < 3){
    fprintf(Stderr, "Invalid number of args to bspline expected >=3 and obtained %d\n", al->n);
    return 0;
  }
  
  x=ra[0]; k=(int)ra[1]; n=ra[2];
  ctl=ra+3;
  knots=ra+n+3;

  /*  for(i=0;i<n;i++)
    fprintf(Stderr, "ctl%i=%f\n", i, ctl[i]);

  for(i=0;i<k+n;i++)
    fprintf(Stderr, "Knots%i=%f\n", i, knots[i]);

    fprintf(Stderr,"t=%f\n", x);*/
  res=0.0;
  for(j=0;j<n;j++){
    if(d){
      d[j+1]=blend(j,k,x,knots);
/*        fprintf(Stderr,"d%i=%f\n", j+1, d[j+1]); */
      res+=ctl[j]*d[j+1];
    } else {
      res+=ctl[j]*blend(j,k,x,knots);
    }
  }
/*    fprintf(Stderr,"sp=%f\n", res); */

  if(d){
    /*    fprintf(Stderr, "first derivative\n");*/
    /* bspline first derivative with respect to the t parameter*/
    d[0]=0.0;
    for(i=0;i<n+1;i++)
      d[0]+=alpha(n, k, 2, i, ctl, knots)*blend(i, k-1, x, knots);
  }
/*    fprintf(Stderr,"spd=%f\n", d[0]); */
  if(h){
    /*    fprintf(Stderr, "second derivative\n");*/
    /* bspline second derivative with respect to the t parameter*/
    h[0]=0.0;
    for(i=0;i<n+2;i++)
      h[0]+=alpha(n, k, 3, i, ctl, knots)*blend(i, k-2, x, knots);
  }

  return res;
}



double dbspline(arglist *al)
{
  double x, res;
  double *d, *h, *ra, *ctl, *knots;
  int *at, i, j, k, n;
  AmplExports *ae = al->AE;
    
  at = al->at;
  ra = al->ra;
  d = al->derivs;
  h = al->hes;

  if (al->n < 4){
    fprintf(Stderr, "Invalid number of args to bspline expected >=4 and obtained %d\n", al->n);
    return 0;
  }
  
  x=ra[0]; j=(int)ra[1]; k=(int)ra[2]; n=ra[3];
  ctl=ra+4;
  knots=ra+n+4;

/*    for(i=0;i<n;i++) */
/*      fprintf(Stderr, "ctl%i=%f\n", i, ctl[i]); */

/*    for(i=0;i<k+n;i++) */
/*      fprintf(Stderr, "Knots%i=%f\n", i, knots[i]); */

/*    fprintf(Stderr,"t=%f\n", x); */
  
  res=0.0;
  for(i=0;i<n+j;i++)
    res+=alpha(n, k, j+1, i, ctl, knots)*blend(i, k-j, x, knots);

/*    fprintf(Stderr,"sp=%f\n", res); */

  if(d){
    d[0]=0.0;
    for(i=0;i<n+j+1;i++)
      d[0]+=alpha(n, k, j+2, i, ctl, knots)*blend(i, k-j-1, x, knots);
/*      fprintf(Stderr,"spd=%f\n", d[0]); */

    for(i=0;i<n;i++){
      d[i+1]=dblend(j, i, k, x, knots);
/*        fprintf(Stderr,"d%i=%f\n", i+1, d[i+1]); */
    }
  }

  if(h){
    h[0]=0.0;
    for(i=0;i<n+j+2;i++)
      h[0]+=alpha(n, k, j+3, i, ctl, knots)*blend(i, k-j-2, x, knots);
  }
      
  return res;
}



double alpha(int n, int k, int j, int r, double *ctl, double *knots)
{
  double res;
  double denom;

  if(r<0 || r>n)return 0.0;

  j--;

  if(j>0){
    denom=knots[r+k-j]-knots[r];
    if(denom){
      res=(k-j)*((alpha(n, k, j, r, ctl, knots)-alpha(n, k, j, r-1, ctl, knots))/denom);
    } else {
      res=0;
    }
  } else {
    res=ctl[r];
  }

  return res;
}

double dalpha(int i, int n, int k, int j, int r, double *ctl, double *knots)
{
  double res;
  double denom;

  if(r<0 || r>n)return 0.0;
  if(r+k<j)return 0.0;
  
  j--;

  if(j>0){
    denom=knots[r+k-j]-knots[r];
    if(denom){
      res=(k-j)*((dalpha(i, n, k, j, r, ctl, knots)-dalpha(i, n, k, j, r-1, ctl, knots))/denom);
    } else {
      res=0.0;
    }
  } else {
    if(i==r){
      res=1.0;
    } else {
      res=0.0;
    }
  }

  return res;
}



double blend(int i,int k,double x,double *knots)
{
  double denom1,denom2,temp1,temp2;


  if(k>1){
    denom1=knots[i+k-1]-knots[i];
    if(denom1){
      temp1=(x-knots[i])*blend(i,k-1,x,knots)/denom1;
    } else {
      temp1=0.;
    }
    denom2=knots[i+k]-knots[i+1];
    if(denom2){
      temp2=(knots[i+k]-x)*blend(i+1,k-1,x,knots)/denom2;
    } else {
      temp2=0.;
    }
    return (temp1+temp2);
  } else {
    if(knots[i]<=x && x<knots[i+1]){
      return 1.;
    } else {
      return 0.;
    }
  }
}


double dblend(int j, int i,int k,double x,double *knots)
{
  double denom1, denom2, res;

  if(j<=1){
    denom1=knots[i+k]-knots[i+1];
    denom2=knots[i+k-1]-knots[i];
    res=0;
    if(denom1)
      res+=-blend(i+1, k-1, x, knots)/denom1;
    if(denom2)
      res+=blend(i, k-1, x, knots)/denom2;
    res*=(k-1);
  } else {
    denom1=knots[i+k]-knots[i+1];
    denom2=knots[i+k-1]-knots[i];
    res=0;
    if(denom1)
      res+=-dblend(j-1, i+1, k-1, x, knots)/denom1;
    if(denom2)
      res+=dblend(j-1, i, k-1, x, knots)/denom2;
    res*=(k-1);
  }

return res;
}



/* solve a tridiagonal system */
int tridiagonal(int n, double *inf, double *diag, double *sup, double *ind,
		double *sol, double *multi, int warm)
{
  int i;
  double m;

  /* These are internal error. It should not happen */
  if(!inf || !diag || !sup || !sol || n<1)
    return 1;

  if(warm && !multi)
    return 1;

  /* Do inf elimination */
  for(i=0;i<n-1;i++){
    if(warm){
      m=multi[i];
    } else {
      m=-inf[i]/diag[i];
      if(multi)
	multi[i]=m;
      diag[i+1]+=m*sup[i];
    }      
    ind[i+1]+=m*ind[i];
  }

  sol[n-1]=ind[n-1]/diag[n-1];

  for(i=n-2;i>=0;i--)
    sol[i]=(ind[i]-sup[i]*sol[i+1])/diag[i];

  return 0;
}


double cspline(arglist *al)
{
  double x, res;
  double *d, *h, *ra, *space, *knots;
  double *sup=NULL, *diag=NULL, *inf=NULL, *ind=NULL, *multi=NULL;
  double vi, vf, ai, af, factor;
  double a, b, ad, bd, tmp1, tmp2;
  int *at, i, j, k, n;
  AmplExports *ae = al->AE;
    
  at = al->at;
  ra = al->ra;
  d = al->derivs;
  h = al->hes;

  if (al->n < 11){
    fprintf(Stderr, "Invalid number of args to cspline."
	    " Expected >=11 and obtained %d.\n", al->n);
    return 0;
  }
  
  x=ra[0]; k=(int)ra[1]; n=ra[2];
  space=ra+3;
  knots=ra+n+3;
  vi=ra[n+n+2];
  vf=ra[n+n+3];
  ai=ra[n+n+4];
  af=ra[n+n+5];


  if(!M || Mdim!=n+1 || !dM || !previous_space || !previous_knots ||
     (d && !dcomputed) ||
     memcmp(space, previous_space, n*sizeof(double)) ||
     memcmp(knots, previous_knots, (n-1)*sizeof(double))){

    if(Mdim!=n+1){ /* dimension changed, reallocate all */


      if(M)free(M);
      Mdim=n+1;
      M=(double *)malloc(Mdim*sizeof(double));

      if(dM)free(dM);
      dM=(double *)malloc((n-1)*n*sizeof(double)); /* dM is allways allocated,
						      even if not needed */


      if(previous_space)free(previous_space);
      previous_space=(double *)malloc(n*sizeof(double));

      if(previous_knots)free(previous_knots);
      previous_knots=(double *)malloc((n-1)*sizeof(double));


    }

    fprintf(Stderr, "Computing new M values\n");

    sup=(double *)malloc((n-2)*sizeof(double));
    inf=(double *)malloc((n-2)*sizeof(double));
    diag=(double *)malloc((n-1)*sizeof(double));
    ind=(double *)malloc((n-1)*sizeof(double));
    

    memcpy(previous_space, space, n*sizeof(double));
    memcpy(previous_knots, knots, (n-1)*sizeof(double));
    
    /* Compute new M array */
    M[0]=ai; M[n]=af; /* initial and final second derivatives */

    /* line above diagonal */
    for(i=0;i<n-3;i++)
      sup[i]=space[i+1];
    sup[n-3]=(space[n-2]-(pow(space[n-1], 2.0)/space[n-2]));

    /* diagonal */
    diag[0]=3*space[0]+2*space[1]+pow(space[0],2.0)/space[1];
    for(i=1;i<n-2;i++)
      diag[i]=2*(space[i]+space[i+1]);
    diag[n-2]=3*space[n-1]+2*space[n-2]+pow(space[n-1],2.0)/space[n-2];

    /* line below diagonal */
    inf[0]=space[1]-pow(space[0],2.0)/space[1];
    for(i=1;i<n-2;i++)
      inf[i]=space[i+1];

    /* independent term */
    factor=(knots[0]+space[0]*vi+pow(space[0],2.0)*ai/3);
    ind[0]=6*(knots[1]/space[1]+knots[0]/space[0])
      -6*(1/space[0]+1/space[1])*factor-space[0]*ai;
    ind[1]=6*factor/space[1]+6*knots[2]/space[2]
      -6*(1/space[1]+1/space[2])*knots[1];
    for(i=2;i<n-3;i++)
      ind[i]=6*((knots[i+1]-knots[i])/space[i+1]
		-(knots[i]-knots[i-1])/space[i]);
    factor=knots[n-2]-vf*space[n-1]+pow(space[n-1],2.0)*af/3;
    ind[n-3]=6*factor/space[n-2]-6*(1/space[n-2]+1/space[n-3])*knots[n-3]+
      6*knots[n-4]/space[n-3];
    ind[n-2]=-6*(1/space[n-1]+1/space[n-2])*factor+6*knots[n-2]/space[n-1]+
      6*knots[n-3]/space[n-2]-af*space[n-1];

    if(d) /* keep multipliers for warm start in derivatives */
      multi=(double *)malloc((n-1)*sizeof(double));
    
    tridiagonal(n-1, inf, diag, sup, ind, M+1, multi, FALSE);


    if(d){
      dcomputed=1;

      /* compute derivatives with warm start */

      /* w.r.t. space[0] */
      memset(ind, 0, (n-1)*sizeof(double));
      tmp1=(knots[0]+space[0]*vi+pow(space[0],2.0)*ai/3);
      tmp2=(vi+2*space[0]*ai/3);
      ind[0]=-6*knots[0]/(space[0]*space[0])
      +6*tmp1/(space[0]*space[0])-6*(1/space[0]+1/space[1])*tmp2-ai;

      ind[0]-=(3+2*space[0]/space[1])*M[1];
      ind[1]+=(2*space[0]/space[1])*M[1];

      tridiagonal(n-1, inf, diag, sup, ind, dM, multi, TRUE);

      /* w.r.t. space[1] */
      memset(ind, 0, (n-1)*sizeof(double));
      ind[0]=-6*knots[1]/(space[1]*space[1])
	+6*tmp1/(space[1]*space[1]);
      ind[1]=-6*tmp1/(space[1]*space[1])
	+6*knots[1]/(space[1]*space[1]);

      ind[0]-=M[2]+(2-pow(space[0],2.0)/(space[1]*space[1]))*M[1];
      ind[1]-=(1-pow(space[0],2.0)/(space[1]*space[1]))*M[1]+2*M[2];

      tridiagonal(n-1, inf, diag, sup, ind, dM+n-1, multi, TRUE);

      /* w.r.t. space[2] */
      memset(ind, 0, (n-1)*sizeof(double));
      ind[1]=-6*knots[2]/(space[2]*space[2])
	-6*knots[1]/(space[2]*space[2]);
      ind[2]=6*(knots[2]-knots[1])/(space[2]*space[2]);

      ind[1]-=M[3]+2*M[2];
      ind[2]-=2*M[3]+M[2];

      tridiagonal(n-1, inf, diag, sup, ind, dM+2*n-2, multi, TRUE);

      for(j=3;j<n-3;j++){ /* derivative w.r.t. space[j] */
	      memset(ind, 0, (n-1)*sizeof(double));
	      ind[j-1]=-6*(knots[j]-knots[j-1])/(space[j]*space[j]);
	      ind[j]=6*(knots[j]-knots[j-1])/(space[j]*space[j]);

	      ind[j-1]-=M[j+1]+2*M[j];
	      ind[j]-=2*M[j+1]+M[j];
	      tridiagonal(n-1, inf, diag, sup, ind, dM+j*(n-1), multi, TRUE);
      }


      /* derivative w.r.t. space[n-3] */
      memset(ind, 0, (n-1)*sizeof(double));
      ind[n-4]=-6*(knots[n-3]-knots[n-4])/(space[n-3]*space[n-3]);
      ind[n-3]=-(6*knots[n-3])/(space[n-3]*space[n-3])
	-6*knots[n-4]/(space[n-3]*space[n-3]);

      ind[n-4]-=M[n-2]+2*M[n-3];
      ind[n-3]-=2*M[n-2]+M[n-3];

      tridiagonal(n-1, inf, diag, sup, ind, dM+(n-3)*(n-1), multi, TRUE);

      /* derivative w.r.t. space[n-2] */
      memset(ind, 0, (n-1)*sizeof(double));
      tmp1=knots[n-2]-vf*space[n-1]+pow(space[n-1],2.0)*af/3;
      ind[n-3]=-6*tmp1/(space[n-2]*space[n-2])
	-(6*knots[n-3])/(space[n-2]*space[n-2]);
      ind[n-2]=6*tmp1/(space[n-2]*space[n-2])
	-6*knots[n-3]/(space[n-2]*space[n-2]);

      ind[n-3]-=(1-pow(space[n-1],2.0)/(space[n-2]*space[n-2]))*M[n-1]
	+2*M[n-2];
      ind[n-2]=(2-pow(space[n-1],2.0)/(space[n-2]*space[n-2]))*M[n-1]+M[n-2];

      tridiagonal(n-1, inf, diag, sup, ind, dM+(n-2)*(n-1), multi, TRUE);      

      /* derivative w.r.t. space[n-1] */
      memset(ind, 0, (n-1)*sizeof(double));
      tmp2=vf+2*space[n-1]*af/3;
      ind[n-2]=6*tmp1/(space[n-1]*space[n-1])
	-6*(1/space[n-1]+1/space[n-2])*tmp2
	-6*knots[n-2]/(space[n-1]*space[n-1])-af;

      ind[n-3]-=(-2*space[n-1]/space[n-2])*M[n-1];
      ind[n-2]-=(3+2*space[n-1]/space[n-2])*M[n-1];

      tridiagonal(n-1, inf, diag, sup, ind, dM+(n-1)*(n-1), multi, TRUE);      

    } else {
      dcomputed=0;
    }


    free(sup);
    free(inf);
    free(diag);
    free(ind);
    if(d)free(multi);

    
  } else {
    fprintf(Stderr, "Using old M values\n");
  }

  printarray(ae, M, n+1);

  /* Use computed M array to compute spline at x */


  /* find spline segment */

  if(x<0)
    fprintf(Stderr, "Using a negative parameter in the spline\n");

  tmp1=0;
  tmp2=space[0];
  for(i=1;x>tmp2 && i<n;i++)
    tmp2+=space[i];

  if(i>1)
    tmp1=tmp2-space[i-1];

  if(x>tmp2)
    fprintf(Stderr, "Using a parameter out of the spline interpolation\n");

  /* spline segment i */

  /*  tmp1=0;  t0 */
  /*  tmp2=space[0];  t1 */

  /*  for(j=1;j<i;j++){
      tmp1+=space[j-1];  t(i)
      tmp2+=space[j];    t(i+1)
      }
  */


  
  /*    fprintf(Stderr, "Computing segment %d, sum=%.3f, psum=%.3f, tmp1=%.3f," */
  /*     	  "tmp2=%.3f, t=%.3f\n", */
  /*     	  i, sum, psum, tmp1, tmp2, x); */

  if(d)
    memset(d, 0, (n+1)*sizeof(double));

  switch(k){ /* derivatives of the spline */
  case 0: /* Cubic spline */

    /*      fprintf(Stderr, "Cubic spline called\n"); */

    if(i==1){
      a=knots[0]+space[0]*vi+pow(space[0],2.0)*(ai+M[1]/2)/3;
      res=M[0]*pow(tmp2-x, 3.0)/(6*space[0])+
	M[1]*pow(x-tmp1, 3.0)/(6*space[0])+
	(a/space[0]-space[0]*M[1]/6)*(x-tmp1)+
	(knots[0]/space[0]-space[0]*M[0]/6)*(tmp2-x);
      if(d){
	/* derivative w.r.t. parameter */
	d[0]=-M[0]*pow(tmp2-x, 2.0)/(2*space[0])+
	  M[1]*pow(x-tmp1, 2.0)/(2*space[0])+
	  (a/space[0]-space[0]*M[1]/6)-
	  (knots[0]/space[0]-space[0]*M[0]/6);
	/* derivative w.r.t. first space */
	ad=vi+2*space[0]*(ai+M[1]/2)/3;
	d[1]=-M[0]*pow(tmp2-x, 3.0)/(6*space[0]*space[0])-
	  M[1]*pow(x-tmp1, 3.0)/(6*space[0]*space[0])+
	  (ad/space[0]-a/(space[0]*space[0])-M[1]/6)*(x-tmp1)+
	  (-knots[0]/(space[0]*space[0])-M[0]/6)*(tmp2-x);
	/* all others are zero */
      }
    }
    
    if(i==2){
      a=knots[0]+space[0]*vi+pow(space[0],2.0)*(ai+M[1]/2)/3;
      res=M[1]*pow(tmp2-x, 3.0)/(6*space[1])+
	M[2]*pow(x-tmp1, 3.0)/(6*space[1])+
	(knots[1]/space[1]-space[1]*M[2]/6)*(x-tmp1)+
	(a/space[1]-space[1]*M[1]/6)*(tmp2-x);
      if(d){
	d[0]=-M[1]*pow(tmp2-x, 2.0)/(2*space[1])+
	  M[2]*pow(x-tmp1, 2.0)/(2*space[1])+
	  (knots[1]/space[1]-space[1]*M[2]/6)-
	  (a/space[1]-space[1]*M[1]/6);
	d[1]=((vi+2*space[0]*(ai+M[1]/2)/3)/space[1])*(tmp2-x);
	d[2]=-M[1]*pow(tmp2-x, 3.0)/(6*space[1]*space[1])-
	  M[2]*pow(x-tmp1, 3.0)/(6*space[1]*space[1])+
	  (-knots[1]/(space[1]*space[1])-M[2]/6)*(x-tmp1)+
	  (-a/(space[1]*space[1])-M[1]/6)*(tmp2-x);
      }
    }
    
    if(i==n-1){
      b=knots[n-2]-space[n-1]*vf+pow(space[n-1],2.0)*(af+M[n-1]/2)/3;
      res=M[n-2]*pow(tmp2-x, 3.0)/(6*space[n-2])+
	M[n-1]*pow(x-tmp1, 3.0)/(6*space[n-2])+
	(b/space[n-2]-space[n-2]*M[n-1]/6)*(x-tmp1)+
	(knots[n-3]/space[n-2]-space[n-2]*M[n-2]/6)*(tmp2-x);
      if(d){
	d[0]=-M[n-2]*pow(tmp2-x, 2.0)/(2*space[n-2])+
	  M[n-1]*pow(x-tmp1, 2.0)/(2*space[n-2])+
	  (b/space[n-2]-space[n-2]*M[n-1]/6)-
	  (knots[n-3]/space[n-2]-space[n-2]*M[n-2]/6);
	d[n-1]=-M[n-2]*pow(tmp2-x, 3.0)/(6*space[n-2]*space[n-2])-
	  M[n-1]*pow(x-tmp1, 3.0)/(6*space[n-2]*space[n-2])+
	  (-b/(space[n-2]*space[n-2])-M[n-1]/6)*(x-tmp1)+
	  (-knots[n-3]/(space[n-2]*space[n-2])-M[n-2]/6)*(tmp2-x);
	d[n]=((-vf+2*space[n-1]*(af+M[n-1]/2)/3)/space[n-2])*(x-tmp1);
      }
    }
    
    if(i==n){
      b=knots[n-2]-space[n-1]*vf+pow(space[n-1],2.0)*(af+M[n-1]/2)/3;
      res=M[n-1]*pow(tmp2-x, 3.0)/(6*space[n-1])+
	M[n]*pow(x-tmp1, 3.0)/(6*space[n-1])+
	(knots[n-2]/space[n-1]-space[n-1]*M[n]/6)*(x-tmp1)+
	(b/space[n-1]-space[n-1]*M[n-1]/6)*(tmp2-x);
      if(d){
	d[0]=-M[n-1]*pow(tmp2-x, 2.0)/(2*space[n-1])+
	  M[n]*pow(x-tmp1, 2.0)/(2*space[n-1])+
	  (knots[n-2]/space[n-1]-space[n-1]*M[n]/6)-
	  (b/space[n-1]-space[n-1]*M[n-1]/6);
	bd=-vf+2*space[n-1]*(af+M[n-1]/2)/3;
	d[n]=-M[n-1]*pow(tmp2-x, 3.0)/(6*space[n-1]*space[n-1])-
	  M[n]*pow(x-tmp1, 3.0)/(6*space[n-1]*space[n-1])+
	  (-knots[n-2]/(space[n-1]*space[n-1])-M[n]/6)*(x-tmp1)+
	  (bd/space[n-1]-b/(space[n-1]*space[n-1])-M[n-1]/6)*(tmp2-x);
      }

    }
    
    if(i!=1 && i!=2 && i!=n-1 && i!=n){
      res=M[i-1]*pow(tmp2-x, 3.0)/(6*space[i-1])+
	M[i]*pow(x-tmp1, 3.0)/(6*space[i-1])+
	(knots[i-1]/space[i-1]-space[i-1]*M[i]/6)*(x-tmp1)+
	(knots[i-2]/space[i-1]-space[i-1]*M[i-1]/6)*(tmp2-x);
      if(d){
	d[0]=-M[i-1]*pow(tmp2-x, 2.0)/(2*space[i-1])+
	  M[i]*pow(x-tmp1, 2.0)/(2*space[i-1])+
	  (knots[i-1]/space[i-1]-space[i-1]*M[i]/6)-
	  (knots[i-2]/space[i-1]-space[i-1]*M[i-1]/6);
	d[i]=-M[i-1]*pow(tmp2-x, 3.0)/(6*space[i-1]*space[i-1])-
	  M[i]*pow(x-tmp1, 3.0)/(6*space[i-1]*space[i-1])+
	  (-knots[i-1]/(space[i-1]*space[i-1])-M[i]/6)*(x-tmp1)+
	  (-knots[i-2]/(space[i-1]*space[i-1])-M[i-1]/6)*(tmp2-x);
      }
      
    }
    break;
  case 1:
    /* Quadratic spline */

    /*      fprintf(Stderr, "Quadratic spline called\n"); */

    if(i==1){
      a=knots[0]+space[0]*vi+pow(space[0],2.0)*(ai+M[1]/2)/3;
      res=-M[0]*pow(tmp2-x, 2.0)/(2*space[0])+
	M[1]*pow(x-tmp1, 2.0)/(2*space[0])+
	(a/space[0]-space[0]*M[1]/6)-
	(knots[0]/space[0]-space[0]*M[0]/6);
      if(d){
	/* derivative w.r.t. parameter */
	d[0]=M[0]*(tmp2-x)/space[0]+
	  M[1]*(x-tmp1)/space[0];
	/* derivative w.r.t. first space */
	ad=vi+2*space[0]*(ai+M[1]/2)/3;
	d[1]=M[0]*pow(tmp2-x, 2.0)/(2*space[0]*space[0])-
	  M[1]*pow(x-tmp1, 2.0)/(2*space[0]*space[0])+
	  (ad/space[0]-a/(space[0]*space[0])-M[1]/6)-
	  (-knots[0]/(space[0]*space[0])-M[0]/6);
	/* all others are zero */
      }
    }
    
    if(i==2){
      a=knots[0]+space[0]*vi+pow(space[0],2.0)*(ai+M[1]/2)/3;
      res=-M[1]*pow(tmp2-x, 2.0)/(2*space[1])+
	  M[2]*pow(x-tmp1, 2.0)/(2*space[1])+
	  (knots[1]/space[1]-space[1]*M[2]/6)-
	  (a/space[1]-space[1]*M[1]/6);
      if(d){
	d[0]=M[1]*(tmp2-x)/space[1]+
	  M[2]*(x-tmp1)/space[1];
	d[1]=-((vi+2*space[0]*(ai+M[1]/2)/3)/space[1]);
	d[2]=M[1]*pow(tmp2-x, 2.0)/(2*space[1]*space[1])-
	  M[2]*pow(x-tmp1, 2.0)/(2*space[1]*space[1])+
	  (-knots[1]/(space[1]*space[1])-M[2]/6)-
	  (-a/(space[1]*space[1])-M[1]/6);
      }
    }
    
    if(i==n-1){
      b=knots[n-2]-space[n-1]*vf+pow(space[n-1],2.0)*(af+M[n-1]/2)/3;
      res=-M[n-2]*pow(tmp2-x, 2.0)/(2*space[n-2])+
	M[n-1]*pow(x-tmp1, 2.0)/(2*space[n-2])+
	(b/space[n-2]-space[n-2]*M[n-1]/6)-
	(knots[n-3]/space[n-2]-space[n-2]*M[n-2]/6);
      if(d){
	d[0]=M[n-2]*(tmp2-x)/space[n-2]+
	  M[n-1]*(x-tmp1)/space[n-2];
	d[n-1]=M[n-2]*pow(tmp2-x, 2.0)/(2*space[n-2]*space[n-2])-
	  M[n-1]*pow(x-tmp1, 2.0)/(2*space[n-2]*space[n-2])+
	  (-b/(space[n-2]*space[n-2])-M[n-1]/6)-
	  (-knots[n-3]/(space[n-2]*space[n-2])-M[n-2]/6);
	d[n]=((-vf+2*space[n-1]*(af+M[n-1]/2)/3)/space[n-2]);
      }
    }
    
    if(i==n){
      b=knots[n-2]-space[n-1]*vf+pow(space[n-1],2.0)*(af+M[n-1]/2)/3;
      res=-M[n-1]*pow(tmp2-x, 2.0)/(2*space[n-1])+
	M[n]*pow(x-tmp1, 2.0)/(2*space[n-1])+
	(knots[n-2]/space[n-1]-space[n-1]*M[n]/6)-
	(b/space[n-1]-space[n-1]*M[n-1]/6);
      if(d){
	d[0]=M[n-1]*(tmp2-x)/space[n-1]+
	  M[n]*(x-tmp1)/space[n-1];
	bd=-vf+2*space[n-1]*(af+M[n-1]/2)/3;
	d[n]=M[n-1]*pow(tmp2-x, 2.0)/(2*space[n-1]*space[n-1])-
	  M[n]*pow(x-tmp1, 2.0)/(2*space[n-1]*space[n-1])+
	  (-knots[n-2]/(space[n-1]*space[n-1])-M[n]/6)-
	  (bd/space[n-1]-b/(space[n-1]*space[n-1])-M[n-1]/6);
      }
    }
    
    if(i!=1 && i!=2 && i!=n-1 && i!=n){
      res=-M[i-1]*pow(tmp2-x, 2.0)/(2*space[i-1])+
	  M[i]*pow(x-tmp1, 2.0)/(2*space[i-1])+
	  (knots[i-1]/space[i-1]-space[i-1]*M[i]/6)-
	  (knots[i-2]/space[i-1]-space[i-1]*M[i-1]/6);
      if(d){
	d[0]=M[i-1]*(tmp2-x)/space[i-1]+
	  M[i]*(x-tmp1)/space[i-1];
	d[i]=M[i-1]*pow(tmp2-x, 2.0)/(2*space[i-1]*space[i-1])-
	  M[i]*pow(x-tmp1, 2.0)/(2*space[i-1]*space[i-1])+
	  (-knots[i-1]/(space[i-1]*space[i-1])-M[i]/6)-
	  (-knots[i-2]/(space[i-1]*space[i-1])-M[i-1]/6);
      }
      
    }

    break;
  case 2:
    /* Linear spline */

    /*      fprintf(Stderr, "Linear spline called\n"); */

    res=M[i-1]*(tmp2-x)/space[i-1]+
      M[i]*(x-tmp1)/space[i-1];
    if(d){
      d[0]=-M[i-1]/space[i-1]+
	M[i]/space[i-1];
      d[i]=-M[i-1]*(tmp2-x)/(space[i-1]*space[i-1])-
	M[i]*(x-tmp1)/(space[i-1]*space[i-1]);
      
    }

    break;
  case 3:
    /* Constant spline */

    fprintf(Stderr, "Constant spline called\n");

    res=-M[i-1]/space[i-1]+M[i]/space[i-1];
    if(d){
      d[0]=0;
      d[i]=M[i-1]/(space[i-1]*space[i-1])-
	M[i]/(space[i-1]*space[i-1]);
    }

    break;
  default:
    res=0;
    break;
  }


  return res;
}    



void
funcadd(AmplExports *ae){
  /* Insert calls on addfunc here... */
  /* Arg 3, called argtype, can be 0 or 1:
   *	0 ==> force all arguments to be numeric
   *	1 ==> pass both symbolic and numeric arguments.
   *
   * Arg 4, called nargs, is interpretted as follows:
   *	>=  0 ==> the function has exactly nargs arguments
   *	<= -1 ==> the function has >= -(nargs+1) arguments.
   *
   * Arg 5, funcinfo, is passed to the functions in struct arglist;
   *	it is not used in these examples, so we just pass 0.
   */
  
  addfunc("bspline", (ufunc*)bspline, 0, -4, 0);
  addfunc("dbspline", (ufunc*)dbspline, 0, -5, 0);
  addfunc("cspline", (ufunc*)cspline, 0, -13, 0);
  /*    addfunc("dcspline", (ufunc*)dcspline, 0, -13, 0); */

}
