#include "intpkkt.h"
#include "int.h"

double int_solve_nlp(void);
double lag_solve_nlp(void);

void intpkkts()
{
  double f0, f1, num, den, tmp, xi;
  double primal=0, dual=0;
  int ko, ieps, i, l;
  int reset=0;
  double miu;  /* penalty parameter */
  FILE *fp; /* for results file */

#if defined __linux__ || defined linux
  double tsample[ntsip];
  double dir[nxsip+2*ntsipc], hss[nxsip*nxsip];
  double x0[nxsip], x1[nxsip], laggdx1[nxsip], laggdx0[nxsip];
  double laggdx0l1[nxsip], yeps[nxsip];
  double s0[ntsipc], s1[ntsipc], term0[ntsipc], term1[ntsipc];
  double fgd0[nxsip], fgd1[nxsip];
  double lag0[ntsipc], lag1[ntsipc];
  double laggdl0[ntsipc], laggdl1[ntsipc];
  double jac0[nxsip*ntsipc], jac1[nxsip*ntsipc];
#else
  double *tsample;
  double *dir, *hss, *x0, *x1, *laggdx1, *laggdx0, *laggdx0l1, *yeps;
  double *s0, *s1, *term0, *term1, *fgd0, *fgd1;
  double *lag0, *lag1, *laggdl0, *laggdl1;
  double *jac0, *jac1;
  tsample=mymalloc(ntsip*sizeof(double));
  dir=mymalloc((nxsip+2*ntsipc)*sizeof(double));
  hss=mymalloc(nxsip*nxsip*sizeof(double));
  x0=mymalloc(nxsip*sizeof(double));
  x1=mymalloc(nxsip*sizeof(double));
  laggdx1=mymalloc(nxsip*sizeof(double));
  laggdx0=mymalloc(nxsip*sizeof(double));
  laggdx0l1=mymalloc(nxsip*sizeof(double));
  yeps=mymalloc(nxsip*sizeof(double));
  s0=mymalloc(ntsipc*sizeof(double));
  s1=mymalloc(ntsipc*sizeof(double));
  term0=mymalloc(ntsipc*sizeof(double));
  term1=mymalloc(ntsipc*sizeof(double));
  fgd0=mymalloc(nxsip*sizeof(double));
  fgd1=mymalloc(nxsip*sizeof(double));
  lag0=mymalloc(ntsipc*sizeof(double));
  lag1=mymalloc(ntsipc*sizeof(double));
  laggdl0=mymalloc(ntsipc*sizeof(double));
  laggdl1=mymalloc(ntsipc*sizeof(double));
  jac0=mymalloc(nxsip*ntsipc*sizeof(double));
  jac1=mymalloc(nxsip*ntsipc*sizeof(double));
#endif





/*    printf("Obj=%f\n", int_solve_nlp()); */
  
/*    return;  */

  ieps=0;

#if 0
  GetInitialGuess(nxsip, X, x1, ntsipc, s1, lag1);
#else

  memcpy(x1, X, nxsip*sizeof(double));  /* x1 with initial guess */


  /*  Equality variables */

  for(l=0;l<ntsipc;l++){
    adapt_trap(&gxtepsilon, TBL, TBU, nxsip, x1, ntsip, tsample,
	       Sip_Opt.adapt_int_n, l, &num, &den,
	       Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
	       NO_DERIV, NULL, 0);
    if(num<Sip_Opt.theta){
      s1[l]=Sip_Opt.theta;
    } else {
      s1[l]=num;
    }
    lag1[l]=-1;
  }

#endif


  /*    printf("X1:\n"); */
  /*    printarray(x1, nxsip); */
  /*    printf("S1:\n"); */
  /*    printarray(s1, ntsipc); */
  /*    printf("LAG1:\n"); */
  /*    printarray(lag1, ntsipc); */
		  


  memcpy(yeps, X, nxsip*sizeof(double));



  miu=0;
  for(l=0;l<ntsipc;l++)
    miu-=lag1[l]*(s1[l]+Sip_Opt.pf_tau);
  miu/=ntsipc;


  printf("%-5s %-24s %-24s %-24s\n", "Iter", "Mu", "Primal", "Dual");


  for(ko=0;ko<Sip_Opt.maxitero;ko++){

    reset=0;

    if(ko){

#define JAMMPRECL 1e-12
#define JAMMPRECU 1e+8

#if 0
      /* compute harmonic average */
      den=0.0;
      i=0;
      for(l=0;l<ntsipc;l++){
	if((s1[l]+Sip_Opt.pf_tau)>=JAMMPRECL || dir[nxsip+l]/s1[l]<=JAMMPRECU){
	  i++;
	  den+=1/(s1[l]+Sip_Opt.pf_tau);
	}
      }

      if(i){
	den=i/den;
      } else {
	den=-s1[l]+(JAMMPRECL)-Sip_Opt.pf_tau;
      }

      /* sanity checkup */
      for(l=0;l<ntsipc;l++){
	if((s1[l]+Sip_Opt.pf_tau)<JAMMPRECL || dir[nxsip+l]/s1[l]>JAMMPRECU){
	  /*  	  printf("Correcting slack %d\n",l); */
	  s1[l]+=den;
	}
      }

#else

      /* sanity checkup */
      for(l=0;l<ntsipc;l++){
	if((s1[l]+Sip_Opt.pf_tau)<JAMMPRECL/Sip_Opt.pf_tau){
	  /*  	  printf("Correcting slack %d\n",l); */
	  s1[l]=JAMMPRECL/Sip_Opt.pf_tau-Sip_Opt.pf_tau;
	}
      }

#endif

    }

    if(ko){

      /* computing barrier parameter */
      
      switch(Sip_Opt.updtb){
      default:
	printf("Using update formula 1 for the barrier parameter\n");
      case 1:
	miu=0;
	for(l=0;l<ntsipc;l++)
	  miu-=lag1[l]*(s1[l]+Sip_Opt.pf_tau);
	miu*=Sip_Opt.delta/ntsipc;
	break;
      case 2:
	miu=xi=-lag1[0]*(s1[0]+Sip_Opt.pf_tau);
	for(l=1;l<ntsipc;l++){
	  tmp=-lag1[l]*(s1[l]+Sip_Opt.pf_tau);
	  miu+=tmp;
	  if(tmp<xi)
	    xi=tmp;
	}
	miu/=ntsipc;
	xi=xi/miu;
	
	tmp=0.05*(1-xi)/xi;
	if(tmp>2)
	  tmp=2;
	tmp=pow(tmp,3.0);
	
	miu*=Sip_Opt.delta*tmp;
	break;
      }
    }
    


    /*      printf("ITERATION %3d ********** miu=%f epsilon=%f\n", ko, miu, */
    /*  	   Sip_Opt.pf_epsilon); */


    /* Get Lagrangian, miu has changed */
    lagintpkkt(nxsip, x1, ntsipc, lag1, s1, ntsip, tsample, miu, FIRST_DERIV,
	       fgd1, laggdx1, laggdl1, jac1, term1, &f1);

     printf("X1:\n");
     printarray(x1, nxsip);
     printf("S1:\n");
     printarray(s1, ntsipc);
     printf("LAG1:\n");
     printarray(lag1, ntsipc);
     printf("Gradient x1:\n");
     printarray(laggdx1, nxsip);
     printf("Gradient l1:\n");
     printarray(laggdl1, ntsipc);
     printf("Term1:\n");
     printarray(term1, ntsipc);
     printf("Jac1:\n");
     printmatrix(jac1, ntsipc, nxsip);

    
    if(ko){          /* don't stop in first iteration */
      /* stopping criteria */
      /* outer stopping criteria */
      if(int_stop(nxsip, ntsipc, fgd1, lag1, s1, jac1, term1, &primal, &dual)){

	/*  	break; */


	if(ieps>0){
	  if(pf_eps_stop(nxsip, yeps, x1)){
	    printf("Precision on eps satisfied\n");
	    break;
	  }
	}
	ieps++;
	memcpy(yeps, x1,    nxsip*sizeof(double));
	printf("Decreasing epsilon\n");
	Sip_Opt.pf_epsilon*=Sip_Opt.epsfactor;
	Sip_Opt.pf_tau*=Sip_Opt.epsfactor;
	if(Sip_Opt.pf_epsilon<Sip_Opt.epslimit){
	  printf("Limit on eps precision\n");
	  break;
	}
	
	cycle;
	
	/*  Equality variables */

	for(l=0;l<ntsipc;l++){
	  adapt_trap(&gxtepsilon, TBL, TBU, nxsip, x1, ntsip, tsample,
		     Sip_Opt.adapt_int_n, l, &num, &den,
		     Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		     NO_DERIV, NULL, 0);
	  if(num<pow(Sip_Opt.epsfactor,ko)*Sip_Opt.theta){
	    s1[l]=pow(Sip_Opt.epsfactor,ko)*Sip_Opt.theta;
	  } else {
	    s1[l]=num;
	  }
	}

	/* recompute miu and Lagrangian */

	/* computing barrier parameter */
	
	switch(Sip_Opt.updtb){
	default:
	  printf("Using update formula 1 for the barrier parameter\n");
	case 1:
	  miu=0;
	  for(l=0;l<ntsipc;l++)
	    miu-=lag1[l]*(s1[l]+Sip_Opt.pf_tau);
	  miu*=Sip_Opt.delta/ntsipc;
	  break;
	case 2:
	  miu=xi=-lag1[0]*(s1[0]+Sip_Opt.pf_tau);
	  for(l=1;l<ntsipc;l++){
	    tmp=-lag1[l]*(s1[l]+Sip_Opt.pf_tau);
	    miu+=tmp;
	    if(tmp<xi)
	      xi=tmp;
	  }
	  miu/=ntsipc;
	  xi=xi/miu;
	  
	  tmp=0.05*(1-xi)/xi;
	  if(tmp>2)
	    tmp=2;
	  tmp=pow(tmp,3.0);
	  
	  miu*=Sip_Opt.delta*tmp;
	  break;
	}

	/* Get Lagrangian, miu has changed */
	lagintpkkt(nxsip, x1, ntsipc, lag1, s1, ntsip, tsample, miu,
		   FIRST_DERIV, fgd1, laggdx1, laggdl1, jac1, term1, &f1);
	reset=1;
      }
    }



    /* compute Lagrangian gradient to update Hessian */
    if(ko){
      memcpy(laggdx0l1, fgd0, nxsip*sizeof(double));
      for(i=0;i<nxsip;i++)
	for(l=0;l<ntsipc;l++)
	  laggdx0l1[i]-=lag1[l]*jac0[l*nxsip+i];
    }
  
    /* Set Hessian or Hessian inverse */
    if(reset){
      switch(Sip_Opt.ordering){
      case PRIMAL:
	set_h(hss, 0, nxsip, x1, x0, laggdx1, laggdx0l1);
	break;
      case DUAL:
      case PREDCORR:
	set_hss(hss, 0, nxsip, x1, x0, laggdx1, laggdx0l1);
	break;
      }
    } else {
      switch(Sip_Opt.ordering){
      case PRIMAL:
	set_h(hss, ko, nxsip, x1, x0, laggdx1, laggdx0l1);
	break;
      case DUAL:
      case PREDCORR:
	set_hss(hss, ko, nxsip, x1, x0, laggdx1, laggdx0l1);
	break;
      }
    }
    /*      printf("Hessian\n"); */
    /*      printmatrix(hss, nxsip, nxsip); */
    
    
    memcpy(x0,      x1,      nxsip*sizeof(double));
    memcpy(laggdx0, laggdx1, nxsip*sizeof(double));
    memcpy(s0,      s1,      ntsipc*sizeof(double));
    memcpy(lag0,    lag1,    ntsipc*sizeof(double));
    memcpy(laggdl0, laggdl1, ntsipc*sizeof(double));
    memcpy(term0,   term1,   ntsipc*sizeof(double));
    memcpy(jac0,    jac1,    nxsip*ntsipc*sizeof(double));
    memcpy(fgd0,    fgd1,    nxsip*sizeof(double));
    f0=f1;

    switch(Sip_Opt.ordering){
    case PRIMAL:
      get_primal_dir(nxsip, x0, s0, ntsipc, lag0, hss, fgd0, jac0, laggdx0,
		     laggdl0, term0, miu, dir);
      break;
    case DUAL:
      get_dual_dir(nxsip, x0, s0, ntsipc, lag0, hss, fgd0, jac0, laggdx0,
		   laggdl0, term0, miu, dir);
      break;
    case PREDCORR:
      get_pc_dir(nxsip, x0, s0, ntsipc, lag0, hss, fgd0, jac0, laggdx0,
		 laggdl0, term0, miu, dir);
      break;
    }
    
    printf("Direction:\n");
    printarray(dir, nxsip+2*ntsipc);
    
/*      get_dir_intpkkt_tmp(nxsip, x0, s0, ntsipc, lag0, hss, fgd0, jac0, laggdx0, */
/*  		    laggdl0, term0, miu, dir1); */

/*      printf("Direction:\n"); */
/*      printarray(dir1, nxsip+2*ntsipc); */
    
/*      printf("X0:\n"); */
/*      printarray(x0, nxsip); */
/*      printf("S0:\n"); */
/*      printarray(s0, ntsipc); */
/*      printf("LAG0:\n"); */
/*      printarray(lag0, ntsipc); */
/*      printf("Gradient x0:\n"); */
/*      printarray(laggdx0, nxsip); */
/*      printf("Gradient l0:\n"); */
/*      printarray(laggdl0, ntsipc); */
/*      printf("Term0:\n"); */
/*      printarray(term0, ntsipc); */
/*      printf("Jac0:\n"); */
/*      printmatrix(jac0, ntsipc, nxsip); */
      
    if(Sip_Opt.ScaleDir)
      scale_dir(nxsip+2*ntsipc, dir);
    
    switch (Sip_Opt.linesearch){
    default:
      printf("Unknown LineSearch\n");
    case L2MERIT:
    case ALMERIT:
      armijo_intpkkt(nxsip, x0, s0, ntsipc, lag0, dir, miu, x1, s1, lag1, hss,
		     reset);
      break;
    case ARMI_BAKRY:
      armi_bakry(nxsip, x0, s0, ntsipc, lag0, dir, miu, x1, s1, lag1);
      break;
    case ARMI_DUAL:
      armijo_intdual(nxsip, x0, s0, ntsipc, lag0, dir, miu, x1, s1, lag1);
      break;
    };

    printf("%5d %4.20f %4.20f %4.20f\n", ko, miu, primal, dual);

    
  }
  
  memcpy(X, x1, nxsip*sizeof(double));
  printf("Solution found\n");
  printarray(X, nxsip);
  printf("Objective function value\n");
  printf("%f\n", f1);


  fp=fopen("./results", "a+");
  f0=eval_sip(SIP_OBJVAL, 0, X, NULL, NULL, NULL, NULL, NULL);

  printf("Gradient of lagrangian:\n");
  printarray(laggdx1, nxsip);


  fprintf(fp, " & %d & %d & %d & %.6e & %.1e & %.1e & %.1e & %.1e \\\\\n",
	  ko+1, stats.nsip_lag, stats.nsip_mer, f0, primal, dual, miu, Sip_Opt.pf_epsilon);
  fclose(fp);

#if !defined __linux__ && !defined linux
  free(tsample);
  free(dir);
  free(hss);
  free(x0);
  free(x1);
  free(laggdx0);
  free(laggdx1);
  free(laggdx0l1);
  free(yeps);
  free(s0);
  free(s1);
  free(term0);
  free(term1);
  free(fgd0);
  free(fgd1);
  free(lag0);
  free(lag1);
  free(laggdl0);
  free(laggdl1);
  free(jac0);
  free(jac1);
#endif


}



/*********************************************************
Evaluate the lagrangian function
*********************************************************/
double lagintpkkt(int nx, double *x, int nl, double *lag, double *s, int nt,
		  double *t, double miu, int dlevel, double *fgd,
		  double *laggdx, double *laggdl, double *jac, double *term,
		  double *f)
{
  double lagval;
  double num, den, dummy;
  int i, k;


  stats.nsip_lag++;

  lagval=0.0;
  if(dlevel==FIRST_DERIV){
    memset(laggdx, 0, sizeof(double)*nx);
    memset(laggdl, 0, sizeof(double)*nl);
  }


  for(i=0; i<nl; i++){
    adapt_trap(&gxtepsilon, TBL, TBU, nx, x, nt, t,
	       Sip_Opt.adapt_int_n, i, &num, &den,
	       Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
	       dlevel, fgd, 0);
    
    
    if(dlevel==FIRST_DERIV && jac!=NULL)
      memcpy(&(jac[i*nx]), fgd, sizeof(double)*nx);

    lagval-=lag[i]*(num+s[i])+miu*mylog(s[i]+Sip_Opt.pf_tau);

    if(term)
      term[i]=num;

    /*      printf("term[%d]=%f\n", i, term[i]); */
    /*      printf("num[%d]=%f\n", i, num); */
    /*      printf("lag[%d]=%f\n", i, lag[i]); */
    
    if(dlevel==FIRST_DERIV){
      if(laggdl)
	laggdl[i]=-num-s[i];
      for(k=0;k<nx;k++)
	laggdx[k]-=lag[i]*fgd[k];
    }
  }
  
  if(f){
    lagval+=*f=eval_sip(SIP_OBJVAL, 0, x, NULL, NULL, NULL, NULL, NULL);
  } else {
    lagval+=eval_sip(SIP_OBJVAL, 0, x, NULL, NULL, NULL, NULL, NULL);
  }
  
  if(dlevel==FIRST_DERIV){
    dummy=eval_sip(SIP_OBJGRD, 0, x, NULL, fgd, NULL, NULL, NULL);
    for(k=0;k<nx;k++)
      laggdx[k]+=fgd[k];
  }

  return lagval;
}



/* Get direction using dual ordering */
void get_dual_dir(int nx, double *x, double *s, int nl, double *lag,
		  double *hss, double *fgd, double *jac, double *laggdx,
		  double *laggdl, double *term, double miu, double *dir)
{
  int i, j, k, sing;

#if defined __linux__ || linux
  double N[nx*nx], Naux[nx*nx], ind[nx], sol[nx];
  double rho[nl], gamma[nl], aux[nx*nl], auxv[nl];
#else
  double *N, *Naux, *ind, *sol, *rho, *gamma, *aux, *auxv;
  N=mymalloc(nx*nx*sizeof(double));
  Naux=mymalloc(nx*nx*sizeof(double));
  ind=mymalloc(nx*sizeof(double));
  sol=mymalloc(nx*sizeof(double));
  rho=mymalloc(nl*sizeof(double));
  gamma=mymalloc(nl*sizeof(double));
  aux=mymalloc(nx*nl*sizeof(double));
  auxv=mymalloc(nl*sizeof(double));
#endif

  
  memcpy(N, hss, nx*nx*sizeof(double));



  for(i=0;i<nx;i++){
    for(j=0;j<nl;j++){
      aux[j*nx+i]=jac[j*nx+i]*lag[j]/(s[j]+Sip_Opt.pf_tau);
    }
  }

  for(i=0;i<nx;i++){
    for(j=0;j<nx;j++){
      for(k=0;k<nl;k++){
	N[i*nx+j]-=aux[k*nx+i]*jac[k*nx+j];
      }
    }
  }

  for(i=0;i<nl;i++)
    rho[i]=term[i]+s[i];

  for(i=0;i<nl;i++)
    auxv[i]=lag[i]*rho[i]-miu;
  
  for(i=0;i<nx;i++){
    for(j=0;j<nl;j++){
      aux[j*nx+i]=jac[j*nx+i]/(s[j]+Sip_Opt.pf_tau);
    }
  }

  for(i=0;i<nx;i++)
    ind[i]=-fgd[i];

  for(i=0;i<nx;i++){
    for(j=0;j<nl;j++){
      ind[i]+=aux[i+j*nx]*auxv[j];
    }
  }

  /*    printf("HSS:\n"); */
  /*    printmatrix(hss, nx, nx); */
  /*    printf("Jac:\n"); */
  /*    printmatrix(jac, nl, nx); */
  /*    printf("N:\n"); */
  /*    printmatrix(N, nx, nx); */
  /*    printf("Ind:\n"); */
  /*    printarray(ind, nx); */

  memcpy(Naux, N, nx*nx*sizeof(double));

/*    cholesky(nx, N, ind, sol, &sing); */


  mod_cholesky(nx, N, ind, sol, Sip_Opt.choleps, &sing);

/*    residual=0.0; */
/*    for(i=0;i<nx;i++){ */
/*      factor=0.0; */
/*      for(j=0;j<nx;j++) */
/*        factor+=Naux[j+i*nx]*sol[j]; */
/*      residual+=pow(factor-ind[i], 2.0); */
/*    } */

/*    printf("Residual=%.20f\n", residual); */
	


  if(sing){
    printf("N is singular\n");
#if !defined __linux__ && !defined linux
    free(N);
    free(Naux);
    free(ind);
    free(sol);
    free(rho);
    free(gamma);
    free(aux);
    free(auxv);
#endif
    exit(1);
  }

  memcpy(dir, sol, nx*sizeof(double));

  for(j=0;j<nl;j++){
    auxv[j]=0.0;
    for(i=0;i<nx;i++){
      auxv[j]+=jac[j*nx+i]*sol[i];
    }
  }

  for(i=0;i<nl;i++){
    dir[nx+i]=-rho[i]-auxv[i];
  }

  for(i=0;i<nl;i++){
    gamma[i]=lag[i]*(s[i]+Sip_Opt.pf_tau)+miu;
  }

  for(i=0;i<nl;i++){
    dir[nx+nl+i]=
      -(lag[i]*dir[nx+i]+gamma[i])/(s[i]+Sip_Opt.pf_tau);
  }
#if !defined __linux__ && !defined linux
  free(N);
  free(Naux);
  free(ind);
  free(sol);
  free(rho);
  free(gamma);
  free(aux);
  free(auxv);
#endif

}


/* Get direction using primal ordering */
void get_primal_dir(int nx, double *x, double *s, int nl, double *lag,
		  double *hss, double *fgd, double *jac, double *laggdx,
		  double *laggdl, double *term, double miu, double *dir)
{
  int i, j, k, sing;


#if defined __linux__ || linux
  double N[nl*nl], sigma[nx], ind[nl], sol[nl];
  double rho[nl], gamma[nl], aux[nx*nl], lauxv[nl], xauxv[nx];
#else
  double *N, *sigma, *ind, *sol, *rho, *gamma, *aux, *lauxv, *xauxv;
  N=mymalloc(nl*nl*sizeof(double));
  sigma=mymalloc(nx*sizeof(double));
  ind=mymalloc(nl*sizeof(double));
  sol=mymalloc(nl*sizeof(double));
  rho=mymalloc(nl*sizeof(double));
  gamma=mymalloc(nl*sizeof(double));
  aux=mymalloc(nx*nl*sizeof(double));
  lauxv=mymalloc(nl*sizeof(double));
  xauxv=mymalloc(nx*sizeof(double));
#endif

  

  for(i=0;i<nx;i++){
    for(j=0;j<nl;j++){
      aux[i*nl+j]=0;
      for(k=0;k<nx;k++){
	aux[i*nl+j]+=hss[k*nx+i]*jac[j*nx+k];
      }
    }
  }

  for(i=0;i<nl;i++){
    for(j=0;j<nl;j++){
      N[i*nl+j]=0;
      for(k=0;k<nx;k++){
	N[i*nl+j]+=aux[k*nl+i]*jac[j*nx+k];
      }
    }
  }

  for(i=0;i<nl;i++)
    N[i*nl+i]-=(s[i]+Sip_Opt.pf_tau)/lag[i];


  for(i=0;i<nx;i++){
    sigma[i]=-fgd[i];
    for(j=0;j<nl;j++)
      sigma[i]+=jac[j*nx+i]*lag[j];
  }

  memset(xauxv, 0, nx*sizeof(double));
  for(i=0;i<nx;i++){
    for(j=0;j<nx;j++)
      xauxv[i]+=hss[i*nx+j]*sigma[j];
  }

  memset(lauxv, 0, nl*sizeof(double));
  for(i=0;i<nl;i++)
    for(j=0;j<nx;j++)
      lauxv[i]+=jac[i*nx+j]*xauxv[j];

  for(i=0;i<nl;i++)
    rho[i]=term[i]+s[i];

  for(i=0;i<nl;i++){
    gamma[i]=lag[i]*(s[i]+Sip_Opt.pf_tau)+miu;
  }

  for(i=0;i<nl;i++)
    ind[i]=gamma[i]/lag[i]-rho[i]-lauxv[i];


  mod_cholesky(nl, N, ind, sol, Sip_Opt.choleps, &sing);

/*    printf("Sol\n"); */
/*    printarray(sol, nl); */

  if(sing){
    printf("N is singular\n");
#if !defined __linux__ && !defined linux
    free(N);
    free(sigma);
    free(ind);
    free(sol);
    free(rho);
    free(gamma);
    free(aux);
    free(lauxv);
    free(xauxv);
#endif
    exit(1);
  }

  memcpy(dir+nx+nl, sol, nl*sizeof(double));


  for(i=0;i<nl;i++)
    dir[nx+i]=-((s[i]+Sip_Opt.pf_tau)*dir[nx+nl+i]+gamma[i])/lag[i];


  memcpy(xauxv, sigma, nx*sizeof(double));
  for(i=0;i<nx;i++)
    for(j=0;j<nl;j++)
      xauxv[i]+=jac[j*nx+i]*dir[nx+nl+j];
  
  memset(dir, 0, nx*sizeof(double));
  for(i=0;i<nx;i++)
    for(j=0;j<nx;j++)
      dir[i]+=hss[i*nx+j]*xauxv[j];

#if !defined __linux__ && !defined linux
  free(N);
  free(sigma);
  free(ind);
  free(sol);
  free(rho);
  free(gamma);
  free(aux);
  free(lauxv);
  free(xauxv);
#endif

}



/* Get direction using predictor corrector with dual ordering */
void get_pc_dir(int nx, double *x, double *s, int nl, double *lag,
		double *hss, double *fgd, double *jac, double *laggdx,
		double *laggdl, double *term, double miu, double *dir)
{
  int i, j, k, l, corr, sing;
  double alpha;

#if defined __linux__ || linux
  double N[nx*nx], Naux[nx*nx], ind[nx], sol[nx];
  double rho[nl], gamma[nl], aux[nx*nl], auxv[nl];
#else
  double *N, *Naux, *ind, *sol, *rho, *gamma, *aux, *auxv;
  N=mymalloc(nx*nx*sizeof(double));
  Naux=mymalloc(nx*nx*sizeof(double));
  ind=mymalloc(nx*sizeof(double));
  sol=mymalloc(nx*sizeof(double));
  rho=mymalloc(nl*sizeof(double));
  gamma=mymalloc(nl*sizeof(double));
  aux=mymalloc(nx*nl*sizeof(double));
  auxv=mymalloc(nl*sizeof(double));
#endif

  
  memcpy(N, hss, nx*nx*sizeof(double));


  for(i=0;i<nx;i++){
    for(j=0;j<nl;j++){
      aux[j*nx+i]=jac[j*nx+i]*lag[j]/(s[j]+Sip_Opt.pf_tau);
    }
  }

  for(i=0;i<nx;i++){
    for(j=0;j<nx;j++){
      for(k=0;k<nl;k++){
	N[i*nx+j]-=aux[k*nx+i]*jac[k*nx+j];
      }
    }
  }

  /* N = H-JS^{-1}LJ^T */

  for(i=0;i<nl;i++)
    rho[i]=term[i]+s[i];

  for(i=0;i<nl;i++)
    auxv[i]=lag[i]*rho[i]; /* for predictor */
  
  for(i=0;i<nx;i++){
    for(j=0;j<nl;j++){
      aux[j*nx+i]=jac[j*nx+i]/(s[j]+Sip_Opt.pf_tau);
    }
  }

  for(i=0;i<nx;i++)
    ind[i]=-fgd[i];

  for(i=0;i<nx;i++){
    for(j=0;j<nl;j++){
      ind[i]+=aux[i+j*nx]*auxv[j];
    }
  }


  memcpy(Naux, N, nx*nx*sizeof(double));


  mod_cholesky(nx, N, ind, sol, Sip_Opt.choleps, &sing);

  if(sing){
    printf("N is singular\n");
#if !defined __linux__ && !defined linux
    free(N);
    free(Naux);
    free(ind);
    free(sol);
    free(rho);
    free(gamma);
    free(aux);
    free(auxv);
#endif
    exit(1);
  }

  memcpy(dir, sol, nx*sizeof(double));

  for(j=0;j<nl;j++){
    auxv[j]=0.0;
    for(i=0;i<nx;i++){
      auxv[j]+=jac[j*nx+i]*sol[i];
    }
  }

  for(i=0;i<nl;i++){
    dir[nx+i]=-rho[i]-auxv[i];
  }

  for(i=0;i<nl;i++){
    gamma[i]=lag[i]*(s[i]+Sip_Opt.pf_tau);
  }

  for(i=0;i<nl;i++){
    dir[nx+nl+i]=
      -(lag[i]*dir[nx+i]+gamma[i])/(s[i]+Sip_Opt.pf_tau);
  }


  /* dir has the predicted direction */
/*    printf("Predicted direction:\n"); */
/*    printarray(dir, nxsip+2*ntsipc); */


  for(corr=0;corr<Sip_Opt.ncorrector;corr++){

    /* compute the maximum allowed step */
    
    alpha=1.0/0.95;
    for(l=0;l<nl;l++){
      if(dir[nx+nl+l]>0 && -lag[l]/dir[nx+nl+l]<alpha){
	alpha=-lag[l]/dir[nx+nl+l];
      }
      if(dir[nx+l]<0 && -(s[l]+Sip_Opt.pf_tau)/dir[nx+l]<alpha){
	alpha=-(s[l]+Sip_Opt.pf_tau)/dir[nx+l];
      }
    }
    
    if(alpha<0)
      alpha=0;

    alpha*=0.95;

/*      printf("alpha=%f\n", alpha); */

    miu=0;
    for(l=0;l<ntsipc;l++)
      miu-=(lag[l]+alpha*dir[nx+nl+l])*(s[l]+Sip_Opt.pf_tau+alpha*dir[nx+l]);
    miu/=ntsipc;

/*      printf("miu=%f\n", miu); */

    memcpy(N, Naux, nx*nx*sizeof(double));

    for(i=0;i<nl;i++)
      auxv[i]=lag[i]*rho[i]-miu-dir[nx+i]*dir[nx+nl+i]; /* for corrector */

    for(i=0;i<nx;i++)
      ind[i]=-fgd[i];
    
    for(i=0;i<nx;i++){
      for(j=0;j<nl;j++){
	ind[i]+=aux[i+j*nx]*auxv[j];
      }
    }

    mod_cholesky(nx, N, ind, sol, Sip_Opt.choleps, &sing);

    if(sing){
      printf("N is singular\n");
      exit(1);
    }

    memcpy(dir, sol, nx*sizeof(double));
  
    for(j=0;j<nl;j++){
      auxv[j]=0.0;
      for(i=0;i<nx;i++){
	auxv[j]+=jac[j*nx+i]*sol[i];
      }
    }

    for(i=0;i<nl;i++){
      gamma[i]=lag[i]*(s[i]+Sip_Opt.pf_tau)+miu+dir[nx+i]*dir[nx+nl+i];
    }

    for(i=0;i<nl;i++){
      dir[nx+i]=-rho[i]-auxv[i];
    }

    for(i=0;i<nl;i++){
      dir[nx+nl+i]=
	-(lag[i]*dir[nx+i]+gamma[i])/(s[i]+Sip_Opt.pf_tau);
    }

/*      printf("Corrected direction:\n"); */
/*      printarray(dir, nxsip+2*ntsipc); */


  }

  /* dir has the predict+correct direction */
#if !defined __linux__ && !defined linux
  free(N);
  free(Naux);
  free(ind);
  free(sol);
  free(rho);
  free(gamma);
  free(aux);
  free(auxv);
#endif

}





void get_dir_intpkkt_tmp(int nx, double *x, double *s, int nl, double *lag,
			 double *hss, double *fgd, double *jac, double *laggdx,
			 double *laggdl, double *term, double miu, double *dir)
{
  int i, j, sing;

#if defined __linux__ || defined linux
  double fullh[(nx+nl)*(nx+nl)], ind[nx+nl], sol[nx+nl];
#else
  double *fullh, *ind, *sol;
  fullh=mymalloc((nx+nl)*(nx+nl)*sizeof(double));
  ind=mymalloc((nx+nl)*sizeof(double));
  sol=mymalloc((nx+nl)*sizeof(double));
#endif

  
  for(i=0;i<nx;i++)
    memcpy(&fullh[i*(nx+nl)], &hss[i*nx], nx*sizeof(double));

  for(i=0;i<nx;i++)
    for(j=0;j<nl;j++){
      fullh[j+nx+i*(nx+nl)]=-jac[j*nx+i];
      fullh[i+(j+nx)*(nx+nl)]=-jac[j*nx+i];
    }

  for(i=0;i<nl;i++){
    memset(&fullh[(nx+nl)*(nx+i)+nx], 0, nl*sizeof(double));
    fullh[(nx+nl)*(nx+i)+nx+i]=(s[i]+Sip_Opt.pf_tau)/lag[i];
  }

  for(i=0;i<nx;i++)
    ind[i]=-laggdx[i];
  for(i=0;i<nl;i++)
    ind[i+nx]=term[i]-miu/lag[i]-Sip_Opt.pf_tau;

  /*    printf("HSS:\n"); */
  /*    printmatrix(hss, nx, nx); */
  /*    printf("Jac:\n"); */
  /*    printmatrix(jac, nl, nx); */
  /*    printf("Fullh:\n"); */
  /*    printmatrix(fullh, (nx+nl), (nx+nl)); */
  /*    printf("Ind:\n"); */
  /*    printarray(ind, nx+nl); */


  egpp(nx+nl, fullh, ind, sol, &sing);


  if(sing){
    printf("Full Hessian is singular\n");
#if !defined __linux__ && !defined linux
    free(fullh);
    free(ind);
    free(sol);
#endif
    exit(1);
  }

  memcpy(dir, sol, nx*sizeof(double));
  memcpy(&dir[nx+nl], &sol[nx], nl*sizeof(double));

  for(i=0;i<nl;i++)
    dir[i+nx]=-((s[i]+Sip_Opt.pf_tau)/lag[i])*sol[nx+i]
      -miu/lag[i]-(s[i]+Sip_Opt.pf_tau);
  
#if !defined __linux__ && !defined linux
    free(fullh);
    free(ind);
    free(sol);
#endif
}




void armijo_intpkkt(int nx, double *x0, double *s0, int nl, double *lag0,
		     double *dir, double miu, double *x1, double *s1,
		     double *lag1, double *hss, int reset)
{
  int l, i;
  double alpha, alpha0, dif, phi0, phiomega, ngd, nd;
  double nbeta;
  static double beta=0;
  static int watch=0;


#if defined __linux__ || defined linux
  double gdx[nx], gds[nl], fgd[nx], rho[nl], jac[nx*nl];
#else
  double *gdx, *gds, *fgd, *rho, *jac;
  gdx=mymalloc(nx*sizeof(double));
  gds=mymalloc(nl*sizeof(double));
  fgd=mymalloc(nx*sizeof(double));
  rho=mymalloc(nl*sizeof(double));
  jac=mymalloc(nx*nl*sizeof(double));
#endif



  /* compute maximum step allowed */

  alpha0=1.0/0.95;
  for(l=0;l<nl;l++){
    if(dir[nx+nl+l]>0 && -lag0[l]/dir[nx+nl+l]<alpha0){
      alpha0=-lag0[l]/dir[nx+nl+l];
/*        printf("limit on lag %d alpha0=%.20f lag0=%.20f dir=%.20f\n", */
/*  	     l, alpha0, lag0[l], dir[nx+nl+l]); */
    }
    if(dir[nx+l]<0 && -(s0[l]+Sip_Opt.pf_tau)/dir[nx+l]<alpha0){
      alpha0=-(s0[l]+Sip_Opt.pf_tau)/dir[nx+l];
/*        printf("limit on slack %d alpha0=%.20f s0=%.20f dir=%.20f\n", */
/*  	     l, alpha0, s0[l], dir[nx+l]); */
    }
  }
  
  if(alpha0<0)
    alpha0=0;

  alpha0=0.95*alpha0;

  alpha=alpha0;
/*    printf("Alpha0=%f\n", alpha0); */

  if(reset){
    beta=0;
    if(Sip_Opt.watchdog){
      watch=0;
      push_pop_sol(PUSH, nx, x0, nl, s0, lag0, dir, &alpha, &beta, &phi0,
		   hss);
    }
  }


  nd=0.0;
  for(l=0;l<nx;l++){
    nd+=dir[l]*dir[l];
  }
  nd=sqrt(nd);


  nbeta=beta;
  if(Sip_Opt.watchdog){
    if(!push_pop_sol(POP_PHI, nx, x0, nl, s0, lag0, dir, &alpha, &beta, &phi0,
		     hss)) 
      phi0=intpkkt_merit(nx, x0, s0, nl, lag0, miu, nbeta, gdx, gds, fgd, rho,
			 jac);
    printf("recovered phi0=%f\n", phi0);
  }


  if(Sip_Opt.watchdog && watch<Sip_Opt.watchmax){

    /* try if a step one is allowed */

    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    for(l=0;l<nl;l++){
      s1[l]=s0[l]+alpha*dir[l+nx];
      lag1[l]=lag0[l]+alpha*dir[nx+nl+l];
    }
    phiomega=intpkkt_merit(nx, x1, s1, nl, lag1, miu, beta, NULL, NULL,
			   NULL, rho, NULL);
/*      printf("Trying first step\nalpha=%f phiomega=%f\n", alpha, phiomega); */
  
    dif=phi0-phiomega;
    if(dif<0){ /*  -(Sip_Opt.armijo)*alpha*ngd){ */
      if(watch==0){
	/* push and exit */
	printf("Making push\n");
	push_pop_sol(PUSH, nx, x0, nl, s0, lag0, dir, &alpha, &beta, &phi0,
		     hss);
      }
      watch++;
/*        printf("Increasing watch=%d\n", watch); */

#if !defined __linux__ && !defined linux
      free(gdx);
      free(gds);
      free(fgd);
      free(rho);
      free(jac);
#endif
      return;
    } else {
      watch=0;
/*        printf("Reseting watch=%d\n", watch); */
/*        printf("Step=%f\n",alpha); */
/*        printf("Making push of phi\n"); */
      push_pop_sol(PUSH, nx, x1, nl, s1, lag1, dir, &alpha, &beta, &phiomega,
		   hss);
#if !defined __linux__ && !defined linux
      free(gdx);
      free(gds);
      free(fgd);
      free(rho);
      free(jac);
#endif
      return;
    }
  }

  if(Sip_Opt.watchdog){
    if(watch>=Sip_Opt.watchmax){
      printf("Making pop\n");
      push_pop_sol(POP, nx, x0, nl, s0, lag0, dir, &alpha, &beta, &phi0, hss);
/*        printf("X0:\n"); */
/*        printarray(x0, nxsip); */
/*        printf("S0:\n"); */
/*        printarray(s0, ntsipc); */
/*        printf("LAG0:\n"); */
/*        printarray(lag0, ntsipc); */
/*        printf("Dir:\n"); */
/*        printarray(dir, nxsip+2*ntsipc); */
/*        printf("Alpha=%f\nBeta=%f\nphi0=%f\n", alpha, beta, phi0); */

      alpha0=alpha;
      alpha/=2;
    }
    watch=0;
  }


  phi0=intpkkt_merit(nx, x0, s0, nl, lag0, miu, nbeta, gdx, gds, fgd, rho,
		     jac);

/*    printf("phi0=%f\n", phi0); */
  
  ngd=0.0;
  while(ngd>=0){

    ngd=0.0;
    for(l=0;l<nx;l++){
      ngd+=gdx[l]*dir[l];
    }
    for(l=0;l<nl;l++)
      ngd+=gds[l]*dir[nx+l];

    if(Sip_Opt.linesearch==ALMERIT)
      for(l=0;l<nl;l++)
	ngd-=rho[l]*dir[nx+nl+l];

/*      printf("ngd=%f\n", ngd); */
    if(ngd>=0){
      if(nbeta<=0){
	nbeta=0.1;
      } else {
	nbeta*=10;
      }
/*        printf("Increasing beta %f\n", nbeta); */
      if(nbeta>1000*(1/Sip_Opt.pf_tau)){
	alpha=0.0;
	beta=nbeta;
	memcpy(x1,   x0,   nx*sizeof(double));
	memcpy(s1,   s0,   nl*sizeof(double));
	memcpy(lag1, lag0, nl*sizeof(double));
#if !defined __linux__ && !defined linux
	free(gdx);
	free(gds);
	free(fgd);
	free(rho);
	free(jac);
#endif
	return;
      }
      /* nbeta increased, compute new gdx and new gds*/
      memcpy(gdx, fgd, nx*sizeof(double));
      memset(gds, 0, nl*sizeof(double));
      if(Sip_Opt.linesearch==ALMERIT){
	for(i=0;i<nl;i++){
	  gds[i]-=lag0[i];
	  for(l=0;l<nx;l++)
	    gdx[l]-=jac[i*nx+l]*lag0[i];
	}
      }
	
      for(i=0;i<nl;i++){
	gds[i]-=miu/(s0[i]+Sip_Opt.pf_tau)-nbeta*rho[i];
	for(l=0;l<nx;l++)
	  gdx[l]+=nbeta*jac[i*nx+l]*rho[i];
      }
      
    }
  }

/*    printf("phi0=%f\n", phi0); */
/*    printf("Beta=%f\n", beta); */
  
  if(nbeta!=beta)
    beta=nbeta*10;

  phi0=intpkkt_merit(nx, x0, s0, nl, lag0, miu, beta, gdx, gds, fgd, rho,
		     jac);

  printf("phi0=%f\n", phi0);
  printf("Beta=%f\n", beta);





  printf("Trying backtracking\n");
  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    for(l=0;l<nl;l++){
      s1[l]=s0[l]+alpha*dir[l+nx];
      lag1[l]=lag0[l]+alpha*dir[nx+nl+l];
    }
    phiomega=intpkkt_merit(nx, x1, s1, nl, lag1, miu, beta, NULL, NULL,
			   NULL, NULL, NULL);
    printf("alpha=%f phiomega=%f\n", alpha, phiomega);
    
    dif=phi0-phiomega;
    if(dif<0){ /*  -(Sip_Opt.armijo)*alpha*ngd){ */
      alpha=alpha/2;
      if(alpha*nd<=1e-8){
	/* this should not happen */
	alpha=alpha0;
	for(l=0;l<nx;l++)
	  x1[l]=x0[l]+alpha*dir[l];
	for(l=0;l<nl;l++){
	  s1[l]=s0[l]+alpha*dir[l+nx];
	  lag1[l]=lag0[l]+alpha*dir[nx+nl+l];
	  }
	  /*  	  memcpy(x1, x0, nx*sizeof(double)); */
	  /*  	  memcpy(s1, s0, nl*sizeof(double)); */
	  /*  	  memcpy(lag1, lag0, nl*sizeof(double)); */
	if(Sip_Opt.watchdog){
	  watch=0;
/*  	  printf("Reseting watch=%d\n", watch); */
/*  	  printf("Making push of phi\n"); */
	  push_pop_sol(PUSH, nx, x1, nl, s1, lag1, dir, &alpha, &beta,
		       &phiomega, hss);
	}
#if !defined __linux__ && !defined linux
	free(gdx);
	free(gds);
	free(fgd);
	free(rho);
	free(jac);
#endif
	return;
      }
    } else {
      printf("Step=%f\n",alpha);
      if(Sip_Opt.watchdog){
	watch=0;
/*  	printf("Reseting watch=%d\n", watch); */
/*  	printf("Making push of phi\n"); */
	push_pop_sol(PUSH, nx, x1, nl, s1, lag1, dir, &alpha, &beta,
		     &phiomega, hss);
      }
#if !defined __linux__ && !defined linux
      free(gdx);
      free(gds);
      free(fgd);
      free(rho);
      free(jac);
#endif
      return;
    }
  }
#if !defined __linux__ && !defined linux
  free(gdx);
  free(gds);
  free(fgd);
  free(rho);
  free(jac);
#endif

}



double intpkkt_merit(int nx, double *x, double *s, int nl, double *lag,
		     double miu, double beta, double *gradx, double *grads,
		     double *fgd, double *rho, double *jac)
{
  double merval, num, den, term1, term2, term3, dummy;
  int i, j;

#if defined __linux__ || defined linux
  double tsample[ntsip];
#else
  double *tsample;
  tsample=mymalloc(ntsip*sizeof(double));
#endif

  
  stats.nsip_mer++;
  
  term1=0.0;
  term2=0.0;
  term3=0.0;

  if(gradx)
    memset(gradx, 0, nx*sizeof(double));
  if(grads)
    memset(grads, 0, nl*sizeof(double));

  for(i=0; i<nl; i++){
    if(gradx){
      adapt_trap(&gxtepsilon, TBL, TBU, nx, x, ntsip, tsample,
		 Sip_Opt.adapt_int_n, i, &num, &den,
		 Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		 FIRST_DERIV, fgd, 0);
    } else {
      adapt_trap(&gxtepsilon, TBL, TBU, nx, x, ntsip, tsample,
		 Sip_Opt.adapt_int_n, i, &num, &den,
		 Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		 NO_DERIV, NULL, 0);
    }
    term1+=mylog(s[i]+Sip_Opt.pf_tau);
    term2+=pow(num+s[i], 2.0);
    term3+=lag[i]*(num+s[i]);

    if(rho)
      rho[i]=num+s[i];

    if(jac)
      memcpy(&jac[i*nx], fgd, nx*sizeof(double));

    if(gradx)
      switch(Sip_Opt.linesearch){
      case L2MERIT:
	for(j=0;j<nx;j++)
	  gradx[j]+=beta*fgd[j]*(num+s[i]);
	break;
      case ALMERIT:
	for(j=0;j<nx;j++)
	  gradx[j]+=beta*fgd[j]*(num+s[i])-lag[i]*fgd[j];
	break;
      }

    if(grads){
      switch(Sip_Opt.linesearch){
      case L2MERIT:
	grads[i]=-miu/(s[i]+Sip_Opt.pf_tau)+beta*(num+s[i]);
	break;
      case ALMERIT:
	grads[i]=-miu/(s[i]+Sip_Opt.pf_tau)-lag[i]+beta*(num+s[i]);
	break;
      }
    }
  }

  /*    printf("Merit term1 (optima  log)=%.20f\n", term1); */
  /*    printf("Merit term2 (feasibility)=%.20f\n", term2); */
  
  switch(Sip_Opt.linesearch){
  case L2MERIT:
    merval=eval_sip(SIP_OBJVAL, 0, x, NULL, NULL, NULL, NULL, NULL)
      -miu*term1+beta*term2/2;
    break;
  case ALMERIT:
    merval=eval_sip(SIP_OBJVAL, 0, x, NULL, NULL, NULL, NULL, NULL)
      -miu*term1-term3+beta*term2/2;
    break;
  }


  if(gradx){
    dummy=eval_sip(SIP_OBJGRD, 0, x, NULL, fgd, NULL, NULL, NULL);
    for(j=0;j<nx;j++)
      gradx[j]+=fgd[j];
  }

#if !defined __linux__ && !defined linux
  free(tsample);
#endif
  
  return merval;
}



double mylog(double x)
{
  if(x<=0){
    printf("Log of a negative or zero value %f, returning %f\n",x, -1E+20);
    return -1E+20;
  }


  return log(x);
}


int int_stop(int nx, int nl, double *fgd, double *lag, double *s,
	     double *jac, double *term, double *primal, double *dual)
{
  double rho_2;
  double sigma_2;
  int i,l;

#if defined __linux__ || defined linux
  double auxv[nx];
#else
  double *auxv;
  auxv=mymalloc(nx*sizeof(double));
#endif


  rho_2=0.0;
  for(l=0;l<nl;l++){
    rho_2+=pow(term[l]+s[l],2);
  }

  *primal=rho_2=sqrt(rho_2);


  memset(auxv, 0, nx*sizeof(double));

  for(i=0;i<nx;i++){
    for(l=0;l<nl;l++){
      auxv[i]+=jac[l*nx+i]*lag[l];
    }
  }

  sigma_2=0.0;
  for(i=0;i<nx;i++){
    sigma_2+=pow(-fgd[i]+auxv[i],2.0);
  }

  *dual=sigma_2=sqrt(sigma_2);

/*    printf("||rho||=%f\n", rho_2); */

/*    printf("||sigma||=%f\n", sigma_2); */

  /*    *primal=rho_2; */
  /*    *dual=sigma_2; */

  /*    return 1; */


#if !defined __linux__ && !defined linux
  free(auxv);
#endif


  if(rho_2>Sip_Opt.int_prec) /* primal feasibility not attained */
    return 0;

  if(sigma_2>Sip_Opt.int_prec)  /* dual feasibility not attained */
    return 0;


  return 1;
}


void second_order(int nx, int nl, double *w, double *jac, double *rho)
{
  int i, j, l, sing;

#if defined __linux__ || defined linux
  double aux[nl*nl], sol[nx];
#else
  double *aux, *sol;
  aux=mymalloc(nl*nl*sizeof(double));
  sol=mymalloc(nx*sizeof(double));
#endif


  for(i=0;i<nl;i++){
    for(j=0;j<nl;j++){
      aux[i+j*nl]=0.0;
      for(l=0;l<nx;l++){
	aux[i+j*nl]=jac[i*nx+l]*jac[j*nx+l];
      }
    }
  }

  egpp(nl, aux, rho, sol, &sing);

  if(sing){
    printf("Second order inverse is singular\n");
    memset(w, 0, nx*sizeof(double));
#if !defined __linux__ && !defined linux
    free(aux);
    free(sol);
#endif
    return;
  }

  for(i=0;i<nx;i++){
    w[i]=0.0;
    for(j=0;j<nl;j++){
      w[i]=jac[j*nx+i]*sol[j];
    }
  }

#if !defined __linux__ && !defined linux
    free(aux);
    free(sol);
#endif

}



void armi_bakry(int nx, double *x0, double *s0, int nl, double *lag0,
		double *dir, double miu, double *x1, double *s1, double *lag1)
{
  int l;
  double alpha0, nd;
  double alpha, dif;
  double phi0, phiomega;



  phi0=intpbakry_merit(nx, x0, s0, nl, lag0);

  printf("phi0=%f\n", phi0);

  alpha0=1.0;
  for(l=0;l<nl;l++){
    if(dir[nx+nl+l]>0 && -lag0[l]/dir[nx+nl+l]<alpha0){
      alpha0=-lag0[l]/dir[nx+nl+l];
      printf("limit on lag %d alpha0=%f lag0=%.20f dir=%.20f\n",
	     l, alpha0, lag0[l], dir[nx+nl+l]);
    }
    if(dir[nx+l]<0 && -(s0[l]+Sip_Opt.pf_tau)/dir[nx+l]<alpha0){
      alpha0=-(s0[l]+Sip_Opt.pf_tau)/dir[nx+l];
      printf("limit on slack %d alpha0=%f s0=%.20f dir=%.20f\n",
	     l, alpha0, s0[l], dir[nx+l]);
    }
  }
  
  if(alpha0<0)
    alpha0=0;

  if(alpha0!=1)
    alpha0=0.95*alpha0;

  alpha=alpha0;
  printf("Alpha0=%f\n", alpha0);


  nd=0.0;
  for(l=0;l<nx;l++){
    nd+=dir[l]*dir[l];
  }
  nd=sqrt(nd);

  /*    for(l=0;l<nx;l++) */
  /*      x1[l]=x0[l]+alpha*dir[l]; */
  /*    for(l=0;l<nl;l++){ */
  /*      s1[l]=s0[l]+alpha*dir[l+nx]; */
  /*      lag1[l]=lag0[l]+alpha*dir[nx+nl+l]; */
  /*    } */
  /*    return; */

  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    for(l=0;l<nl;l++){
      s1[l]=s0[l]+alpha*dir[l+nx];
      lag1[l]=lag0[l]+alpha*dir[nx+nl+l];
    }
    phiomega=intpbakry_merit(nx, x1, s1, nl, lag1);
    printf("alpha=%f phiomega=%f\n", alpha, phiomega);
    
    dif=phi0-phiomega;
    if(dif<0){ /*  -(Sip_Opt.armijo)*alpha*ngd){ */
      alpha=alpha/2;
      if(alpha*nd<=1e-8){
	/* this should not happen */
	printf("Possible mistake in merit function\n");
	for(l=0;l<nx;l++)
	  x1[l]=x0[l]+alpha0*dir[l];
	for(l=0;l<nl;l++){
	  s1[l]=s0[l]+alpha0*dir[l+nx];
	  lag1[l]=lag0[l]+alpha0*dir[nx+nl+l];
	}
	/*  	memcpy(x1, x0, nx*sizeof(double)); */
	/*  	memcpy(s1, s0, nl*sizeof(double)); */
	/*  	memcpy(lag1, lag0, nl*sizeof(double)); */
	return;
      }
    } else {
      printf("Step=%f\n",alpha);
      return;
    }
  }


}


double intpbakry_merit(int nx, double *x, double *s, int nl, double *lag)
{
  int i, l;
  double num, den;
  double primal_f, dual_f, comp, dummy;

#if defined __linux__ || defined linux
  double tsample[ntsip], gd[nx], sigma[nx];
#else
  double *tsample, *gd, *sigma;
  gd=mymalloc(nx*sizeof(double));
  sigma=mymalloc(nx*sizeof(double));
  tsample=mymalloc(ntsip*sizeof(double));
#endif



  comp=primal_f=dual_f=0.0;
  memset(sigma, 0, nx*sizeof(double));

  for(i=0; i<nl; i++){
    adapt_trap(&gxtepsilon, TBL, TBU, nx, x, ntsip, tsample,
	       Sip_Opt.adapt_int_n, i, &num, &den,
	       Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
	       FIRST_DERIV, gd, 0);

    primal_f+=pow(num+s[i], 2.0);
    printf("lag[%d]=%f\ts[%d]=%f\n", i, lag[i], i, s[i]);
    comp+=pow(lag[i]*(s[i]+Sip_Opt.pf_tau), 2.0);

    for(l=0;l<nx;l++)
      sigma[l]+=gd[l]*lag[i];
  }

  dummy=eval_sip(SIP_OBJGRD, 0, x, NULL, gd, NULL, NULL, NULL);

  for(l=0;l<nx;l++)
    dual_f+=pow(sigma[l]-gd[l], 2.0);

  printf("primal_f=%f\ncomp=%.20f\ndual_f=%f\n", primal_f, comp, dual_f);

#if !defined __linux__ && !defined linux
    free(tsample);
    free(gd);
    free(sigma);
#endif

  return dual_f+comp+primal_f;
}





void armijo_intdual(int nx, double *x0, double *s0, int nl, double *lag0,
		     double *dir, double miu, double *x1, double *s1,
		     double *lag1)
{
  int l, i;
  double alpha, alpha0, dif, phi0, phiomega, ngd, nd;
  static double beta;

#if defined __linux__ || defined linux
  double gdx[nx], gds[nl], fgd[nx], rho[nl], jac[nx*nl];
#else
  double *gdx, *gds, *fgd, *rho, *jac;
  gdx=mymalloc(nx*sizeof(double));
  fgd=mymalloc(nx*sizeof(double));
  gds=mymalloc(nl*sizeof(double));
  rho=mymalloc(nl*sizeof(double));
  jac=mymalloc(nx*nl*sizeof(double));
#endif


  /*    beta=0; */
  phi0=intpkkt_merit(nx, x0, s0, nl, lag0, miu, beta, gdx, gds, fgd, rho, jac);
  
  
  ngd=0.0;
  while(ngd>=0){

    ngd=0.0;
    for(l=0;l<nx;l++){
      ngd+=gdx[l]*dir[l];
    }
    for(l=0;l<nl;l++)
      ngd+=gds[l]*dir[nx+l];

    printf("ngd=%f\n", ngd);
    if(ngd>=0){
      if(beta<=0){
	beta=0.1;
      } else {
	beta*=10;
      }
      printf("Increasing beta %f\n", beta);
      if(beta>1000*(1/Sip_Opt.pf_tau)){
	alpha=0.0;
	memcpy(x1,   x0,   nx*sizeof(double));
	memcpy(s1,   s0,   nl*sizeof(double));
	memcpy(lag1, lag0, nl*sizeof(double));

#if !defined __linux__ && !defined linux
  free(gdx);
  free(gds);
  free(fgd);
  free(rho);
  free(jac);
#endif
	return;
      }
      /* beta increased, compute new gdx and new gds*/
      memcpy(gdx, fgd, nx*sizeof(double));
      for(i=0;i<nl;i++){
	for(l=0;l<nx;l++){
	  gdx[l]+=beta*jac[i*nx+l]*rho[i];
	}
      }

      for(i=0;i<nl;i++){
	gds[i]=-miu/(s0[i]+Sip_Opt.pf_tau)+beta*rho[i];
      }

    }
  }

  phi0=intpkkt_merit(nx, x0, s0, nl, lag0, miu, beta, gdx, gds, fgd, rho, jac);

  printf("phi0=%f\n", phi0);
  printf("Beta=%f\n", beta);

  alpha0=1.0;
  for(l=0;l<nl;l++){
    if(dir[nx+l]<0 && -(s0[l]+Sip_Opt.pf_tau)/dir[nx+l]<alpha0){
      alpha0=-(s0[l]+Sip_Opt.pf_tau)/dir[nx+l];
      printf("limit on slack %d alpha0=%f s0=%.20f dir=%.20f\n",
	     l, alpha0, s0[l], dir[nx+l]);
    }
  }
  
  if(alpha0<0)
    alpha0=0;

  if(alpha0!=1)
    alpha0=0.99995*alpha0;

  alpha=alpha0;
  printf("Alpha0=%f\n", alpha0);


  nd=0.0;
  for(l=0;l<nx;l++){
    nd+=dir[l]*dir[l];
  }
  nd=sqrt(nd);


  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    for(l=0;l<nl;l++)
      s1[l]=s0[l]+alpha*dir[l+nx];
    phiomega=intpkkt_merit(nx, x1, s1, nl, lag0, miu, beta, NULL, NULL,
			   NULL, NULL, NULL);
    printf("alpha=%f phiomega=%f\n", alpha, phiomega);
    
    dif=phi0-phiomega;
    if(dif<0){ /*  -(Sip_Opt.armijo)*alpha*ngd){ */
      alpha=alpha/2;
      if(alpha*nd<=1e-8){
	/* this should not happen */
	alpha=alpha0;
	for(l=0;l<nx;l++)
	  x1[l]=x0[l]+alpha*dir[l];
	for(l=0;l<nl;l++)
	  s1[l]=s0[l]+alpha*dir[l+nx];
	break;
      }
    } else {
      printf("Step=%f\n",alpha);
      break;
    }
  }

    
  phi0=intdual_merit(nx, x0, s0, nl, lag0);
  printf("phi0=%f\n", phi0);

  alpha0=1.0;
  for(l=0;l<nl;l++){
    if(dir[nx+nl+l]>0 && -lag0[l]/dir[nx+nl+l]<alpha0){
      alpha0=-lag0[l]/dir[nx+nl+l];
      printf("limit on lag %d alpha0=%f lag0=%.20f dir=%.20f\n",
	     l, alpha0, lag0[l], dir[nx+nl+l]);
    }
  }
  
  if(alpha0<0)
    alpha0=0;

  if(alpha0!=1)
    alpha0=0.99995*alpha0;

  alpha=alpha0;
  printf("Alpha0=%f\n", alpha0);

  for(;;){
    for(l=0;l<nl;l++)
      lag1[l]=lag0[l]+alpha*dir[l+nx+nl];
    phiomega=intdual_merit(nx, x0, s0, nl, lag1);
    printf("alpha=%f phiomega=%f\n", alpha, phiomega);
    
    dif=phi0-phiomega;
    if(dif<0){ /*  -(Sip_Opt.armijo)*alpha*ngd){ */
      alpha=alpha/2;
      if(alpha*nd<=1e-8){
	/* this should not happen */
	alpha=alpha0;
	for(l=0;l<nl;l++)
	  lag1[l]=lag0[l]+alpha*dir[l+nx+nl];
	break;
      }
    } else {
      printf("Step=%f\n",alpha);
      break;
    }
  }

#if !defined __linux__ && !defined linux
  free(gdx);
  free(gds);
  free(fgd);
  free(rho);
  free(jac);
#endif

  return;
}



double intdual_merit(int nx, double *x, double *s, int nl, double *lag)
{
  int i, l;
  double num, den;
  double dual_f, comp, dummy;

#if defined __linux__ || defined linux
  double tsample[ntsip], gd[nx], sigma[nx];
#else
  double *gd, *tsample, *sigma;
  gd=mymalloc(nx*sizeof(double));
  tsample=mymalloc(ntsip*sizeof(double));
  sigma=mymalloc(nx*sizeof(double));
#endif



  dual_f=0.0;
  memset(sigma, 0, nx*sizeof(double));

  for(i=0; i<nl; i++){
    adapt_trap(&gxtepsilon, TBL, TBU, nx, x, ntsip, tsample,
	       Sip_Opt.adapt_int_n, i, &num, &den,
	       Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
	       FIRST_DERIV, gd, 0);

    comp+=pow(lag[i]*(s[i]+Sip_Opt.pf_tau), 2.0);    
    for(l=0;l<nx;l++)
      sigma[l]+=gd[l]*lag[i];
  }

  dummy=eval_sip(SIP_OBJGRD, 0, x, NULL, gd, NULL, NULL, NULL);

  for(l=0;l<nx;l++)
    dual_f+=pow(sigma[l]-gd[l], 2.0);

  printf("dual_f=%f comp=%f\n", dual_f, comp);
#if !defined __linux__ && !defined linux
  free(tsample);
  free(gd);
  free(sigma);
#endif

  return dual_f+comp;
}



int push_pop_sol(int op, int nx, double *x, int nl, double *s, double *lag,
		 double *dir, double *alpha, double *beta, double *phi,
		 double *hss)
{
  static double *x_sol=NULL, *s_sol=NULL, *lag_sol=NULL, *dir_sol=NULL,
    *hss_sol, alpha_sol, beta_sol, phi_sol;


  if(op==POP && (!x_sol || !s_sol || !lag_sol || !dir_sol || !hss_sol)){
    printf("POP without a PUSH\n I have nothing to return!!!\n");
    return FALSE;
  }

  if(op==POP_PHI && (!x_sol || !s_sol || !lag_sol || !dir_sol || !hss_sol)){
    return FALSE;
  }

  switch(op){
  case POP:
    if(x_sol)
      memcpy(x, x_sol, nx*sizeof(double));
    if(s_sol)
      memcpy(s, s_sol, nl*sizeof(double));
    if(lag_sol)
      memcpy(lag, lag_sol, nl*sizeof(double));
    if(dir_sol)
      memcpy(dir, dir_sol, (nx+2*nl)*sizeof(double));
    if(hss_sol)
      memcpy(hss, hss_sol, (nx*nx)*sizeof(double));
    *alpha=alpha_sol;
    *beta=beta_sol;
    *phi=phi_sol;
    break;
  case PUSH:
    if(!x_sol)
      x_sol=mymalloc(nx*sizeof(double));
    if(!s_sol)
      s_sol=mymalloc(nl*sizeof(double));
    if(!lag_sol)
      lag_sol=mymalloc(nl*sizeof(double));
    if(!dir_sol)
      dir_sol=mymalloc((nx+2*nl)*sizeof(double));
    if(!hss_sol)
      hss_sol=mymalloc((nx*nx)*sizeof(double));
    if(x)
      memcpy(x_sol, x, nx*sizeof(double));
    if(s)
      memcpy(s_sol, s, nl*sizeof(double));
    if(lag)
      memcpy(lag_sol, lag, nl*sizeof(double));
    if(dir)
      memcpy(dir_sol, dir, (nx+2*nl)*sizeof(double));
    if(hss)
      memcpy(hss_sol, hss, (nx*nx)*sizeof(double));
    alpha_sol=*alpha;
    beta_sol=*beta;
    phi_sol=*phi;
    break;
  case POP_PHI:
    *phi=phi_sol;
    break;
  case CLEAN:
    if(x_sol)
      free(x_sol);
    if(s_sol)
      free(s_sol);
    if(lag_sol)
      free(lag_sol);
    if(dir_sol)
      free(dir_sol);
    if(hss_sol)
      free(hss_sol);
    alpha_sol=beta_sol=0;
    break;
  }

  return TRUE;
}


void GetInitialGuess(int nx, double *x, double *x1, int nl, double *s1,
		     double *lag1)
{
  int i, j, l, sing;
  double num, den;

#if defined __linux__ || defined linux
  double tsample[ntsip], gd[nx];
  double jac[nx*nl], xaux[nx*nx], laux[nl*nl], xauxv[nx], lauxv[nl], gbar[nl];
#else
  double *gd, *tsample;
  double *jac, *xaux, *laux, *xauxv, *lauxv, *gbar;
  gd=mymalloc(nx*sizeof(double));
  tsample=mymalloc(ntsip*sizeof(double));
  xauxv=mymalloc(nx*sizeof(double));
  lauxv=mymalloc(nl*sizeof(double));
  gbar=mymalloc(nl*sizeof(double));
  xaux=mymalloc(nx*nx*sizeof(double));
  jac=mymalloc(nx*nl*sizeof(double));
  laux=mymalloc(nl*nl*sizeof(double));
#endif



  /*  Equality variables */

  for(l=0;l<ntsipc;l++){
    adapt_trap(&gxtepsilon, TBL, TBU, nxsip, x, ntsip, tsample,
	       Sip_Opt.adapt_int_n, l, &num, &den,
	       Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
	       FIRST_DERIV, gd, 0);
    memcpy(&(jac[l*nx]), gd, nx*sizeof(double));
    gbar[l]=num;
    if(num<Sip_Opt.theta){
      s1[l]=Sip_Opt.theta;
    } else {
      s1[l]=num;
    }
  }

  /* primal initial guess */
  for(i=0;i<nx;i++){
    for(j=0;j<nx;j++){ /* symmetric, some operations could be avoided */
      xaux[i*nx+j]=0.0;
      for(l=0;l<nl;l++)
	xaux[i*nx+j]+=jac[l*nx+i]*jac[l*nx+j];
    }
  }

  eval_sip(SIP_OBJGRD, 0, x, NULL, gd, NULL, NULL, NULL);

  for(i=0;i<nx;i++){
    xauxv[i]=-gd[i];
    for(l=0;l<nl;l++)
      xauxv[i]+=jac[l*nx+i]*gbar[l];
  }
      
  mod_cholesky(nx, xaux, xauxv, x1, Sip_Opt.choleps, &sing);
  
  if(sing)
    memcpy(x1, x, nx*sizeof(double));
    
  

  /* Lagrange multipliers */

  for(i=0;i<nl;i++){
    for(j=0;j<nl;j++){ /* symmetric, some operations could be avoided */
      laux[i*nl+j]=0.0;
      for(l=0;l<nx;l++)
	laux[i*nl+j]+=jac[i*nx+l]*jac[j*nx+l];
    }
  }

  for(i=0;i<nl;i++){
    lauxv[i]=0.0;
    for(j=0;j<nx;j++)
      lauxv[i]+=jac[i*nx+j]*gd[j];
  }


  mod_cholesky(nl, laux, lauxv, lag1, Sip_Opt.choleps, &sing);

  if(sing)
    for(i=0;i<nl;i++)
      lag1[i]=-1;

  /* checkup on Lagrange multipliers */

  for(l=0;l<ntsipc;l++)
    if(lag1[l]>=0)
      lag1[l]=-1;

#if !defined __linux__ && !defined linux
  free(tsample);
  free(gd);
  free(jac);
  free(xaux);
  free(laux);
  free(xauxv);
  free(lauxv);
  free(gbar);
#endif

}


#ifdef NPSOL


/**************************************************************
NPSOL to solve the transcribed problem
**************************************************************/
extern int npsol_(long *n, long *nclin, long *ncnln, long *nrowa,
        long *nrowuj, long *nrowr, double *a, double *bl, double *bu,
        void (*confun) (), void (*objfun) (), long *inform, long *iter,
        long *istate, double *c, double *ujac, double *clamda, double *objf,
        double *ugrad, double *r, double *x, long *iw, long *leniw,
        double *w, long *lenw);



void int_npsol_objfun(long *MODE, long *N, double *X0, double *F, double *G,
		  long *NSTATE)
{
  if (*MODE != 1)
    *F = eval_sip(SIP_OBJVAL, 0, X0, NULL, NULL, NULL, NULL, NULL);
  if (*MODE > 0) {
    eval_sip(SIP_OBJGRD, 0, X0, NULL, G, NULL, NULL, NULL);
  }
}


void int_npsol_confun(long *MODE, long *NCNLN, long *N, long *NROWJ, long *NEEDC,
		  double *X0, double *C, double *CJAC, long *NSTATE)
{
  int i,k;
  double den;

#if defined __linux__ || defined linux
  double tsample[ntsip], gd[nxsip];
#else
  double *tsample, *gd;
  gd=mymalloc(nxsip*sizeof(double));
  tsample=mymalloc(ntsip*sizeof(double));
#endif

  

  for(i=0;i<ntsipc;i++){
    if(*MODE==0) /* NEEDS constraint value */
      adapt_trap(&gxtepsilon, TBL, TBU, nxsip, X0, ntsip, tsample,
		 Sip_Opt.adapt_int_n, i, &C[i], &den,
		 Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		 NO_DERIV, NULL, 0);
    if(*MODE>0){ /* NEEDS constraint gradient */
      adapt_trap(&gxtepsilon, TBL, TBU, nxsip, X0, ntsip, tsample,
		 Sip_Opt.adapt_int_n, i, &C[i], &den,
		 Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		 FIRST_DERIV, gd, 0);
      for(k=0;k<nxsip;k++)
	CJAC[i+k*(*NROWJ)]=gd[k];
    }
  }


#if !defined __linux__ && !defined linux
  free(tsample);
  free(gd);
#endif

  return;
}



double int_solve_nlp()
{
  double *npsol_bl, *npsol_bu, *npsol_a, *npsol_c, *npsol_cjac, *npsol_clambda;
  double *npsol_objgrd, *npsol_r, *npsol_w;
  double npsol_objf;
  long npsol_n, npsol_nclin, npsol_ncnln, npsol_nrowa, npsol_nrowj;
  long npsol_nrowr;
  long npsol_inform, npsol_iter;
  long *npsol_iw, *npsol_istate, npsol_leniw, npsol_lenw;
  int i, coni;

  
  npsol_n=nxsip;
  npsol_nclin=0; /* no linear constraints */
  npsol_nrowa=0;
  npsol_a=NULL;

  npsol_ncnln=ntsipc;

  npsol_bl=mymalloc((nxsip+npsol_ncnln)*sizeof(double));
  npsol_bu=mymalloc((nxsip+npsol_ncnln)*sizeof(double));

  coni=0; /* coni will count for constraints and variables */

  /* variables bounds */
  for(i=0;i<nxsip;i++){
    npsol_bl[coni]=XBL[i];
    npsol_bu[coni++]=XBU[i];
  }

  /* infinite constraints bounds */
  for(i=0;i<ntsipc;i++){ /* for all infinite constraints */
    npsol_bl[coni]=-1e-20;
    npsol_bu[coni++]=Sip_Opt.pf_tau;
  }


  printf("npsol coni=%d\n",coni);
  printf("%d restricoes para o NPSOL\n", npsol_ncnln);

  /* allocate space for jacobian */
  npsol_nrowj=npsol_ncnln;
  npsol_cjac=mymalloc(npsol_ncnln*nxsip*sizeof(double));

  /* allocate istate array */
  npsol_istate=mymalloc((nxsip+npsol_ncnln)*sizeof(long));

  /* allocate space for c array */
  npsol_c=mymalloc(npsol_ncnln*sizeof(double));

  /* allocate clambda array */
  npsol_clambda=mymalloc((nxsip+npsol_ncnln)*sizeof(double));

  /* allocate objgrd array */
  npsol_objgrd=mymalloc((nxsip)*sizeof(double));

  /* allocate r matrix */
  npsol_nrowr=nxsip;
  npsol_r=mymalloc((nxsip*npsol_nrowr)*sizeof(double));

  /* allocate workspace for npsol */
  npsol_leniw=3*nxsip+2*npsol_ncnln;
  npsol_iw=mymalloc(npsol_leniw*sizeof(long));
  npsol_lenw=2*nxsip*nxsip+nxsip*npsol_ncnln+
    2*nxsip*npsol_ncnln+20*nxsip+11*npsol_ncnln+21*npsol_ncnln;
  npsol_w=mymalloc(npsol_lenw*sizeof(double));
  
  printf("Starting point for NPSOL\n");
  printarray(X, nxsip);
  
  stats.nnlpc+=npsol_ncnln;
  stats.nnlp++;
  do {
  npsol_(&npsol_n, &npsol_nclin, &npsol_ncnln, &npsol_nrowa, &npsol_nrowj,
	 &npsol_nrowr, npsol_a, npsol_bl, npsol_bu, int_npsol_confun, int_npsol_objfun,
	 &npsol_inform, &npsol_iter, npsol_istate, npsol_c, npsol_cjac,
	 npsol_clambda, &npsol_objf, npsol_objgrd, npsol_r, X, npsol_iw,
	 &npsol_leniw, npsol_w, &npsol_lenw);
  printf("\tNonlinear used %ld iterations Obj=%f Inform=%ld\n",npsol_iter,npsol_objf, npsol_inform);
  } while(npsol_inform==4);

  printf("NPSOL inform=%ld\n", npsol_inform);
  printf("Exit point for NPSOL\n");
  printarray(X, nxsip);

  /* free space */
  free(npsol_bl);
  free(npsol_bu);
  free(npsol_cjac);
  free(npsol_istate);
  free(npsol_c);
  free(npsol_clambda);
  free(npsol_objgrd);
  free(npsol_r);
  free(npsol_iw);
  free(npsol_w);
  return npsol_objf;
}



/**************************************************************
NPSOL to minimize lagrangian
**************************************************************/



void lag_npsol_objfun(long *MODE, long *N, double *X0, double *F, double *G,
		  long *NSTATE)
{
  double fx;
  int i;

#if defined __linux__ || defined linux
  double tsample[ntsip], fgd[nxsip], jac[nxsip*ntsipc], term[ntsipc];
#else
  double *tsample, *fgd, *jac, *term;
  fgd=mymalloc(nxsip*sizeof(double));
  tsample=mymalloc(ntsip*sizeof(double));
  jac=mymalloc(nxsip*ntsipc*sizeof(double));
  term=mymalloc(ntsipc*sizeof(double));
#endif


  if (*MODE == 0)
    *F=lagintpkkt(nxsip, X0, ntsipc, X0+nxsip+ntsipc, X0+nxsip, ntsip, tsample, 0.0001, NO_DERIV, NULL, NULL, NULL, NULL, NULL, &fx);
  if (*MODE > 0) {
    *F=lagintpkkt(nxsip, X0, ntsipc, X0+nxsip+ntsipc, X0+nxsip, ntsip, tsample, 0.0001, FIRST_DERIV, fgd, G, G+nxsip+ntsipc, jac, term, &fx);
    for(i=0;i<ntsipc;i++)
      *(G+nxsip+i)=-0.0001/(*(X0+nxsip+i)+Sip_Opt.pf_tau)-*(X0+nxsip+ntsipc+i);
  }
#if !defined __linux__ && !defined linux
  free(tsample);
  free(fgd);
  free(jac);
  free(term);
#endif

}


void lag_npsol_confun(long *MODE, long *NCNLN, long *N, long *NROWJ,
		      long *NEEDC, double *X0, double *C, double *CJAC,
		      long *NSTATE)
{

  printf("Constraint function called\n");

  return;
}



double lag_solve_nlp()
{
  double *npsol_bl, *npsol_bu, *npsol_a, *npsol_c, *npsol_cjac, *npsol_clambda;
  double *npsol_objgrd, *npsol_r, *npsol_w;
  double npsol_objf;
  long npsol_n, npsol_nclin, npsol_ncnln, npsol_nrowa, npsol_nrowj;
  long npsol_nrowr;
  long npsol_inform, npsol_iter;
  long *npsol_iw, *npsol_istate, npsol_leniw, npsol_lenw;
  int i, coni;


#if defined __linux__ || defined linux
  double start[nxsip+2*ntsipc];
#else
  double *start;
  start=mymalloc((nxsip+2*ntsipc)*sizeof(double));
#endif

  
  npsol_n=nxsip+2*ntsipc;
  npsol_nclin=0; /* no linear constraints */
  npsol_nrowa=0;
  npsol_a=NULL;

  npsol_ncnln=0;

  npsol_bl=mymalloc((npsol_n+npsol_ncnln)*sizeof(double));
  npsol_bu=mymalloc((npsol_n+npsol_ncnln)*sizeof(double));

  coni=0; /* coni will count for constraints and variables */

  /* variables bounds */
  for(i=0;i<nxsip;i++){
    start[coni]=X[coni];
    npsol_bl[coni]=XBL[i];
    npsol_bu[coni++]=XBU[i];
  }

  /* slack variables bounds */
  for(i=0;i<ntsipc;i++){
    start[coni]=1.0;
    npsol_bl[coni]=0;
    npsol_bu[coni++]=1e+20;
  }

  /* lagrange multipliers */
  for(i=0;i<ntsipc;i++){
    start[coni]=-1.0;
    npsol_bl[coni]=-1e+20;
    npsol_bu[coni++]=0;
  }


  printf("npsol coni=%d\n",coni);
  printf("%d restricoes para o NPSOL\n", npsol_ncnln);

  /* allocate space for jacobian */
  npsol_nrowj=npsol_ncnln;
  npsol_cjac=NULL; /*  mymalloc(npsol_ncnln*nxsip*sizeof(double)); */

  /* allocate istate array */
  npsol_istate=mymalloc((npsol_n+npsol_ncnln)*sizeof(long));

  /* allocate space for c array */
  npsol_c=NULL; /*  mymalloc(npsol_ncnln*sizeof(double)); */

  /* allocate clambda array */
  npsol_clambda=mymalloc((npsol_n+npsol_ncnln)*sizeof(double));

  /* allocate objgrd array */
  npsol_objgrd=mymalloc((npsol_n)*sizeof(double));

  /* allocate r matrix */
  npsol_nrowr=npsol_n;
  npsol_r=mymalloc((npsol_n*npsol_nrowr)*sizeof(double));

  /* allocate workspace for npsol */
  npsol_leniw=3*npsol_n+2*npsol_ncnln;
  npsol_iw=mymalloc(npsol_leniw*sizeof(long));
  npsol_lenw=2*npsol_n*npsol_n+npsol_n*npsol_ncnln+
    2*npsol_n*npsol_ncnln+20*npsol_n+11*npsol_ncnln+21*npsol_ncnln;
  npsol_w=mymalloc(npsol_lenw*sizeof(double));
  
  printf("Starting point for NPSOL\n");
  printarray(start, nxsip+2*ntsipc);
  
  stats.nnlpc+=npsol_ncnln;
  stats.nnlp++;
  do {
  npsol_(&npsol_n, &npsol_nclin, &npsol_ncnln, &npsol_nrowa, &npsol_nrowj,
	 &npsol_nrowr, npsol_a, npsol_bl, npsol_bu, lag_npsol_confun, lag_npsol_objfun,
	 &npsol_inform, &npsol_iter, npsol_istate, npsol_c, npsol_cjac,
	 npsol_clambda, &npsol_objf, npsol_objgrd, npsol_r, start, npsol_iw,
	 &npsol_leniw, npsol_w, &npsol_lenw);
  printf("\tNonlinear used %ld iterations Obj=%f Inform=%ld\n",npsol_iter,npsol_objf, npsol_inform);
  } while(npsol_inform==4);

  printf("NPSOL inform=%ld\n", npsol_inform);
  printf("Exit point for NPSOL\n");
  printarray(start, nxsip+2*ntsipc);

  /* free space */
  free(npsol_bl);
  free(npsol_bu);
  free(npsol_cjac);
  free(npsol_istate);
  free(npsol_c);
  free(npsol_clambda);
  free(npsol_objgrd);
  free(npsol_r);
  free(npsol_iw);
  free(npsol_w);
#if !defined __linux__ && !defined linux
  free(start);
#endif

  return npsol_objf;
}





#else

double int_solve_nlp()
{

  printf("\n\nNPSOL not available\nPlease read the user manual about it\n");
  printf("Aborting....\n");
  exit(1);

  return 0; /* don't complain */

}


#endif
