#include "solver.h"


extern SIP_S sip_s;
extern Mesh T_Set;
extern STATS stats;


#ifdef NPSOL

/*****************************************
NPSOL to solve the local Nonlinear Program
*****************************************/
extern int npsol_(long *n, long *nclin, long *ncnln, long *nrowa,
        long *nrowuj, long *nrowr, double *a, double *bl, double *bu,
        void (*confun) (), void (*objfun) (), long *inform, long *iter,
        long *istate, double *c, double *ujac, double *clamda, double *objf,
        double *ugrad, double *r, double *x, long *iw, long *leniw,
        double *w, long *lenw);

/*  extern void printarray(double *a, int n); */


void npsol_objfun(long *MODE, long *N, double *X0, double *F, double *G,
		  long *NSTATE)
{
  if (*MODE != 1)
    *F = eval_sip(SIP_OBJVAL, 0, X0, NULL, NULL, NULL, NULL, NULL);
  if (*MODE > 0) {
    eval_sip(SIP_OBJGRD, 0, X0, NULL, G, NULL, NULL, NULL);
  }
}


void npsol_confun(long *MODE, long *NCNLN, long *N, long *NROWJ, long *NEEDC,
		  double *X0, double *C, double *CJAC, long *NSTATE)
{
  int i, j, k, coni;
  Points *tmp;

#if defined __linux__ || defined linux
  double RX[nxsip], RT[ntsip];
#else
  double *RX, *RT;
  RX=mymalloc(nxsip*sizeof(double));
  RT=mymalloc(ntsip*sizeof(double));
#endif
  

  /* constraints bounds
     first the infinite for each points and last the finite */
  coni=0;
  for(i=0;i<HASH_SIZE;i++){ /* for each hash pointer */
    for(tmp=T_Set.Mesh[i];tmp;tmp=tmp->next){
      for(j=0;j<ntsipc;j++){ /* for all infinite constraints */
	if(NEEDC[coni]){ /* needs constraint coni */
	  if(*MODE!=1) /* NEEDS constraint value */
	    C[coni]=eval_sip(SIP_CONTVAL, j, X0, tmp->coord, NULL, NULL, NULL, NULL);
	  if(*MODE>0){ /* NEEDS constraint gradiente */
	    eval_sip(SIP_CONTGRD, j, X0, tmp->coord, RX, RT, NULL, NULL);
	    for(k=0;k<nxsip;k++)
	      CJAC[coni+k*(*NROWJ)]=RX[k];
	  }
	}
	coni++;
      }
    }
  }
  /*  printarray(X0,nxsip);
      printmatrix(CJAC, coni-1,*NROWJ);*/
  /* finite constraints */
  for(j=0;j<nxsipc;j++){ /* for all finite constraints */
    if(NEEDC[coni]){ /* needs constraint coni */
      if(*MODE!=1) /* NEEDS constraint value */
	C[coni]=eval_sip(SIP_CONXVAL, j, X0, NULL, NULL, NULL, NULL, NULL);
      if(*MODE>0){ /* NEEDS constraint gradiente */
	eval_sip(SIP_CONXGRD, j, X0, NULL, RX, RT, NULL, NULL);
	for(k=0;k<nxsip;k++)
	  CJAC[coni+k*(*NROWJ)]=RX[k];
      }
    }
    coni++;
  }
  /*  printf("confun coni=%d\n",coni);*/

#if !defined __linux__ && !defined linux
  free(RX);
  free(RT);
#endif

  return;
}



double solve_nlp()
{
  double *npsol_bl, *npsol_bu, *npsol_a, *npsol_c, *npsol_cjac, *npsol_clambda;
  double *npsol_objgrd, *npsol_r, *npsol_w;
  double npsol_objf;
  long npsol_n, npsol_nclin, npsol_ncnln, npsol_nrowa, npsol_nrowj;
  long npsol_nrowr;
  long npsol_inform, npsol_iter;
  long *npsol_iw, *npsol_istate, npsol_leniw, npsol_lenw;
  int i, j, coni;

  
  npsol_n=nxsip;
  npsol_nclin=0; /* no linear constraints */
  npsol_nrowa=0;
  npsol_a=NULL;

 /* ncnln = the number of points in the T set times the number of
    infinite constraints plus the finite ones */
  npsol_ncnln=(T_Set.nPoints*ntsipc)+nxsipc;

  npsol_bl=mymalloc((nxsip+npsol_ncnln)*sizeof(double));
  npsol_bu=mymalloc((nxsip+npsol_ncnln)*sizeof(double));

  coni=0; /* coni will count for constraints and variables */

  /* variables bounds */
  for(i=0;i<nxsip;i++){
    npsol_bl[coni]=XBL[i];
    npsol_bu[coni++]=XBU[i];
  }

  /* infinite constraints bounds */
  for(i=0;i<T_Set.nPoints;i++){ /* for all points */
    for(j=0;j<ntsipc;j++){ /* for all infinite constraints */
      npsol_bl[coni]=TCBL[j];
      npsol_bu[coni++]=TCBU[j];
    }
  }

  /* finite constraints bounds */
  for(i=0;i<nxsipc;i++){
    npsol_bl[coni]=XCBL[i];
    npsol_bu[coni++]=XCBU[i];
  }

  /*  printf("npsol coni=%d\n",coni);*/
  /*  printf("%d restricoes para o NPSOL\n", coni);*/

  /* allocate space for jacobian */
  npsol_nrowj=npsol_ncnln;
  npsol_cjac=mymalloc(npsol_ncnln*nxsip*sizeof(double));

  /* allocate istate array */
  npsol_istate=mymalloc((nxsip+npsol_ncnln)*sizeof(long));

  /* allocate space for c array */
  npsol_c=mymalloc(npsol_ncnln*sizeof(double));

  /* allocate clambda array */
  npsol_clambda=mymalloc((nxsip+npsol_ncnln)*sizeof(double));

  /* allocate objgrd array */
  npsol_objgrd=mymalloc((nxsip)*sizeof(double));

  /* allocate r matrix */
  npsol_nrowr=nxsip;
  npsol_r=mymalloc((nxsip*npsol_nrowr)*sizeof(double));

  /* allocate workspace for npsol */
  npsol_leniw=3*nxsip+2*npsol_ncnln;
  npsol_iw=mymalloc(npsol_leniw*sizeof(long));
  npsol_lenw=2*nxsip*nxsip+nxsip*npsol_ncnln+
    2*nxsip*npsol_ncnln+20*nxsip+11*npsol_ncnln+21*npsol_ncnln;
  npsol_w=mymalloc(npsol_lenw*sizeof(double));

  /*  printf("Starting point for NPSOL\n");*/
  /*  printarray(X, nxsip);*/

  stats.nnlpc+=npsol_ncnln;
  stats.nnlp++;
  do {
  npsol_(&npsol_n, &npsol_nclin, &npsol_ncnln, &npsol_nrowa, &npsol_nrowj,
	 &npsol_nrowr, npsol_a, npsol_bl, npsol_bu, npsol_confun, npsol_objfun,
	 &npsol_inform, &npsol_iter, npsol_istate, npsol_c, npsol_cjac,
	 npsol_clambda, &npsol_objf, npsol_objgrd, npsol_r, X, npsol_iw,
	 &npsol_leniw, npsol_w, &npsol_lenw);
  printf("\tNonlinear used %ld iterations Obj=%f Inform=%ld\n",npsol_iter,npsol_objf, npsol_inform);
  } while(npsol_inform==4);

  /*  printf("NPSOL inform=%ld\n", npsol_inform);
  printf("Exit point for NPSOL\n");
  printarray(X, nxsip);*/

  /* free space */
  free(npsol_bl);
  free(npsol_bu);
  free(npsol_cjac);
  free(npsol_istate);
  free(npsol_c);
  free(npsol_clambda);
  free(npsol_objgrd);
  free(npsol_r);
  free(npsol_iw);
  free(npsol_w);
  return npsol_objf;
}

#else



double solve_nlp()
{

  printf("\n\nNPSOL not available\nPlease read the user manual about it\n");
  printf("Aborting....\n");
  exit(1);

  return 0; /* to stop warning */
}


#endif
