#include "pfs.h"
#include "int.h"



void penaltyfs()
{
  double f0, f1, dummy;
  int ko, ki, tki;
  int i, ieps;
  double miu, pt, norg;  /* penalty parameter */
  FILE *fp; /* for results file */

#if defined __linux__ || defined linux
  double tsample[ntsip];
  double dir[nxsip];
  double h[nxsip*nxsip];
  double x0[nxsip], x1[nxsip], g1[nxsip], g0[nxsip], y[nxsip];
  double yeps[nxsip];
#else
  double *tsample, *dir, *h, *x0, *x1, *g1, *g0, *y, *yeps;
  tsample=mymalloc(ntsip*sizeof(double));
  dir=mymalloc(nxsip*sizeof(double));
  h=mymalloc(nxsip*nxsip*sizeof(double));
  x0=mymalloc(nxsip*sizeof(double));
  x1=mymalloc(nxsip*sizeof(double));
  g1=mymalloc(nxsip*sizeof(double));
  g0=mymalloc(nxsip*sizeof(double));
  y=mymalloc(nxsip*sizeof(double));
  yeps=mymalloc(nxsip*sizeof(double));
#endif



  miu=Sip_Opt.startmiu;

  tki=0;
  ieps=0;
  memcpy(x1, X, nxsip*sizeof(double)); /* x1 with initial guess */
  memcpy(yeps, X, nxsip*sizeof(double));

  for(ko=0;ko<Sip_Opt.maxitero;ko++){


    /* y with initial guess for inner iter */
    memcpy(y, x1, nxsip*sizeof(double));

    printf("STARTING INNER ITERATIONS ********** miu=%f, epsilon=%f\n",
	   miu, Sip_Opt.pf_epsilon);

    /* Get new penalty value, miu or epsilon have changed */
    gpenaltyf(nxsip, x1, ntsip, tsample, miu, g1, &f1, &pt);

    
    for(ki=0;ki<Sip_Opt.maxiteri;ki++){ /* Quasi-Newton inner iteration */

      /* Set Hessian inverse */
      set_h(h, ki, nxsip, x1, x0, g1, g0);

      /*      printf("Hessian inverse\n");
	      printmatrix(h, nxsip, nxsip);*/
    
    
      memcpy(x0, x1, nxsip*sizeof(double));
      memcpy(g0, g1, nxsip*sizeof(double));
      f0=f1;

      get_dir(nxsip, x0, h, g0, dir);

      printf("X: ");
      printarray(x0, nxsip);
      printf("Penalty %f\n", f0);
      printf("Gradient: ");
      printarray(g0, nxsip);
      printf("Direction: ");
      printarray(dir, nxsip);

      if(Sip_Opt.ScaleDir)
	scale_dir(nxsip, dir);

      switch (Sip_Opt.linesearch){
      default:
	printf("Unknown LineSearch\nUsing Armijo\n");
      case ARMIJO:
	armijo(nxsip, x0, dir, x1, g0, miu);
	break;
      case ARGO:
	argo(nxsip, x0, dir, x1, g0, miu);
	break;
      case WOLFE:
	wolfe(nxsip, x0, dir, x1, g0, miu);
	break;
      };


      printf("X: ");
      printarray(x1, nxsip);

      gpenaltyf(nxsip, x1, ntsip, tsample, miu, g1, &f1, &pt);

      printf("Penalty terms %f %f constraints %f\n", f0, f1, pt);


      /* inner stopping criteria */
      if(pf_i_stop(nxsip, x0, x1, g1, pt)){
	printf("INNER STOPPING IN ITERATION ********* %d\n", ki+1);
	break;
      }
    }

    tki+=ki+1;

    if(pt>Sip_Opt.zero){
      printf("Increasing penalty\n");
      miu*=Sip_Opt.miufactor;
/*        if(Sip_Opt.pf_type==PF_GXTTSENGTEO) */
/*  	Sip_Opt.pf_epsilon/=Sip_Opt.miufactor; */
      if(miu>1e+8)
	break;
    } else {
      if(Sip_Opt.pf_type==PF_GXTPLUS2)
	break;
      if(ieps>0){
	if(pf_eps_stop(nxsip, yeps, x1)){
	  break;
	}
      }
      ieps++;
      memcpy(yeps, x1,    nxsip*sizeof(double));
      printf("Decreasing epsilon\n");
      Sip_Opt.pf_epsilon/=10;
      if(Sip_Opt.pf_epsilon<1e-8)
	break;
    }

    /* outer stopping criteria */
    /*      if(pf_o_stop(nxsip, y, x1, g1, pt)){ */
    /*      printf("OUTER STOPPING IN ITERATION ********* %d\n", ko+1); */
    /*        break; */
    /*      } */
  }

  printf("STOPPING IN ITERATION ********* %d\n", ko+1);  
  memcpy(X, x1, nxsip*sizeof(double));
  printf("Solution found\n");
  printarray(X, nxsip);
  printf("Penalty function value\n");
  printf("%f\n",f1);

/*  penaltyf(nxsip, x1, ntsip, tsample, miu, &pt, NO_DERIV, NULL)); */


  fp=fopen("./results", "a+");
  f0=eval_sip(SIP_OBJVAL, 0, X, NULL, NULL, NULL, NULL, NULL);

  printf("Gradient of penalty:\n");
  printarray(g1, nxsip);

  norg=0.0;
  for(i=0;i<nxsip;i++)
    norg+=g1[i]*g1[i];
  norg=sqrt(norg);

  dummy=eval_sip(SIP_OBJGRD, 0, x1, NULL, g1, NULL, NULL, NULL);

  printf("Gradient of objective:\n");
  printarray(g1, nxsip);
  

  /*  fprintf(fp, " & %d & %d & %d & %d & %d & %d & %.4e & %.4e & %.2e\\\\\n",
	  tki, ko+1, stats.nsip_objval, stats.nsip_objgrd, stats.nsip_contval,
	  stats.nsip_contgrd, f0, f1, norg);
  */
  fprintf(fp, " & %d & %d & %d & %d & %.6e & %.1e & %.1e \\\\\n",
	  tki, ko+1, stats.nsip_pfval, stats.nsip_gpfval, f0, miu, Sip_Opt.pf_epsilon);
  fclose(fp);
#if !defined __linux__ && !defined linux
  free(tsample);
  free(dir);
  free(h);
  free(x0);
  free(x1);
  free(g1);
  free(g0);
  free(y);
  free(yeps);
#endif
}


/******************************************************
Functions for integral in the penalty function
******************************************************/
double gxt(double *x, double *t, int j, int dlevel, double *grad, double miu)
{
  double dummy;
#if defined __linux || defined linux
  double gt[ntsip];
#else
  double *gt;
  gt=mymalloc(ntsip*sizeof(double));
#endif

  if(dlevel==FIRST_DERIV)
    dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);

#if !defined __linux__ && !defined linux
  free(gt);
#endif

  return (eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j]);
}

double gxtplus(double *x, double *t, int j, int dlevel, double *grad,
	       double miu)
{
  double res, dummy;
#if defined __linux || defined linux
  double gt[ntsip];
#else
  double *gt;
  gt=mymalloc(ntsip*sizeof(double));
#endif
  
  res=eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j];

  if(dlevel==FIRST_DERIV){
    if(res>0.0){
      dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
    } else {
      memset(grad, 0, sizeof(double)*nxsip);
    }
  }

#if !defined __linux__ && !defined linux
  free(gt);
#endif
  
  return((res>0? res:0.0));
}

double gxtplus2(double *x, double *t, int j, int dlevel, double *grad,
		double miu)
{
  double res, dummy;
  int k;

#if defined __linux || defined linux
  double gt[ntsip];
#else
  double *gt;
  gt=mymalloc(ntsip*sizeof(double));
#endif

  
  res=eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j];

  if(dlevel==FIRST_DERIV){
    if(res>0.0){
      dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
      for(k=0;k<nxsip;k++)
	grad[k]*=2.0*res;
    } else {
      memset(grad, 0, sizeof(double)*nxsip);
    }
  }

#if !defined __linux__ && !defined linux
  free(gt);
#endif

  return((res>0? pow(res,2.0):0.0));
}

double gxtepsilon(double *x, double *t, int j, int dlevel, double *grad,
		  double miu)
{
  double res, term;
  double dummy;
  int k;
#if defined __linux || defined linux
  double gt[ntsip];
#else
  double *gt;
  gt=mymalloc(ntsip*sizeof(double));
#endif

  
  res=eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j];

  term=sqrt(pow(res, 2.0)+pow(Sip_Opt.pf_epsilon, 2.0));


  if(dlevel==FIRST_DERIV){
    dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
    for(k=0;k<nxsip;k++)
      grad[k]*=(term+res)/(2*term);
  }

#if !defined __linux__ && !defined linux
  free(gt);
#endif
  
  return((term+res)/2);
}

double gxtteo(double *x, double *t, int j, int dlevel, double *grad,
	      double miu)
{
  double res;
  double dummy;
  int k;

#if defined __linux || defined linux
  double gt[ntsip];
#else
  double *gt;
  gt=mymalloc(ntsip*sizeof(double));
#endif

  
  res=eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j];

  if(res<=-Sip_Opt.pf_epsilon){
    if(dlevel==FIRST_DERIV)
      memset(grad, 0, sizeof(double)*nxsip);
#if !defined __linux__ && !defined linux
    free(gt);
#endif
    return 0;
  }

  if(res>=Sip_Opt.pf_epsilon){
    if(dlevel==FIRST_DERIV)
      dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
#if !defined __linux__ && !defined linux
    free(gt);
#endif
    return res;
  }

  if(dlevel==FIRST_DERIV){
    dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
    for(k=0;k<nxsip;k++)
      grad[k]*=((res+Sip_Opt.pf_epsilon)/(2*Sip_Opt.pf_epsilon));
  }
#if !defined __linux__ && !defined linux
  free(gt);
#endif
  return(pow((res+Sip_Opt.pf_epsilon),2.0)/(4*Sip_Opt.pf_epsilon));
}

double gxtexpplus(double *x, double *t, int j, int dlevel, double *grad,
		  double miu)
{
  double res, dummy;
  int k;

#if defined __linux || defined linux
  double gt[ntsip];
#else
  double *gt;
  gt=mymalloc(ntsip*sizeof(double));
#endif

  
  res=eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j];
  
  if(dlevel==FIRST_DERIV){
    if(res>0.0){
      dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
      for(k=0;k<nxsip;k++)
	grad[k]*=exp(res);
    } else {
      memset(grad, 0, sizeof(double)*nxsip);
    }
  }

#if !defined __linux__ && !defined linux
  free(gt);
#endif
  
  return((res>0.0 ? (exp(res)-1):0.0));
}

double gxtexpepsilon(double *x, double *t, int j, int dlevel, double *grad,
		     double miu)
{
  double res;
  
  res=eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j];
  
  return(exp((res+sqrt(pow(res,2.0)+Sip_Opt.pf_epsilon))/2)-1);
}

double gxtexpteo(double *x, double *t, int j, int dlevel, double *grad,
		 double miu)
{
  double res, dummy;
  int k;

#if defined __linux || defined linux
  double gt[ntsip];
#else
  double *gt;
  gt=mymalloc(ntsip*sizeof(double));
#endif

  
  res=eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j];

  if(res<=-Sip_Opt.pf_epsilon){
    if(dlevel==FIRST_DERIV)
      memset(grad, 0, sizeof(double)*nxsip);
#if !defined __linux__ && !defined linux
    free(gt);
#endif
    return 0.0;
  }
  if(res>=Sip_Opt.pf_epsilon){
    if(dlevel==FIRST_DERIV){
      dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
      for(k=0;k<nxsip;k++)
	grad[k]*=exp(res);
    }
#if !defined __linux__ && !defined linux
    free(gt);
#endif
    return (exp(res)-1);
  }

  if(dlevel==FIRST_DERIV){
    dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
    for(k=0;k<nxsip;k++)
      grad[k]*=(res+Sip_Opt.pf_epsilon)*exp(pow((res+Sip_Opt.pf_epsilon),2.0)/(4*Sip_Opt.pf_epsilon))/2*Sip_Opt.pf_epsilon;
  }
  
#if !defined __linux__ && !defined linux
  free(gt);
#endif

  return(exp(pow((res+Sip_Opt.pf_epsilon),2.0)/(4*Sip_Opt.pf_epsilon))-1);
}

double gxtexp1teo(double *x, double *t, int j, int dlevel, double *grad,
		      double miu)
{
  double res;

  if(dlevel==FIRST_DERIV){
    printf("FIRST_DERIV should not be called: gxtexp1teo\n");
    exit(1);
  }

  res=eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j];

  if(res<=-Sip_Opt.pf_epsilon)
    return 1.0;
  
  if(res>=Sip_Opt.pf_epsilon)
    return (exp(res));
  

  return(exp(pow((res+Sip_Opt.pf_epsilon),2.0)/(4*Sip_Opt.pf_epsilon)));
}


double gxttsengteo(double *x, double *t, int j, int dlevel, double *grad,
		 double miu)
{
  double res, dummy;
  int k;

#if defined __linux || defined linux
  double gt[ntsip];
#else
  double *gt;
  gt=mymalloc(ntsip*sizeof(double));
#endif


  res=eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j];

  if(res<=-Sip_Opt.pf_epsilon){
    if(dlevel==FIRST_DERIV)
      memset(grad, 0, sizeof(double)*nxsip);
#if !defined __linux__ && !defined linux
  free(gt);
#endif
    return 0.0;
  }
  if(res>=Sip_Opt.pf_epsilon){
    if(dlevel==FIRST_DERIV){
      dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
      for(k=0;k<nxsip;k++)
	grad[k]*=exp(miu*res);
    }
#if !defined __linux__ && !defined linux
  free(gt);
#endif
    return (exp(miu*res)-1);
  }

  if(dlevel==FIRST_DERIV){
    dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
    for(k=0;k<nxsip;k++)
      grad[k]*=(res+Sip_Opt.pf_epsilon)*exp(miu*pow((res+Sip_Opt.pf_epsilon),2.0)/(4*Sip_Opt.pf_epsilon))/2*Sip_Opt.pf_epsilon;
  }
  
#if !defined __linux__ && !defined linux
  free(gt);
#endif

  return(exp(miu*pow((res+Sip_Opt.pf_epsilon),2.0)/(4*Sip_Opt.pf_epsilon))-1);
}


double gxthart(double *x, double *t, int j, int dlevel, double *grad,
	       double miu)
{
  double res, dummy;
  int k;

#if defined __linux || defined linux
  double gt[ntsip];
#else
  double *gt;
  gt=mymalloc(ntsip*sizeof(double));
#endif

  
  res=eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j];

  res*=miu;

  if(dlevel==FIRST_DERIV){
    if(res>0.0){
      dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
      for(k=0;k<nxsip;k++)
	grad[k]*=miu*exp(res);
    } else {
      memset(grad, 0, sizeof(double)*nxsip);
    }
  }

#if !defined __linux__ && !defined linux
  free(gt);
#endif
  
  return((res>0.0 ? (exp(res)-1):0.0));
}

double gxtgross(double *x, double *t, int j, int dlevel, double *grad,
		  double miu)
{
  double res, dummy;
  int k;
#if defined __linux || defined linux
  double gt[ntsip];
#else
  double *gt;
  gt=mymalloc(ntsip*sizeof(double));
#endif

  
  res=eval_sip(SIP_CONTVAL, j, x, t, NULL, NULL, NULL, NULL)-TCBU[j];

  res*=miu;
  
  if(dlevel==FIRST_DERIV){
    if(res>0.0){
      dummy=eval_sip(SIP_CONTGRD, j, x, t, grad, gt, NULL, NULL);
      for(k=0;k<nxsip;k++)
	grad[k]*=miu*exp(res);
    } else {
      memset(grad, 0, sizeof(double)*nxsip);
    }
  }

#if !defined __linux__ && !defined linux
  free(gt);
#endif
  
  return((res>0.0 ? (exp(res)):0.0));
}



/*********************************************************
Evaluate the penalty function
*********************************************************/
double penaltyf(int nx, double *x, int nt, double *t, double miu, double *pt,
		int dlevel, double *grad)
{
  double pfval, dummy;
  double num, den;
  int i, k;

#if defined __linux || defined linux
  double gd[nx];
#else
  double *gd;
  gd=mymalloc(nx*sizeof(double));
#endif


  stats.nsip_pfval++;


  pfval=0.0;
  *pt=0.0;
  if(dlevel==FIRST_DERIV)
    memset(grad, 0, sizeof(double)*nx);

  for(i=0; i<ntsipc; i++){
    switch (Sip_Opt.pf_type){
    case PF_GXTPLUS:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtplus, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtplus, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTPLUS2:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtplus2, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEPSILON:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtepsilon, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEXPPLUS:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtexpplus, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtexpplus, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtexpplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtexpplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEXPEPSILON:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtexpepsilon, TBL, TBU, nx, x, nt, t, i,
		 Sip_Opt.adapt_int_n, &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtexpepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtexpepsilon, TBL, TBU, nx, x, nt, t,
		   Sip_Opt.adapt_int_n, i, &num, &den,
		   Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		   dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtexpepsilon, TBL, TBU, nx, x, nt, t,
		    Sip_Opt.adapt_int_n, i, &num, &den,
		    Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		    dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_EXPGXTTEO:
    case PF_TSENGGXTTEO:
    case PF_GXTTEO:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtteo, TBL, TBU, nx, x, nt, t, i,
		 Sip_Opt.adapt_int_n, &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtteo, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtteo, TBL, TBU, nx, x, nt, t,
		   Sip_Opt.adapt_int_n, i, &num, &den,
		   Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		   dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtteo, TBL, TBU, nx, x, nt, t,
		    Sip_Opt.adapt_int_n, i, &num, &den,
		    Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		    dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEXPTEO:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtexpteo, TBL, TBU, nx, x, nt, t, i,
		 Sip_Opt.adapt_int_n, &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtexpteo, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtexpteo, TBL, TBU, nx, x, nt, t,
		   Sip_Opt.adapt_int_n, i, &num, &den,
		   Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		   dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtexpteo, TBL, TBU, nx, x, nt, t,
		    Sip_Opt.adapt_int_n, i, &num, &den,
		    Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		    dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTTSENGTEO:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxttsengteo, TBL, TBU, nx, x, nt, t, i,
		 Sip_Opt.adapt_int_n, &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxttsengteo, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxttsengteo, TBL, TBU, nx, x, nt, t,
		   Sip_Opt.adapt_int_n, i, &num, &den,
		   Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		   dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxttsengteo, TBL, TBU, nx, x, nt, t,
		    Sip_Opt.adapt_int_n, i, &num, &den,
		    Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		    dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTHART:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxthart, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxthart, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxthart, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxthart, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTGROSS:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtgross, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtgross, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtgross, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtgross, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    default:
      printf("Unknown Penalty integral function\n");
    }

    switch (Sip_Opt.pf_type){
    case PF_GXTTEO:
      pfval+=miu*num;
      *pt+=num;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=miu*gd[k];
      }
      break;
    case PF_GXTPLUS2:
      pfval+=0.5*miu*num;
      *pt+=num;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=0.5*miu*gd[k];
      }
      break;
    case PF_EXPGXTTEO:
      pfval+=miu*(exp(num)-1);
      *pt+=num;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=miu*gd[k]*exp(num);
      }
      break;
    case PF_TSENGGXTTEO:
      pfval+=(1/miu)*(exp(miu*(num-Sip_Opt.pf_tau))-1);
      *pt+=num;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=gd[k]*exp(miu*(num-Sip_Opt.pf_tau));
      }
      break;
    case PF_GXTHART:
    case PF_GXTGROSS:
      pfval+=num/miu;
      *pt+=num;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=gd[k]/miu;
      }
      break;
    case PF_GXTTSENGTEO:
      pfval+=num/miu;
      *pt+=num;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=gd[k];
      }
      break;
    default:
      pfval+=miu*num;
      *pt+=num;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=miu*gd[k];
      }
      break;
    }
  }

  pfval+=eval_sip(SIP_OBJVAL, 0, x, NULL, NULL, NULL, NULL, NULL);

  if(dlevel==FIRST_DERIV){
    dummy=eval_sip(SIP_OBJGRD, 0, x, NULL, gd, NULL, NULL, NULL);
    for(k=0;k<nx;k++)
      grad[k]+=gd[k];
  }

#if !defined __linux__ && !defined linux
  free(gd);
#endif
  
  return pfval;
}


/****************************************************
gradient of penalty with respect to x
Arguments are similar of the penalty function
dx   - gradient
****************************************************/
void gpenaltyf(int nx, double *x, int nt, double *t, double miu,
	       double *dx, double *fx, double *pt)
{
  double fx1; /*, fx2;*/
  int i;

#if defined __linux || defined linux
  double x1[nx];
#else
  double *x1;
  x1=mymalloc(nx*sizeof(double));
#endif


  stats.nsip_gpfval++;
    

  *fx=penaltyf(nx, x, nt, t, miu, pt, Sip_Opt.derivatives, dx);
  if(Sip_Opt.derivatives==FIRST_DERIV){
    printf("Gradient:\n");
    printarray(dx, nx);
  }
  if(Sip_Opt.derivatives==NO_DERIV){
    /* Central differences 
       for(i=0; i<nx;i++){
       memcpy(x1, x, nx*sizeof(double));
       memcpy(x2, x, nx*sizeof(double));
       x1[i]+=Sip_Opt.fdiff;
       x2[i]-=Sip_Opt.fdiff;
       fx1=penaltyf(nx, x1, nt, t, miu, pt);
       fx2=penaltyf(nx, x2, nt, t, miu, pt);
       dx[i]=(fx1-fx2)/(2*Sip_Opt.fdiff);
    } */
    
    /* forward differences */
    for(i=0; i<nx;i++){
      memcpy(x1, x, nx*sizeof(double));
      x1[i]+=Sip_Opt.fdiff;
      fx1=penaltyf(nx, x1, nt, t, miu, pt, NO_DERIV, NULL);
      dx[i]=(fx1- *fx)/(Sip_Opt.fdiff);
    }
    printf("Gradient with forward differences:\n");
    printarray(dx,nx);
  }
#if !defined __linux__ && !defined linux
  free(x1);
#endif

}


/******************************************************
Set the inverse of the Hessian
h      - Hessian inverse
k      - Iteration number (for reset)
nx     - dimension
x1     - x in iteration
x0     - x in previous iteration
g1     - gradient in iteration
g0     - gradient in previous iteration
******************************************************/
void set_h(double *h, int k, int nx, double *x1, double *x0,
	   double *g1, double *g0)
{
  int i, j, l;
  double den, teta, aux1, nors, nory;

#if defined __linux || defined linux
  double y[nx], s[nx], mat[nx*nx], aux[nx*nx], auxv[nx];
#else
  double *y, *s, *mat, *aux, *auxv;
  y=mymalloc(nx*sizeof(double));
  s=mymalloc(nx*sizeof(double));
  auxv=mymalloc(nx*sizeof(double));
  aux=mymalloc(nx*nx*sizeof(double));
  mat=mymalloc(nx*nx*sizeof(double));
#endif
  

  if((!(k%nxsip) && Sip_Opt.reset_h) || !k){
    /*  if(1){*/

    identity(h, nx);
/*      printf("Identity Hessian Inverse\n"); */
    
  } else {

      for(i=0;i<nx;i++){
	y[i]=g1[i]-g0[i];
	s[i]=x1[i]-x0[i];
      }

    if(Sip_Opt.dampedBFGS){
      /* Damped BFGS formula */
      printf("Damped BFGS formula\n");
      den=0.0;
      for(l=0;l<nx;l++)
	den+=s[l]*y[l];

      for(i=0;i<nx;i++){
	auxv[i]=0.0;
	for(j=0;j<nx;j++)
	  auxv[i]+=s[j]*h[j*nx+i];
      }

      aux1=0.0;
      for(i=0;i<nx;i++)
	aux1+=auxv[i]*s[i];

      if(den>=0.2*aux1){
	teta=1; /* r=s */
      } else {
	teta=(0.8*aux1)/(aux1-den);
      }

      printf("Teta=%f\n", teta);
      for(i=0;i<nx;i++)
	y[i]=teta*y[i]+(1-teta)*auxv[i];

    }

    den=0.0;
    nors=0.0;
    nory=0.0;
    for(l=0;l<nx;l++){
      den+=s[l]*y[l];
      nors+=s[l]*s[l];
      nory+=y[l]*y[l];
    }

    if(fabs(den)<=Sip_Opt.reseth){
      if(Sip_Opt.scaleBFGS){
	teta=(sqrt(Sip_Opt.reseth))/(sqrt(sqrt(nors)*sqrt(nory)));
	for(l=0;l<nx;l++){
	  s[l]*=teta;
	  y[l]*=teta;
	}
	den=0.0;
	for(l=0;l<nx;l++)
	  den+=s[l]*y[l];

	printf("Hessian scaled by %f\n", teta);
      } else {
	printf("Not changing Hessian Inverse den=%.20f\n", den);
#if !defined __linux__ && !defined linux
  free(y);
  free(s);
  free(mat);
  free(aux);
  free(auxv);
#endif
	return;
      }
    }

    memcpy(aux, h, nx*nx*sizeof(double));

    for(i=0;i<nx;i++){
      for(j=0;j<nx;j++){
	mat[i*nx+j]=-(s[i]*y[j])/den;
	if(i==j)
	  mat[i*nx+j]+=1.0;
      }
    }

    memset(h, 0, nx*nx*sizeof(double));
    for(i=0;i<nx;i++)
      for(j=0;j<nx;j++)
	for(l=0;l<nx;l++)
	  h[i*nx+j]+=mat[i*nx+l]*aux[l*nx+j];
    
    memset(aux, 0, nx*nx*sizeof(double));
    for(i=0;i<nx;i++)
      for(j=0;j<nx;j++)
	for(l=0;l<nx;l++)
	  aux[i*nx+j]+=h[i*nx+l]*mat[j*nx+l];
    
    
    for(i=0;i<nx;i++){
      mat[i*nx+i]=(s[i]*s[i])/den;
      for(j=i+1;j<nx;j++){
	mat[i*nx+j]=(s[i]*s[j])/den;
	mat[j*nx+i]=mat[i*nx+j];
      }
    }

    for(i=0;i<nx;i++){
      for(j=0;j<nx;j++){
	h[i*nx+j]=aux[i*nx+j]+mat[i*nx+j];
      }
    }
  }
#if !defined __linux__ && !defined linux
  free(y);
  free(s);
  free(mat);
  free(aux);
  free(auxv);
#endif

}


void identity(double *h, int nx)
{
  int i;

  memset(h, 0, nx*nx*sizeof(double));
  
  for(i=0;i<nx;i++)
    h[i*nx+i]=1.0;

}



void get_dir(int nx, double *x, double *h, double *grad, double *dir)
{
  int i, j;

  memset(dir, 0, nx*sizeof(double));
  for(i=0;i<nx;i++){
    for(j=0;j<nx;j++)
      dir[i]-=h[i*nx+j]*grad[j];
  }
}



void scale_dir(int nx, double *dir)
{
  double factor, nor;
  int l;

  nor=0.0;
  for(l=0;l<nx;l++)
    nor+=dir[l]*dir[l];

  nor=sqrt(nor);

  if(nor>Sip_Opt.ScaleDir2){
    factor=Sip_Opt.ScaleDir2/nor;
    for(l=0;l<nx;l++)
      dir[l]*=factor;
    printf("Direction Scaled by %f\n", factor);
    return;
  }

  if(nor<Sip_Opt.ScaleDir1){
    factor=Sip_Opt.ScaleDir1/nor;
    for(l=0;l<nx;l++)
      dir[l]*=factor;
    printf("Direction Scaled by %f\n", factor);
  }

  return;
}


void armijo(int nx, double *x0, double *dir, double *x1,
	    double *grad, double miu)
{
  int l;
  double alpha, dif, phi0, phiomega, nor, gd, pt;

#if defined __linux__ || defined linux
  double tsample[ntsip];
#else
  double *tsample;
  tsample=mymalloc(ntsip*sizeof(double));
#endif



  alpha=1.0;
  phi0=penaltyf(nx, x0, ntsip, tsample, miu, &pt, NO_DERIV, NULL);
  printf("phi0=%f pt=%f\n", phi0, pt);

  gd=0.0;
  nor=0.0;
  for(l=0;l<nx;l++){
    gd+=grad[l]*dir[l];
    nor+=dir[l]*dir[l];
  }

  /*    printf("gd=%f\n", gd); */

  nor=sqrt(nor);

  /*    if(nor<1e-4){ */
  /*      printf("Step=1, Maratos efect\n"); */
  /*      for(l=0;l<nx;l++) */
  /*        x1[l]=x0[l]+dir[l]; */
  /*      return; */
  /*    } */
  

  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    phiomega=penaltyf(nx, x1, ntsip, tsample, miu, &pt, NO_DERIV, NULL);
    printf("alpha=%f phiomega=%f pt=%f\n",alpha, phiomega, pt);

    dif=phi0-phiomega;
    if(dif<-(Sip_Opt.armijo)*alpha*gd){
      /*        printf("dif=%f < %f\n", dif, -0.4*alpha*gd); */
      alpha=alpha/2;
      if(alpha*nor<=1e-8){
	alpha=0;
	memcpy(x1,x0,nx*sizeof(double));
/*    	for(l=0;l<nx;l++) */
/*  	  x1[l]=x0[l]+alpha*dir[l]; */
	printf("Step zero %.20f\n", alpha);
#if !defined __linux__ && !defined linux
	free(tsample);
#endif
	return;
      }
    } else {
      printf("Step=%f\n",alpha);
#if !defined __linux__ && !defined linux
      free(tsample);
#endif
      return;
    }
  }
#if !defined __linux__ && !defined linux
  free(tsample);
#endif

}

void argo(int nx, double *x0, double *dir, double *x1,
	    double *grad, double miu)
{
  int l;
  double alpha, dif, phi0, phiomega, nor, gd, pt;

#if defined __linux__ || defined linux
  double tsample[ntsip];
#else
  double *tsample;
  tsample=mymalloc(ntsip*sizeof(double));
#endif


  alpha=1;
  phi0=penaltyf(nx, x0, ntsip, tsample, miu, &pt, NO_DERIV, NULL);
  /*  printf("phi0=%f\n", phi0);*/

  gd=0;
  nor=0;
  for(l=0;l<nx;l++){
    gd+=grad[l]*dir[l];
    nor+=dir[l]*dir[l];
  }

  nor=sqrt(nor);

  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    phiomega=penaltyf(nx, x1, ntsip, tsample, miu, &pt, NO_DERIV, NULL);
    printf("alpha=%f phiomega=%f\n",alpha, phiomega);

    dif=phi0-phiomega;
    if(dif<-(Sip_Opt.argo1)*alpha*gd || dif>-(Sip_Opt.argo2)*alpha*gd){
      alpha=alpha/2;
      if(alpha*nor<=Sip_Opt.zero){
	/*  	alpha=0; */
	for(l=0;l<nx;l++)
	  x1[l]=x0[l]+alpha*dir[l];
	printf("Step=%f, zero\n", alpha);
#if !defined __linux__ && !defined linux
	free(tsample);
#endif
	return;
      }
    } else {
      printf("Step=%f\n",alpha);
#if !defined __linux__ && !defined linux
	free(tsample);
#endif
      return;
    }
  }
#if !defined __linux__ && !defined linux
  free(tsample);
#endif
}


void wolfe(int nx, double *x0, double *dir, double *x1,
	   double *grad, double miu)
{
  int l;
  double alpha, dif, phi0, phiomega, nor, gd, pt;
  double gfd;

#if defined __linux__ || defined linux
  double tsample[ntsip], g1[nx];
#else
  double *tsample, *g1;
  tsample=mymalloc(ntsip*sizeof(double));
  g1=mymalloc(nx*sizeof(double));
#endif



  alpha=1.0;
  phi0=penaltyf(nx, x0, ntsip, tsample, miu, &pt, NO_DERIV, NULL);
  /*  printf("phi0=%f\n", phi0);*/

  gd=0.0;
  nor=0.0;
  for(l=0;l<nx;l++){
    gd+=grad[l]*dir[l];
    nor+=dir[l]*dir[l];
  }

  nor=sqrt(nor);

  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    gpenaltyf(nx, x1, ntsip, tsample, miu, g1, &phiomega, &pt);
    printf("alpha=%f phiomega=%f\n",alpha, phiomega);

    dif=phi0-phiomega;
    gfd=0.0;
    for(l=0;l<nx;l++)
      gfd+=g1[l]*dir[l];
    if(dif<-(Sip_Opt.wolfe1)*alpha*gd || gfd<(Sip_Opt.wolfe2)*gd){
      alpha=alpha/2;
      if(alpha*nor<=Sip_Opt.zero){
	for(l=0;l<nx;l++)
	  x1[l]=x0[l]+alpha*dir[l];
	printf("Step=%f, zero\n", alpha);
#if !defined __linux__ && !defined linux
	free(tsample);
#endif
	return;
      }
    } else {
      printf("Step=%f\n",alpha);
#if !defined __linux__ && !defined linux
	free(tsample);
#endif
      return;
    }
  }
#if !defined __linux__ && !defined linux
  free(tsample);
#endif
}


/*******************************
check inner stopping criteria
*******************************/
int pf_i_stop(int nx, double *x0, double *x1, double *g1, double pt)
{
  int i;
  double aux1, aux2, aux3;


  if(g1){
    aux1=0;
    for(i=0;i<nx;i++)
      aux1+=pow(g1[i],2.0);
    if(aux1<1e-8)return 1;
  }


  aux1=aux2=0;
  for(i=0;i<nx;i++){
    aux1+=pow((x1[i]-x0[i]),2.0);
    aux2+=pow(x1[i],2.0);
  }

  if(sqrt(aux2)<=Sip_Opt.pf_preci){
    aux3=sqrt(aux1);
  } else {
    aux3=sqrt(aux1)/sqrt(aux2);
  }

  if(aux3>Sip_Opt.pf_preci){
    printf("Not stopping in inner iteration norm x=%f\n", aux3);
    return 0;
  }


  /*    aux1=0; */
  /*    for(i=0;i<nx;i++) */
  /*      aux1+=pow(g1[i],2.0); */
  /*    if(aux1>100*Sip_Opt.pf_preci){ */
  /*      printf("Not stopping in inner iteration norm gd=%f\n", aux1); */
  /*      return 0; */
  /*    } */
  

  /*    if(pt>Sip_Opt.zero){ */
  /*      printf("Not stopping in inner iteration %f\n", aux3); */
  /*      return 0; */
  /*    } */

  return 1;
}


/*******************************
check outer stopping criteria
*******************************/
int pf_o_stop(int nx, double *x0, double *x1, double *g1, double pt)
{
  int i;
  double aux1, aux2, aux3;
  static int zero=0;

  aux1=aux2=0;
  for(i=0;i<nx;i++){
    aux1+=pow((x1[i]-x0[i]),2.0);
    aux2+=pow(x1[i],2.0);
  }

  if(sqrt(aux2)<=Sip_Opt.zero){
    aux3=sqrt(aux1);
  } else {
    aux3=sqrt(aux1)/sqrt(aux2);
  }

  if(aux3>Sip_Opt.pf_preco){
        printf("Not stopping in outer iteration %f\n", aux3);
	return 0;
  }

  if(!memcmp(x0,x1,nx*sizeof(double))){
    if(zero++>4){
      printf("\n\n*******************************************************\n");
      printf("* NO PROGRESS IN SOLUTION. SOLUTION MAY BE UNFEASIBLE *\n");
      printf("*******************************************************\n\n\n");
      return 1;
    }
  }


  /*  aux1=0;
  for(i=0;i<nx;i++)
    aux1+=pow(g1[i],2.0);
    if(aux1>Sip_Opt.pf_preco)return 0;*/


  if(pt>Sip_Opt.pf_preco){
        printf("Not stopping in outer iteration pt=%f\n", pt);
	return 0;
  }


  return 1;
}


int pf_eps_stop(int nx, double *x0, double *x1)
{
  int i;
  double aux1, aux2, aux3;


  aux1=aux2=0;
  for(i=0;i<nx;i++){
    aux1+=pow((x1[i]-x0[i]),2.0);
    aux2+=pow(x1[i],2.0);
  }

  if(sqrt(aux2)<=Sip_Opt.zero){
    aux3=sqrt(aux1);
  } else {
    aux3=sqrt(aux1)/sqrt(aux2);
  }

  if(aux3>Sip_Opt.pf_preco){
        printf("Not stopping in eps iteration %f\n", aux3);
	return 0;
  }

  return 1;
}
