#include "solver.h"


SSO Sip_Opt;
SIP_S sip_s;

/* the options */

char nsips_version[] = "NSIPS v2.0";


/* This struct member names must be in alphabetic order,
   for binary search */

keyword keywds[] = {
  KW("armijo"    , sip_opt_d, (Char *)&Sip_Opt.armijo,
     "Constant in Armijo rule"),

  KW("bfgsskip"  , sip_opt_d, (Char *)&Sip_Opt.reseth,
     "Constant for BFGS curvature condition"),

  KW("damped"    , sip_opt_i, (Char *)&Sip_Opt.dampedBFGS,
     "Damped BFGS updating formula"),

  KW("delta"     , sip_opt_d, (Char *)&Sip_Opt.delta,
     "Barrier parameter delta"),

  KW("disc_dist" , sip_opt_d, (Char *)&Sip_Opt.disc_dist,
     "Distance between neighbours"),

  KW("disc_eps" , sip_opt_d, (Char *)&Sip_Opt.disc_eps,
     "Epsilon starting value in discretization method, Reemtsen version"),

  KW("disc_h"    , sip_opt_d, (Char *)&Sip_Opt.disc_h,
     "Initial discretization grid space"),

  KW("disc_h_mx" , sip_opt_i, (Char *)&Sip_Opt.disc_h_mx,
     "Total number of Halton points"),

  KW("disc_ha_mx", sip_opt_i, (Char *)&Sip_Opt.disc_ha_mx,
     "Number of Halton points added in each level"),

  KW("disc_inner", sip_opt_i, (Char *)&Sip_Opt.disc_inner,
     "Number of inner iterations"),

  KW("disc_k"    , sip_opt_i, (Char *)&Sip_Opt.disc_k,
     "Level of discretization"),

  KW("dual_ini"  , sip_opt_i, (Char *)&Sip_Opt.dual_ini_res,
     "Reset in initial guess"),

  KW("int_amp"   , sip_opt_d, (Char *)&Sip_Opt.adapt_int_amp,
     "Minimum integral amplitude"),

  KW("int_corr"  , sip_opt_i, (Char *)&Sip_Opt.ncorrector,
     "Number of correction in predictor corrector"),

  KW("int_eps"    , sip_opt_d, (Char *)&Sip_Opt.pf_epsilon,
     "Smoothing parameter for differentiability"),

  KW("int_epsfactor", sip_opt_d, (Char *)&Sip_Opt.epsfactor,
     "Epsilon factor"),

  KW("int_epslimit" , sip_opt_d, (Char *)&Sip_Opt.epslimit,
     "Epsilon limit"),

  KW("int_error" , sip_opt_d, (Char *)&Sip_Opt.adapt_int_error,
     "Precision in integral computation"),

  KW("int_maxit" , sip_opt_i, (Char *)&Sip_Opt.maxitero,
     "Maximum number of iterations in interior point"),

  KW("int_merit" , sip_opt_s, (Char *)&Sip_Opt.linesearch,
     "Merit function in interior point method"),

  KW("int_n"     , sip_opt_i, (Char *)&Sip_Opt.adapt_int_n,
     "Number of initial intervals in adaptative integration"),

  KW("int_ord"   , sip_opt_s, (Char *)&Sip_Opt.ordering,
     "Ordering in solving KKT system"),

  KW("int_prec"  , sip_opt_d, (Char *)&Sip_Opt.int_prec,
     "Interior point stopping criteria"),

  KW("int_tau"    , sip_opt_d, (Char *)&Sip_Opt.pf_tau,
     "Relaxation on problem transcription"),

  KW("maxiteri"  , sip_opt_i, (Char *)&Sip_Opt.maxiteri,
     "Maximum of inner iterations"),

  KW("maxitero"  , sip_opt_i, (Char *)&Sip_Opt.maxitero,
     "Maximum of outer iterations"),

  KW("method"    , sip_opt_s, (Char *)&Sip_Opt.method,
     "The solver method"),

  KW("miu0"      , sip_opt_d, (Char *)&Sip_Opt.startmiu,
     "Initial penalty parameter value"),

  KW("miuf"      , sip_opt_d, (Char *)&Sip_Opt.miufactor,
     "Multiplying factor for the penalty parameter"),

  KW("pf_eps"    , sip_opt_d, (Char *)&Sip_Opt.pf_epsilon,
     "Smoothing parameter for differentiability"),

  KW("pf_int"    , sip_opt_s, (char *)&Sip_Opt.pf_int,
     "Adaptative formula to use"),

  KW("pf_preci"  , sip_opt_d, (Char *)&Sip_Opt.pf_preci,
     "Inner iteration stopping criteria"),

  KW("pf_preco"  , sip_opt_d, (Char *)&Sip_Opt.pf_preco,
     "Outer iteration stopping criteria"),

  KW("pf_type"   , sip_opt_s, (Char *)&Sip_Opt.pf_type,
     "The penalty function"),

  KW("reset"     , sip_opt_i, (Char *)&Sip_Opt.reset_h,
     "Reset of inverse Hessian estimation after n updates"),

  KW("scale"     , sip_opt_i, (Char *)&Sip_Opt.ScaleDir,
     "Scaling search direction"),

  KW("theta"     , sip_opt_d, (Char *)&Sip_Opt.theta,
     "Minimum of the initial slack variables"),

  KW("updtb"     , sip_opt_s, (char *)&Sip_Opt.updtb,
     "Update barrier formula to use"),

  KW("watchdog"  , sip_opt_i, (Char *)&Sip_Opt.watchdog,
     "with or without watchdog tecnique"),

  KW("watchmax"  , sip_opt_i, (Char *)&Sip_Opt.watchmax,
     "Maximum number of increase in the merit function (watchdog)")

};


struct Option_Info Oinfo = { "nsips", "NSIPS", "nsips_options",
			     keywds, nkeywds, 1, nsips_version, 0, NULL};




/**********************************************
Set options. String type
**********************************************/
char *sip_opt_s(Option_Info *oi, keyword *kw, char *value)
{
  char *s;


  /* never echo options */
  oi->option_echo &= ~ASL_OI_echo; 


  if(!strncmp("method", kw->name, 6)){
    s=value;
    while(*s!=' ' && *s!=0)
      s++;
    if(s<=value)
      return value;

    if(!strncmp("disc_hett", value, 9)){
      *(int *)kw->info=DISC_METHOD;
      printf("Discretization method selected Hettich version\n");
      return s;
    }
    if(!strncmp("disc_reem", value, 9)){
      *(int *)kw->info=DISC_METHOD_REEM;
      printf("Discretization method selected Reemtsen version\n");
      return s;
    }
    if(!strncmp("disc_halt", value, 9)){
      *(int *)kw->info=DISC_METHOD_HALTON;
      printf("Discretization method selected Hettich version with Halton points\n");
      return s;
    }

    if(!strncmp("penalty_m", value, 9)){
      *(int *)kw->info=LPFS;
      printf("Penalty multiplier method selected\n");
      return s;
    }
    if(!strncmp("penalty", value, 7)){
      *(int *)kw->info=PFS;
      printf("Penalty method selected\n");
      return s;
    }
    if(!strncmp("sqp", value, 3)){
      *(int *)kw->info=SQP_DUAL_LAG;
      printf("SQP method selected\n");
      return s;
    }
    if(!strncmp("intp", value, 4)){
      *(int *)kw->info=INTP;
      /* default value for merit function */
      Sip_Opt.linesearch=L2MERIT;
      printf("Interior point method selected\n");
      return s;
    }

    /* unknown method */
    return value;
  }

  if(!strncmp("pf_type", kw->name, 7)){
    s=value;
    while(*s!=' ' && *s!=0)
      s++;
    if(s<=value)
      return value;

    if(!strncmp("p1", value, 2)){
      *(int *)kw->info=PF_GXTTEO;
      printf("Penalty function 1\n");
      return s;
    }

    if(!strncmp("p2", value, 2)){
      *(int *)kw->info=PF_GXTPLUS2;
      printf("Penalty function 2\n");
      return s;
    }

    if(!strncmp("p3", value, 2)){
      *(int *)kw->info=PF_GXTEXPTEO;
      printf("Penalty function 3\n");
      return s;
    }

    /* unknown penalty */
    return value;
  }

  if(!strncmp("pf_int", kw->name, 7)){
    s=value;
    while(*s!=' ' && *s!=0)
      s++;
    if(s<=value)
      return value;

    if(!strncmp("gaussian", value, 8)){
      *(int *)kw->info=PF_GXTTEO;
      printf("Simple number 1 penalty function\n");
      return s;
    }

    if(!strncmp("trapzoid", value, 8)){
      *(int *)kw->info=PF_GXTPLUS2;
      printf("Simple number 2 penalty function\n");
      return s;
    }

    /* unknown penalty */
    return value;
  }

  if(!strncmp("updtb", kw->name, 5)){
    s=value;
    while(*s!=' ' && *s!=0)
      s++;
    if(s<=value)
      return value;
    
    if(!strncmp("f1", value, 2)){
      *(int *)kw->info=1;
      printf("Update barrier parameter by formula 1\n");
      return s;
    }

    if(!strncmp("f2", value, 2)){
      *(int *)kw->info=2;
      printf("Update barrier parameter by formula 2\n");
      return s;
    }

    /* unknown formula */
    return value;
  }

  if(!strncmp("int_ord", kw->name, 7)){
    s=value;
    while(*s!=' ' && *s!=0)
      s++;
    if(s<=value)
      return value;

    if(!strncmp("pc", value, 2)){
      *(int *)kw->info=PREDCORR;
      printf("Predictor Corrector with dual ordering\n");
      return s;
    }

    if(!strncmp("p", value, 1)){
      *(int *)kw->info=PRIMAL;
      printf("Primal ordering\n");
      return s;
    }

    if(!strncmp("d", value, 1)){
      *(int *)kw->info=DUAL;
      printf("Dual ordering\n");
      return s;
    }

    /* unknown penalty */
    return value;
  }

  if(!strncmp("int_merit", kw->name, 7)){
    s=value;
    while(*s!=' ' && *s!=0)
      s++;
    if(s<=value)
      return value;

    if(!strncmp("m1", value, 2)){
      *(int *)kw->info=L2MERIT;
      printf("L2 merit function\n");
      return s;
    }

    if(!strncmp("m2", value, 2)){
      *(int *)kw->info=ALMERIT;
      printf("Augmented Lagrangian merit function\n");
      return s;
    }

    /* unknown penalty */
    return value;
  }


  /* not implemented option */
  return value;
}


/**********************************************
Set options. Integer type
**********************************************/
char *sip_opt_i(Option_Info *oi, keyword *kw, char *value)
{
  long optval;
  char *s;

  /* never echo options */
  oi->option_echo &= ~ASL_OI_echo;

  optval=strtol(value, &s, 10);
  if(s > value){
    /* existing integer number */
    *(int *)kw->info=(int)optval;
    printf("%s=%d\n", kw->name, *(int *)kw->info);
    return s;
  }

return value;
}


/**********************************************
Set options. Double type
**********************************************/
char *sip_opt_d(Option_Info *oi, keyword *kw, char *value)
{
  double optval;
  char *s;
  double hvals[256];
  int nh;

  /* never echo options */
  oi->option_echo &= ~ASL_OI_echo;  

  optval=strtod(value, &s);
  if(s > value){
    /* existing integer number */
    if(!strncmp("disc_h", kw->name, 6)){
      /* disc_h is an complex option */
      nh=1;
      hvals[0]=optval;
      value=s;
      while(*value==','){
	value++;
	optval=strtod(value, &s);
	if(s > value){
	  hvals[nh]=optval;
	  nh++;
	  value=s;
	} else {
	  printf("Expecting double value and obtained nothing\n");
	  break;
	}
      }

      Sip_Opt.ntvars=nh;
      Sip_Opt.disc_h=mymalloc(nh*sizeof(double));
      memcpy(Sip_Opt.disc_h, hvals, nh*sizeof(double));
      return s;
    } else {
      /* existing double number */
      *(double *)kw->info=optval;
      return s;
    }
  }

return value;
}



/**********************************************
Set default options.
**********************************************/
void sip_option()
{

  /* set default options */
  Sip_Opt.debug_level=255;
  Sip_Opt.method=PFS;
  Sip_Opt.multipliersUPD=MULTI; /*  GENINV; */
  Sip_Opt.zero=1E-6; /* important value for discretization method 1E-6 */
  Sip_Opt.reset_h=0;
  Sip_Opt.dual_ini_res=1;
  Sip_Opt.pf_preci=1.0E-4;
  Sip_Opt.pf_preco=1.0E-4;
  Sip_Opt.int_prec=1.0E-3;
  Sip_Opt.pf_epsilon=1.0E-4; /* 1.0E-4*/
  Sip_Opt.pf_tau=1.0E-4; /* 1.0E-8 */
  Sip_Opt.pf_type=PF_GXTEXPTEO; /* PLUS2; TEO;   GXTEPSILON; */
  Sip_Opt.derivatives=FIRST_DERIV;
  Sip_Opt.pf_int=PF_ADAPT;
  Sip_Opt.n_int=1000;
  Sip_Opt.adapt_int_n=20;
  Sip_Opt.adapt_int_amp=1e-2; /* fixo */
  Sip_Opt.adapt_int_error=1e-8;
  
  Sip_Opt.disc_k=3;
  Sip_Opt.disc_h=NULL;
  Sip_Opt.disc_h_mx=0;
  Sip_Opt.disc_ha_mx=0;
  Sip_Opt.disc_inner=100;
  Sip_Opt.disc_dist=0.2;
  Sip_Opt.disc_eps=0.01;
  
  Sip_Opt.fdiff=1e-8;
  Sip_Opt.reseth=1e-8; /* fixo */
  Sip_Opt.dampedBFGS=TRUE;
  Sip_Opt.scaleBFGS=FALSE;
  Sip_Opt.ScaleDir=FALSE;
  Sip_Opt.ScaleDir1=1e-2;
  Sip_Opt.ScaleDir2=1e+2;
  
  Sip_Opt.maxitero=400;
  Sip_Opt.maxiteri=400;
  
  Sip_Opt.startmiu=1;
  Sip_Opt.miufactor=10;

  Sip_Opt.epsfactor=0.1;
  Sip_Opt.epslimit=1e-8;
  
  Sip_Opt.linesearch=ARMIJO;
  Sip_Opt.armijo=1e-1;
  Sip_Opt.argo1=1e-4;
  Sip_Opt.argo2=0.9;
  Sip_Opt.wolfe1=1e-4;
  Sip_Opt.wolfe2=0.9;
  
  Sip_Opt.powell_nmax=1000;
  Sip_Opt.powell_epson=1E-4;

  Sip_Opt.theta=1.0;
  Sip_Opt.delta=0.1;
  Sip_Opt.updtb=1;

  Sip_Opt.watchdog=0;
  Sip_Opt.watchmax=3;

  Sip_Opt.choleps=1e-12;

  Sip_Opt.ordering=DUAL;
  Sip_Opt.ncorrector=1;
  
  return ; /* its not ok, but continue */
  
}


  /* split opt in option and value */
/*    for(optlen=0,optval=opt;optlen<strlen(opt) && *(opt+optlen)!='=';optlen++,optval++); */
/*    optval++; */

/*    if(!strncmp("disc_k", opt, optlen)){ */
/*      Sip_Opt->disc_k=atoi(optval); */
/*      return 0; */
/*    } */

/*    if(!strncmp("disc_h", opt, optlen)){ */
/*      for(j=0;*optval && *optval!=',' && j<255;buff[j++]=*optval++); */
/*      buff[j]=0; */
/*      Sip_Opt->ntvars=atoi(buff); */
/*      if(Sip_Opt->ntvars<0){ */
/*        fprintf(stderr, "Invalid number of t vars in option: %s", opt); */
/*        fprintf(stderr, "Ignoring option\n"); */
/*        return 0;  its not ok, but continue  */
/*      } */

/*      Sip_Opt->disc_h=mymalloc(Sip_Opt->ntvars*sizeof(double)); */
/*      for(i=0;i<Sip_Opt->ntvars;i++){ */
/*        optval++; */
/*        if(!(*optval)){ */
/*  	fprintf(stderr, "Missing component in option: %s\n", opt); */
/*  	fprintf(stderr, "Option Ignored\n"); */
/*  	return 0;  its not ok, but continue  */
/*        } */
/*        for(j=0;*optval && *optval!=',' && j<255;buff[j++]=*optval++); */
/*        buff[j]=0; */
/*        Sip_Opt->disc_h[i]=atof(buff); */
/*      } */
/*      return 0; */
/*    } */

/*    if(!strncmp("disc_h_mx", opt, optlen)){ */
/*      Sip_Opt->disc_h_mx=atoi(optval); */
/*      return 0; */
/*    } */

/*    if(!strncmp("disc_ha_mx", opt, optlen)){ */
/*      Sip_Opt->disc_ha_mx=atoi(optval); */
/*      return 0; */
/*    } */

/*    if(!strncmp("disc_inner", opt, optlen)){ */
/*      Sip_Opt->disc_inner=atoi(optval); */
/*      return 0; */
/*    } */

/*    if(!strncmp("zero", opt, optlen)){ */
/*      Sip_Opt->zero=atof(optval); */
/*      return 0; */
/*    } */

/*    if(!strncmp("disc_dist", opt, optlen)){ */
/*      Sip_Opt->disc_dist=atof(optval); */
/*      return 0; */
/*    } */

/*    if(!strncmp("debug", opt, optlen)){ */
/*      Sip_Opt->debug_level=atoi(optval); */
/*      return 0; */
/*    } */

/*    fprintf(stderr,"Invalid Option: %s \n", opt); */
/*    fprintf(stderr,"Option Ignored\n"); */
/*  return 0;  its not ok, but continue */
/*  } */


/************************************************
   Main solver subroutine
   Check for initial guess or find one.
   Call the apropriate method
************************************************/
void sip_solver(int Lnxsip,double *LXBU,double *LXBL,int Lntsip,double *LTBU,double *LTBL,int Lnxsipc,double *LXCBU,double *LXCBL,int Lntsipc,double *LTCBU,double *LTCBL,double *LX,int guess,double (*Lsip_eval) ())
{
  nxsip=Lnxsip;
  ntsip=Lntsip;
  nxsipc=Lnxsipc;
  ntsipc=Lntsipc;
  X=LX;
  XBU=LXBU;
  XBL=LXBL;
  TBU=LTBU;
  TBL=LTBL;
  XCBU=LXCBU;
  XCBL=LXCBL;
  TCBU=LTCBU;
  TCBL=LTCBL;
  eval_sip=Lsip_eval;

  /* No initial guess for IPM */
  if(!guess){ /*   && Sip_Opt.method!=INTP){ */
    printf("Without Initial Guess\n");
    sip_iguess(X);
    printarray(X, nxsip);
    /*    for(i=0;i<nxsip;i++)
	  X[i]=1;*/
  }


  printf("Nx=%d\t Nt=%d\n",nxsip, ntsip);
  switch(Sip_Opt.method){
  case DISC_METHOD:
    check_finite_bound();
    discretization();
    break;
  case DISC_METHOD_HALTON:
    check_finite_bound();
    disc_halton();
    break;
  case DISC_METHOD_REEM:
    check_finite_bound();
    disc_reem();
    break;
  case PFS:
    check_onedim_finite();
    check_no_finite_const();
    check_const_leq();
    penaltyfs();
    break;
  case PFS_DUAL:
    check_onedim_finite();
    check_no_finite_const();
    check_const_leq();
    penaltyfs_dual();
    break;
  case LPFS:
    check_onedim_finite();
    check_no_finite_const();
    check_const_leq();
    penaltylfs();
    break;
  case PFS_TEO:
    check_onedim_finite();
    check_no_finite_const();
    check_const_leq();
    penaltyfs_teo();
    break;
    /*  case SQP:
    QPs();
    break;*/
  case PFPOWELLS:
    check_onedim_finite();
    check_no_finite_const();
    check_const_leq();
    pfpowells();
    break;
  case NPSOL_PF:
    check_onedim_finite();
    check_no_finite_const();
    check_const_leq();
    npsol_penaltyfs();
    break;
  case SQP_DUAL_LAG:
    check_onedim_finite();
    check_no_finite_const();
    check_const_leq();
    dual_lags();
    break;
    /*    case PF_TSENGS: */
    /*      pf_tsengs(); */
    /*      break; */
    /*    case FILTER: */
    /*      filters(); */
    /*      break; */
  case INTP:
    check_onedim_finite();
    check_no_finite_const();
    check_const_leq();
    /*      if(!guess){ */
    /* a ramdom initial guess for IPM */
    /*        srandom(random()); */
    /*        for(i=0;i<nxsip;i++) */
    /*  	X[i]=((double)random())/RAND_MAX; */
    /*      } */
    intpkkts();
    break;
  default:
    printf("Invalid Method\n");
  }

  return;
}


#define INF 0x7ff00000


/* Check if there is only one infinite variable of finite bounds */
void check_onedim_finite(void)
{
  int i;

  if(ntsip!=1){
    printf("Number of infinite variables must be one\n");
    printf("This method should not be used for this problem\n");
  }

  /* Check bounds */

  for(i=0;i<ntsip;i++){
    if(TBL[i]<=-INF)
      printf("Lower bound of infinite variable %d is -infinity\n"
	     "Bounds on infinite variables must be finite\n", i);
    if(TBU[i]>=+INF)
      printf("Upper bound of infinite variable %d is +infinity\n"
	     "Bounds on infinite variables must be finite\n", i);
  }
    
}

/* Check if there is no finite constraints */
void check_no_finite_const(void)
{
  int i;

  if(nxsipc>=1){
    printf("There should be no finite constraints\n");
    printf("This method should not be used for this problem\n");
  }

  /* Check finite bounds */

  for(i=0;i<ntsip;i++){
    if(XBL[i]>-INF)
      printf("Lower bound of finite variable %d is finite\n"
	     "There should be no finite bounds on finite variables\n", i);
    if(XBU[i]<+INF)
      printf("Upper bound of finite variable %d is finite\n"
	     "There should be no finite bounds on finite variables\n", i);
  }


}

/* Check if infinite constraints are of lower or equal type */
void check_const_leq(void)
{
  int i;

  for(i=0;i<ntsipc;i++){
    if(TCBL[i]>-INF)
      printf("Infinite constraint %d has finite lower bound\n"
	     "Infinite constraints must be of lower or equal type\n", i);
    if(TCBU[i]>=+INF)
      printf("Infinite constraint %d has infinite upper bound\n"
	     "Infinite constraints must be of lower or equal type\n", i);
  }

}

/* Check if infinite variables are all of finite bound */
void check_finite_bound(void)
{
  int i;

  /* Check bounds */

  for(i=0;i<ntsip;i++){
    if(TBL[i]<=-INF)
      printf("Lower bound of infinite variable %d is -infinity\n"
	     "Bounds on infinite variables must be finite\n", i);
    if(TBU[i]>=+INF)
      printf("Upper bound of infinite variable %d is +infinity\n"
	     "Bounds on infinite variables must be finite\n", i);
  }

}
