###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#   Described by A.J. Anderson and A.S. Lewis in "An Extension pf the Simplex
#                 Algorithm for Semi-Infinite Linear Programming",
#    Mathematical Programming, vol. 44, 1989, pp.247-269
#
#   Example on page 267
#    Coded by A. Ismael F. Vaz 21/12/2001  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	x[3];

subject to tcons:
	-x[1]*t[1]-x[2]*t[2]-x[3]-(1/6)*((t[1]-1)^2+t[2])*(t[1]+(2-t[2])) <= 0;
subject to bounds {i in 1..nt}:
	0 <= t[i] <= 2;

data;

param nx := 3;
param nt := 2;


param xinit :=
	1 0
	2 0
	3 0;

param tinit :=
	1 0
	2 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf,%lf) f=%lf\n", 0.5, 1/6, -1/3, -1/3;
