###############################################################################
# Objective: Linear
# Constraints: Generic
###############################################################################
#
#   As described by J.W. Blankenship and J.E. Falk in "Infinitely Constrained
#                 Optimization Problems",
#    Journal of Optimization Theory and Applications (JOTA), Vol. 19, No. 2,
#                 June 1976
#
#   Example 3.2
#    Coded by A. Ismael F. Vaz 04/03/2001  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################
#
#  This problem falls out from my initial definition
#  T!=[a,b]
#
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];



minimize fx:
	x[1]+x[2];

subject to tcons1:
	-x[1]*(t[1]-2)^2-x[2]*(t[2]-2)^2+2 <= 0;
subject to tcons2:
	t[1]+t[2]<=2;
subject to tcons3:
	-sqrt(x[1])*(t[3]-2)^2-sqrt(x[2])*(t[4]-2)^2+2 <= 0;
subject to tcons4:
	t[3]^2+t[4]^2<=2;
subject to xbounds {i in 1..nx}:
	0 <= x[i] <=1;
subject to tbounds {i in 1..nt}:	
	0 <= t[i];

data;

param nx := 2;
param nt := 4;

# starting point
param xinit :=
	1 1
	2 1;

param tinit :=
	1 0
	2 0
	3 0
	4 0;



# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf) fx=%lf\n", 1.0, 1.0, 2.0;
