###############################################################################
# Objective: Linear
# Constraints: Generic
###############################################################################
#
#   As described by J.W. Blankenship and J.E. Falk in "Infinitely Constrained
#                 Optimization Problems",
#    Journal of Optimization Theory and Applications (JOTA), Vol. 19, No. 2,
#                 June 1976
#
#   Example 3.3
#    Coded by A. Ismael F. Vaz 04/03/2001  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################
#
#  This problem falls out from my initial definition
#  T!=[a,b]
#
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

var Q = if (x[1] = 0) then 0 else 200;


minimize fx:
	-x[3];

subject to tcons1:
	-3*x[1]*exp(-0.02*t[1])-2*x[2]*exp(-0.5*t[2]/x[2])+x[3] <= 0;

subject to xcons1:
	20*x[1]+10*x[2] <= 360;
subject to xcons2:
	Q+60*x[1]+10*x[2] <= 850;
subject to xcons3:
	Q+60*x[2]+40*x[2] <= 1600; # 60+40=100 ???
subject to xbounds {i in 1..(nx-1)}:
	0 <= x[i];


subject to tcons2:
	35*t[1]+5*t[2] <= 400;
subject to tcons3:
	35*t[1]+5*t[2] <= 800; # !!!
subject to tbounds {i in 1..nt}:	
	0 <= t[i];

data;

param nx := 3;
param nt := 2;

# starting point
param xinit :=
	1 1
	2 1
	3 1;

param tinit :=
	1 0
	2 0;



# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf) fx=%lf\n", 1.0, 1.0, 2.0;
