###############################################################################
# Objective: Linear
# Constraints: Quadratic
###############################################################################
#
#   As described in C.J. Price and I.D. Coope
#    "Numerical Experiments in Semi-infinite Programming",
#    Computational Optimization and Applications 6, 169-189 (1996)
#    Kluwer Academic Publishers.
#
#   Problem N
#    Coded by A. Ismael F. Vaz 11/02/2000  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	x[2];

subject to tcons:
	2*x[1]^2*t[1]^2-t[1]^4+x[1]^2-x[2] <= 0;
subject to bounds:
	-1 <= t[1] <= 1;

data;

param nx := 2;
param nt := 1;

# Price starting point
param xinit :=
	1 0.5
	2 0.5;

param tinit :=
	1 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf) f=%lf\n", 0, 0, 0;


