###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#  Fractional semi-infinite programming problem
#  As described by Martin Gugat, PhD Thesis,
#   submitted to Trier University, "Fractional Semi-Infinite Programming", 1994
#
#   Example 6
#    Coded by A. Ismael F. Vaz 18/03/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of z vars
param nz := 8;

# the gamma parameter
param gamma := 10^4;

# z variables
var z {1..nz};
# infinite variable
var t;
# d distance
var d;


# The functions
var f = t;
var v = z[5]*t^8+z[4]*t^6+z[3]*t^4+z[2]*t^2+z[1];
var w = z[6]*t^4+z[7]*t^2+z[8];



minimize dp:
	d;

subject to tcons1:
	-(f-v/w)-d <= 0;
subject to tcons2:
	(f-v/w)-d <= 0;
subject to tcons3:
	-w <= -1;
subject to tcons4:
	w <= gamma;
subject to bounds:
	0 <= t <= 1;


###################
# End of Problem  #
###################


# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display d;
display z;

################################################
# solution of problem presented by the authors #
################################################

# Gamma=10^4
printf "Original Solution\n";
printf "%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf\n", 0.27567922e-2, 0.49793688e2,
	0.40846220e4, 0.76996733e4, -0.18277917e4, 0.92629502e4, 0.73604985e3,
	0.1e1;
# Gamma=10^5
#printf "Original Solution\n";
#printf "%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf\n", 0.21898865e-1, 0.43372369e3,
#	0.39236288e5, 0.811796e5, -0.21120695e5, 0.93246275e5, 0.67394953e4,
#	0.82602905e1;
