###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#  Fractional semi-infinite programming problem
#  As described by Martin Gugat, PhD Thesis,
#   submitted to Trier University, "Fractional Semi-Infinite Programming", 1994
#
#   Example 9 Function C (Same as gugat1.mod, with different gamma)
#
#    Function C was squared to remove the non-diferentiable square root
#
#    Coded by A. Ismael F. Vaz 19/03/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of z vars
param nz := 8;

# the gamma parameter
param gamma := 10^3;

# z variables
var z {1..nz};
# infinite variable
var t;
# d distance
var d;


# The functions
var f = t;
var v = z[5]*t^8+z[4]*t^6+z[3]*t^4+z[2]*t^2+z[1];
var w = z[6]*t^4+z[7]*t^2+z[8];



minimize dp:
	d;

subject to tcons1:
	-(f-v/w)-d <= 0;
subject to tcons2:
	(f-v/w)-d <= 0;
subject to tcons3:
	-w <= -1;
subject to tcons4:
	w <= gamma;
subject to bounds:
	0 <= t <= 1;


###################
# End of Problem  #
###################


# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display d;
display z;

################################################
# solution of problem presented by the authors #
################################################

# Gamma=10^3
printf "Original Solution\n";
printf "%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf\n", 0.23806669e-3, 0.48377195e2,
	0.60113550e3, 0.40497727e3, -0.54553848e2, 0.75335388e3, 0.24290542e3,
	0.37496998e1;