###############################################################################
# Objective: Linear
# Constraints: Generic
###############################################################################
#  Fractional semi-infinite programming problem
#  As described by Martin Gugat, PhD Thesis,
#   submitted to Trier University, "Fractional Semi-Infinite Programming", 1994
#
#   Example 11
#    Coded by A. Ismael F. Vaz 19/03/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of z vars
param nz := 5;

# Gamma parameter
param gamma := 10^3;


# z variables
var z {1..nz};
# infinite variable
var t;
# d distance
var d;


# The functions
var f = log(1+t);
var v = z[1]*t^3+z[2]*t^2+z[3]*t+z[4];
var w = gamma*exp(-z[5]*t)/100;



minimize dp:
	d;

subject to tcons1:
	-(f-v/w)-d <= 0;
subject to tcons2:
	(f-v/w)-d <= 0;
subject to tcons3:
	-w <= -1;
subject to tcons4:
	w <= gamma;
subject to bounds:
	0 <= t <= 1;


###################
# End of Problem  #
###################


# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display d;
display z;

# Gamma=10^3
printf "Original Solution\n";
printf "%lf,%lf,%lf,%lf,%lf\n", 1.6926948, 1.7899316, 9.9848842, 0.23303442e-3,
	-0.66425892;