###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  As described by T. Leon, S. Sanmatias and E. Vercher,
#      "A multi-local optimization algorithm"
#    Top (1998) Vol. 6, No. 1, pp. 1-18
#
#   Example LV, Appendix
#
#    Coded by A. Ismael F. Vaz 19/06/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of x vars
param nx;
#number of t vars
param nt;
# initial guess
param x0 {1..nx};
param t0 {1..nt};

# x variables
var x {i in 1..nx} := x0[i];
# t variable
var t {i in 1..nt} := t0[i];

minimize obj:
	x[1]+(1/2)*x[2]+(1/3)*x[3]+(1/4)*x[4]+(1/5)*x[5];

subject to tcons1:
	(1/(1+t[1]^2))-x[1]-x[2]*t[1]-x[3]*t[1]^2-x[4]*t[1]^3-x[5]*t[1]^4<=0;
subject to bounds {i in 1..nt}:
	0 <= t[i] <= 1;


###################
# End of Problem  #
###################

data;

param nx := 5;
param nt := 1;

param x0 :=
	1	1
	2	0
	3	0
	4	0
	5	0;

param t0 :=
	1	0;

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display x;
display obj;
