###############################################################################
# Objective: Quadratic
# Constraints: Generic
###############################################################################
#   Math Works Inc. Optimization Toolbox, Version 2.1
#
#   Example 1 - One-dimensional example.
#    Coded by A. Ismael F. Vaz 10/11/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	(x[1]-0.5)^2+(x[2]-0.5)^2+(x[3]-0.5)^2;

subject to tcons1:
	sin(t[1]*x[1])*cos(t[1]*x[2])
	  -(1/1000)*(t[1]-50)^2-sin(t[1]*x[3])-x[3]-1 <= 0;
subject to tcons2:
	sin(t[1]*x[2])*cos(t[1]*x[1])
	  -(1/1000)*(t[1]-50)^2-sin(t[1]*x[3])-x[3]-1 <= 0;
subject to bounds:
	1 <= t[1] <= 100;

data;

param nx := 3;
param nt := 1;

# MATLAB starting guess
param xinit :=
	1 0.5
	2 0.2
	3 0.3;

param tinit :=
	1 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
#printf "Original Solution\n";
#printf "x=(%lf,%lf) f=%lf\n", 0.0, 0.5, -0.25;
#Solution presented by MATLAB

