###############################################################################
# Objective: Quadratic
# Constraints: Generic
###############################################################################
#
#  Price PhD. Thesis
#   As described in C.J. Price 
#    "Nonlinear Semi-infinite Programming",
#    PhD. Thesis submitted to University of Canterbury,
#    New Zealand, August 1992.
#
#   Problem S n=6
#    Coded by A. Ismael F. Vaz 24/12/1999  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	x[1]*x[2]+x[2]*x[3]+x[3]*x[4];

subject to tcons:
	2*(x[1]^2+x[2]^2+x[3]^2+x[4]^2)-6-2*nt
	+sin(t[1]-x[1]-x[4])+sin(t[2]-x[2]-x[3])
	+sin(t[3]-x[1])+sin(2*t[4]-x[2])+sin(t[5]-x[3])+sin(2*t[6]-x[4]) <= 0;
subject to bounds {i in 1..nt}:
	0 <= t[i] <= 2;

data;

param nx := 4;
param nt := 6;

# Price starting point
param xinit :=
	1 1
	2 1
	3 1
	4 1;

param tinit :=
	1 0
	2 0
	3 0
	4 0
	5 0
	6 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf,%lf,%lf) f=%lf\n", 0.960921, -1.456291, 1.581476, -0.905873, -5.135086;
