###############################################################################
# Objective: Quadratic
# Constraints: Generic
###############################################################################
#
#  Price PhD. Thesis
#   As described in C.J. Price 
#    "Nonlinear Semi-infinite Programming",
#    PhD. Thesis submitted to University of Canterbury,
#    New Zealand, August 1992.
#
#   Problem U
#    Coded by A. Ismael F. Vaz 24/12/1999  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	sum {i in 1..4} ((1/10)*x[i]^2-x[i]);

subject to tcons:
	(x[4]/5)*sin(30*t[1]*sin(x[1])+30*t[2]*cos(x[2]))+
	(x[3]/10)*sin((t[1]*t[2])/10)+t[3]*x[1]+t[4]*x[2]+t[5]*x[3]+t[6]*x[4]-
	4 <=0 ;

subject to bounds {i in 1..nt}:
	-1 <= t[i] <= 1;

data;

param nx := 4;
param nt := 6;

# Price starting point
param xinit :=
	1 3
	2 2
	3 1
	4 0;

param tinit :=
	1 0
	2 0
	3 0
	4 0
	5 0
	6 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf,%lf,%lf) f=%lf\n", 1.173288, 1.179673, 1.142275, 0.412150, -3.483097;
