###############################################################################
# Objective: Linear
# Constraints: Quadratic
###############################################################################
#
#   As described by Tanaka, "A trust region method for semi-infinite
#                                programming problems"
#   in International Journal of Systems Science, 1999, vol. 30, n. 2, 199-204
#
#   Example 2
#    Coded by A. Ismael F. Vaz 15/04/2000  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# PI value
param pi := 4*atan(1);

#number of t
param nt := 1;
#number of x
param nx := 2;

# initial guess
param xinit {1..nx};
param tinit {1..nt};

# x variable
var x {i in 1..nx}:= xinit[i];
# t variable
var t {i in 1..nt}:= tinit[i];

minimize fx:
	-x[1]+10*(x[1]^2+x[2]^2-1);

subject to tcons1:
	x[1]^2*cos(t[1])+x[2]^2-1<=0;
subject to cons1:
	-x[1]^2-x[2]^2+1<=0;
subject to bound1:
	0 <= t[1] <= pi/2;

data;

param xinit =
	1	0.8
	2	0.6;
param tinit =
	1	0;

###################
# End of Problem  #
###################

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;

##################
# Solution found #
##################
printf "Original Solution\n";
printf "x=(%lf, %lf)\n", 1, 0;
printf "\nSolution found\n";
display x;
display fx;
